@@ -19,6 +19,7 @@
 
 #include <simgear_config.h>
 
+#include "vg/openvg.h"
 #include "Canvas.hxx"
 #include "CanvasEventManager.hxx"
 #include "CanvasEventVisitor.hxx"
@@ -29,6 +30,7 @@
 #include <simgear/misc/strutils.hxx>
 #include <simgear/scene/util/parse_color.hxx>
 #include <simgear/scene/util/RenderConstants.hxx>
+#include <simgear/scene/util/SGImageUtils.hxx>
 
 #include <osg/Camera>
 #include <osg/Geode>
@@ -39,6 +41,74 @@ namespace simgear
 {
 namespace canvas
 {
+  /**
+   * Camera Draw Callback for moving completed canvas images to subscribed listener.
+   * This is attached to the underlying ODGauge camera.
+   */
+  class CanvasImageCallback : public osg::Camera::DrawCallback {
+  public:
+    CanvasImageCallback(osg::Texture2D *texture) : _texture(texture) {
+      _previousFrameTick = osg::Timer::instance()->tick();
+    }
+
+    virtual void operator()(osg::RenderInfo& renderInfo) const {
+      if (!_texture) {
+        return;
+      }
+      osg::Timer_t n = osg::Timer::instance()->tick();
+      double dt = osg::Timer::instance()->delta_s(_previousFrameTick, n);
+      if (dt < _min_delta_tick) {
+        // Do nothing
+        return;
+      }
+      _previousFrameTick = n;
+
+      if (hasSubscribers()) {
+        // Transfer the FBO texture from the GPU to the CPU
+        osg::State *state = renderInfo.getState();
+        state->applyAttribute(_texture);
+        osg::ref_ptr<osg::Image> image = new osg::Image;
+        image->readImageFromCurrentTexture(renderInfo.getContextID(), false, GL_UNSIGNED_BYTE);
+        // Convert from RGBA to RGB because the subscriber might use JPEG
+        osg::ref_ptr<osg::Image> image_rgb = ImageUtils::convertToRGB8(image);
+        {
+          OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_lock);
+          while (!_subscribers.empty()) {
+            try {
+              CanvasImageReadyListener *subs = _subscribers.back();
+              if (subs) {
+                subs->imageReady(image_rgb);
+              } else {
+                SG_LOG(SG_GENERAL, SG_WARN, "CanvasImageCallback: Subscriber is null");
+              }
+            } catch (...) { }
+            _subscribers.pop_back();
+          }
+        }
+      }
+    }
+
+    void subscribe(CanvasImageReadyListener * subscriber) {
+      OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_lock);
+      _subscribers.push_back(subscriber);
+    }
+
+    void unsubscribe(CanvasImageReadyListener * subscriber) {
+      OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_lock);
+      _subscribers.remove(subscriber);
+    }
+
+    bool hasSubscribers() const {
+      OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_lock);
+      return !_subscribers.empty();
+    }
+  private:
+    osg::Texture2D *_texture;
+    mutable std::list<CanvasImageReadyListener*> _subscribers;
+    mutable OpenThreads::Mutex _lock;
+    mutable double _previousFrameTick;
+    double _min_delta_tick = 1.0 / 8.0;
+  };
 
   //----------------------------------------------------------------------------
   Canvas::CullCallback::CullCallback(const CanvasWeakPtr& canvas):
@@ -68,6 +138,8 @@ namespace canvas
     _status(node, "status"),
     _status_msg(node, "status-msg")
   {
+    // Looks like we need to propogate value changes upwards.
+    node->setAttribute(SGPropertyNode::VALUE_CHANGED_DOWN, true);
     _status = 0;
     setStatusFlags(MISSING_SIZE_X | MISSING_SIZE_Y);
 
@@ -227,36 +299,35 @@ namespace canvas
 
     if( _status & STATUS_DIRTY )
     {
-      // Retrieve reference here, to ensure the scene group is not deleted while
-      // creating the new texture and camera
-      osg::ref_ptr<osg::Group> root_scene_group = _root_group->getSceneGroup();
-
+      // STATUS_DIRTY can only happen when size has changed
       _texture.setSize(_size_x, _size_y);
 
-      if( !_texture.serviceable() )
-      {
+      // Create the ODGauge if it wasn't initialized yet
+      if (!_texture.serviceable()) {
         _texture.useImageCoords(true);
         _texture.useStencil(true);
         _texture.allocRT(/*_camera_callback*/);
+
+        osg::Camera* camera = _texture.getCamera();
+
+        // XXX: Render order is determined by the Canvas index
+        int order = _node->getIndex();
+        camera->setRenderOrder(osg::Camera::PRE_RENDER, order);
+
+        // Set the clear color to the background color
+        osg::Vec4 clear_color(0.0f, 0.0f, 0.0f, 1.0f);
+        parseColor(_node->getStringValue("background"), clear_color);
+        camera->setClearColor(clear_color);
+
+        // Add the scene group as children to the RTT camera.
+        // NOTE: The scene group we are retrieving is a weak pointer. The only
+        // ref_ptr holding the scene group is the child we've just added. If the
+        // camera is destroyed, the scene group will also be destroyed.
+        camera->addChild(_root_group->getSceneGroup());
+
+        // Add the screenshot draw callback
+        camera->setFinalDrawCallback(new CanvasImageCallback(_texture.getTexture()));
       }
-      else
-      {
-        // Resizing causes a new texture to be created so we need to reapply all
-        // existing placements
-        reloadPlacements();
-      }
-
-      osg::Camera* camera = _texture.getCamera();
-
-      // TODO Allow custom render order? For now just keep in order with
-      //      property tree.
-      camera->setRenderOrder(osg::Camera::PRE_RENDER, _node->getIndex());
-
-      osg::Vec4 clear_color(0.0f, 0.0f, 0.0f , 1.0f);
-      parseColor(_node->getStringValue("background"), clear_color);
-      camera->setClearColor(clear_color);
-
-      camera->addChild(root_scene_group);
 
       if( _texture.serviceable() )
       {
@@ -316,6 +387,13 @@ namespace canvas
       _sampling_dirty = false;
       _render_dirty = true;
     }
+	
+    if( _anisotropy_dirty )
+    {
+      _texture.setMaxAnisotropy( _node->getFloatValue("anisotropy") );
+      _anisotropy_dirty = false;
+      _render_dirty = true;
+    }
 
     while( !_dirty_placements.empty() )
     {
@@ -347,6 +425,37 @@ namespace canvas
     }
   }
 
+  int Canvas::subscribe(CanvasImageReadyListener * subscriber)
+  {
+    const std::string &canvasname = _node->getStringValue("name");
+    SG_LOG(SG_GENERAL, SG_DEBUG, "Canvas: subscribe to canvas '" << canvasname << "'");
+    if (_texture.serviceable()) {
+      osg::Camera* camera = _texture.getCamera();
+      auto cb = dynamic_cast<CanvasImageCallback*>(camera->getFinalDrawCallback());
+      if (cb) {
+        SG_LOG(SG_GENERAL, SG_DEBUG, "Canvas: adding subscriber to camera draw callback");
+        cb->subscribe(subscriber);
+        enableRendering(true);
+        return 1;
+      }
+    }
+    return 0;
+  }
+
+  void Canvas::unsubscribe(CanvasImageReadyListener * subscriber)
+  {
+    const std::string &canvasname = _node->getStringValue("name");
+    SG_LOG(SG_GENERAL, SG_DEBUG, "Canvas: unsubscribe from canvas '" << canvasname << "'");
+    if (_texture.serviceable()) {
+      osg::Camera* camera = _texture.getCamera();
+      auto cb = dynamic_cast<CanvasImageCallback*>(camera->getFinalDrawCallback());
+      if (cb) {
+        SG_LOG(SG_GENERAL, SG_DEBUG, "Canvas: removing subscriber from camera draw callback");
+        cb->unsubscribe(subscriber);
+      }
+    }
+  }
+
   //----------------------------------------------------------------------------
   bool Canvas::addEventListener( const std::string& type,
                                  const EventListener& cb )
@@ -506,10 +615,16 @@ namespace canvas
     if( parent != _node )
       return;
 
-    if( child->getNameString() == "placement" )
-      _placements[ child->getIndex() ].clear();
-    else if( _root_group.get() )
-      static_cast<Element*>(_root_group.get())->childRemoved(parent, child);
+    if (child->getNameString() == "placement") {
+        const size_t index = child->getIndex();
+        if (index < _placements.size()) {
+            _placements[child->getIndex()].clear();
+        } else {
+            SG_LOG(SG_GENERAL, SG_DEV_ALERT, "Cavas::childRemoved: no placement at index:" << child->getPath());
+        }
+    } else if (_root_group.get()) {
+        static_cast<Element*>(_root_group.get())->childRemoved(parent, child);
+    }
   }
 
   //----------------------------------------------------------------------------
@@ -572,6 +687,10 @@ namespace canvas
       {
         _sampling_dirty = true;
       }
+      else if( name == "anisotropy" )
+      {
+        _anisotropy_dirty = true;
+      }
       else if( name == "additive-blend" )
       {
         _texture.useAdditiveBlend( node->getBoolValue() );
@@ -612,6 +731,12 @@ namespace canvas
       _root_group->valueChanged(node);
   }
 
+  //----------------------------------------------------------------------------
+  osg::Camera* Canvas::getCamera() const
+  {
+    return _texture.getCamera();
+  }
+
   //----------------------------------------------------------------------------
   osg::Texture2D* Canvas::getTexture() const
   {
@@ -707,6 +832,13 @@ namespace canvas
       _status_msg = "Ok";
   }
 
+  //----------------------------------------------------------------------------
+  ElementPtr Canvas::getFocusedElement() const
+  {
+    return _focus_element.lock();
+  }
+
+
   //----------------------------------------------------------------------------
   Canvas::PlacementFactoryMap Canvas::_placement_factories;
   SystemAdapterPtr Canvas::_system_adapter;
