@@ -1,4 +1,5 @@
-// Window for placing a Canvas onto it (for dialogs, menus, etc.)
+///@file
+/// Window for placing a Canvas onto it (for dialogs, menus, etc.)
 //
 // Copyright (C) 2012  Thomas Geymayer <tomgey@gmail.com>
 //
@@ -22,6 +23,7 @@
 #include "CanvasWindow.hxx"
 
 #include <simgear/canvas/Canvas.hxx>
+#include <simgear/canvas/events/CanvasKeyBinding.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/scene/util/OsgMath.hxx>
 
@@ -105,6 +107,23 @@ namespace canvas
     return SGVec2<float>( m(3, 0), m(3, 1) );
   }
 
+  //----------------------------------------------------------------------------
+
+  SGVec2<float> Window::toScreenPosition(const osg::Vec2f& pos) const
+  {
+    const auto windowPos = getPosition();
+    return windowPos + toSG(pos) + _contentOffset;
+  }
+
+  //----------------------------------------------------------------------------
+  FocusScope* Window::focusScope()
+  {
+    if (!_focus_scope) {
+      _focus_scope.reset(new FocusScope);
+    }
+    return _focus_scope.get();
+  }
+
   //----------------------------------------------------------------------------
   const SGRect<float> Window::getScreenRegion() const
   {
@@ -212,6 +231,27 @@ namespace canvas
       _resize_left = getRegion().l() + offset.x();
   }
 
+  //----------------------------------------------------------------------------
+  bool Window::handleEvent(const EventPtr& event)
+  {
+    if( auto mouse_event = dynamic_cast<MouseEvent*>(event.get()) )
+    {
+      mouse_event->local_pos =
+      mouse_event->client_pos =
+        mouse_event->screen_pos - toOsg(getPosition());
+    }
+
+    bool handled = Image::handleEvent(event);
+
+    // we can't use 'handled' value here because EventManager::propogateEvent
+    // always returns true, hmmmm.
+    if (_focus_scope && !event->isPropagationStopped()) {
+      handled = _focus_scope->handleEvent(event);
+    }
+
+    return handled;
+  }
+
   //----------------------------------------------------------------------------
   void Window::parseDecorationBorder(const std::string& str)
   {
@@ -240,7 +280,7 @@ namespace canvas
       setSrcCanvas(content);
       set<int>("size[0]", content_view.width());
       set<int>("size[1]", content_view.height());
-
+      _contentOffset = {};
       _image_content.reset();
       _image_shadow.reset();
       if( _canvas_decoration )
@@ -296,6 +336,8 @@ namespace canvas
     _canvas_decoration->setViewWidth( outer_width );
     _canvas_decoration->setViewHeight( outer_height );
 
+    _contentOffset = SGVec2f{border.l, border.t};
+
     set<int>("size[0]", outer_width - shad2);
     set<int>("size[1]", outer_height - shad2);
     set<int>("outset", shadow_radius);
