@@ -1,4 +1,5 @@
-// Interface for 2D Canvas element
+///@file
+/// Interface for 2D Canvas element
 //
 // Copyright (C) 2012  Thomas Geymayer <tomgey@gmail.com>
 //
@@ -29,7 +30,6 @@
 #include <osg/Drawable>
 #include <osg/Geode>
 #include <osg/StateAttribute>
-#include <osg/Version>
 
 #include <cassert>
 #include <cmath>
@@ -254,7 +254,7 @@ namespace canvas
     SG_LOG
     (
       SG_GENERAL,
-      SG_INFO,
+      SG_DEBUG,
       "addEventListener(" << _node->getPath() << ", " << type_str << ")"
     );
 
@@ -271,11 +271,19 @@ namespace canvas
   //----------------------------------------------------------------------------
   void Element::setFocus()
   {
-    CanvasPtr canvas = _canvas.lock();
-    if( canvas )
+    if( auto canvas = _canvas.lock() )
       canvas->setFocusElement(this);
   }
 
+  bool Element::hasFocus() const
+  {
+    if (auto canvas = _canvas.lock()) {
+      return canvas->getFocusedElement() == this;
+    }
+
+    return false;
+  }
+
   //----------------------------------------------------------------------------
   bool Element::accept(EventVisitor& visitor)
   {
@@ -365,13 +373,7 @@ namespace canvas
 
     // Drawables have a bounding box...
     if( _drawable )
-      return _drawable->
-#if OSG_VERSION_LESS_THAN(3,3,2)
-      getBound()
-#else
-      getBoundingBox()
-#endif
-      .contains(osg::Vec3f(local_pos, 0));
+      return _drawable->getBoundingBox().contains(osg::Vec3f(local_pos, 0));
     else if( _scene_group.valid() )
       // ... for other elements, i.e. groups only a bounding sphere is available
       return _scene_group->getBound().contains(osg::Vec3f(parent_pos, 0));
@@ -407,6 +409,8 @@ namespace canvas
       // TODO log warning?
       return pos;
 
+    // this looks heavy, but both updateMatrix and getInverseMatrix
+    // use dirty flags to cache the matrices, so it's actually efficient
     updateMatrix();
     const osg::Matrix& m = _scene_group->getInverseMatrix();
     return osg::Vec2f
@@ -416,6 +420,51 @@ namespace canvas
     );
   }
 
+
+  //----------------------------------------------------------------------------
+  osg::Vec2f Element::posFromLocal(const osg::Vec2f& pos) const
+  {
+    if (!_scene_group) {
+      // TODO log warning?
+      return pos;
+    }
+
+    updateMatrix();
+    const osg::Matrix& m = _scene_group->getMatrix();
+    return osg::Vec2f(
+        m(0, 0) * pos[0] + m(1, 0) * pos[1] + m(3, 0),
+        m(0, 1) * pos[0] + m(1, 1) * pos[1] + m(3, 1));
+  }
+
+
+  //----------------------------------------------------------------------------
+
+  osg::Vec2f Element::canvasToLocal(const osg::Vec2f& pos) const
+  {
+    ElementPtr parent = getParent();
+    if (parent) {
+      return posToLocal(parent->canvasToLocal(pos));
+    }
+
+    // no parent, we are the root group presumably. Maybe this is overkill
+    // because we can assume no tranformation on the root group but I'm not
+    // certain of that.
+    return posToLocal(pos);
+  }
+
+  //----------------------------------------------------------------------------
+
+  osg::Vec2f Element::localToCanvas(const osg::Vec2f& pos) const
+  {
+    const auto ppos = posFromLocal(pos);
+    ElementPtr parent = getParent();
+    if (parent) {
+      return parent->localToCanvas(ppos);
+    }
+
+    return ppos;
+  }
+
   //----------------------------------------------------------------------------
   void Element::childAdded(SGPropertyNode* parent, SGPropertyNode* child)
   {
@@ -643,11 +692,7 @@ namespace canvas
   osg::BoundingBox Element::getBoundingBox() const
   {
     if( _drawable )
-#if OSG_VERSION_LESS_THAN(3,3,2)
-      return _drawable->getBound();
-#else
       return _drawable->getBoundingBox();
-#endif
 
     osg::BoundingBox bb;
 
@@ -670,12 +715,7 @@ namespace canvas
       return osg::BoundingBox();
 
     osg::BoundingBox transformed;
-    const osg::BoundingBox& bb =
-#if OSG_VERSION_LESS_THAN(3,3,2)
-      _drawable->getBound();
-#else
-      _drawable->getBoundingBox();
-#endif
+    const osg::BoundingBox& bb = _drawable->getBoundingBox();
 
     for(int i = 0; i < 4; ++i)
       transformed.expandBy( bb.corner(i) * m );
@@ -753,7 +793,7 @@ namespace canvas
       return true;
 
     // Parent values do not override if element has own value
-    return isStyleEmpty( _node->getChild(child->getName()) );
+    return isStyleEmpty( _node->getChild(child->getNameString()) );
   }
 
   //----------------------------------------------------------------------------
