@@ -1,4 +1,5 @@
-// A group of 2D Canvas elements
+///@file
+/// A group of 2D Canvas elements
 //
 // Copyright (C) 2012  Thomas Geymayer <tomgey@gmail.com>
 //
@@ -25,6 +26,7 @@
 #include "CanvasText.hxx"
 
 #include <simgear/canvas/CanvasEventVisitor.hxx>
+#include <simgear/canvas/events/CanvasKeyBinding.hxx>
 #include <simgear/canvas/events/MouseEvent.hxx>
 
 namespace simgear
@@ -119,7 +121,7 @@ namespace canvas
         SG_GENERAL,
         SG_WARN,
         "Group::getOrCreateChild: type missmatch! "
-        "('" << type << "' != '" << child->getProps()->getName() << "', "
+        "('" << type << "' != '" << child->getProps()->getNameString() << "', "
         "id = '" << id << "')"
       );
 
@@ -168,6 +170,7 @@ namespace canvas
     if( !_scene_group.valid() )
       return warnSceneGroupExpired("clearEventListener");
 
+    // TODO should this be recursive?
     for(size_t i = 0; i < _scene_group->getNumChildren(); ++i)
       getChildByIndex(i)->clearEventListener();
   }
@@ -389,7 +392,7 @@ namespace canvas
     return ud ? ud->element : ElementPtr();
   }
 
-  //----------------------------------------------------------------------------
+
   ElementPtr Group::findChild( const SGPropertyNode* node,
                                const std::string& id ) const
   {
@@ -418,5 +421,39 @@ namespace canvas
     return {};
   }
 
+  //----------------------------------------------------------------------------
+  FocusScope* Group::getOrCreateFocusScope()
+  {
+    if (_focus_scope) {
+      return _focus_scope.get();
+    }
+
+    _focus_scope.reset(new FocusScope);
+    return _focus_scope.get();
+  }
+
+  //----------------------------------------------------------------------------
+  FocusScope* Group::getFocusScope() const
+  {
+    return _focus_scope.get();
+  }
+
+  //----------------------------------------------------------------------------
+  bool Group::handleEvent(const EventPtr& event)
+  {
+    // can't rely on return value from handleEvent,
+    // so we don't look at it here.
+    bool handled = Element::handleEvent(event);
+
+    // the event manager which calls us in propagateEvent
+    // will check the propogation_stopped flag
+    if (!event->isPropagationStopped() && _focus_scope) {
+      return _focus_scope->handleEvent(event);
+    }
+
+    return handled;
+  }
+
+
 } // namespace canvas
 } // namespace simgear
