@@ -97,17 +97,26 @@ namespace canvas
           || (!geo_node->isDirty() && !_projection_dirty) )
         continue;
 
-      GeoCoord lat = parseGeoCoord(geo_node->getLat());
-      if( lat.type != GeoCoord::LATITUDE )
-        continue;
-
-      GeoCoord lon = parseGeoCoord(geo_node->getLon());
-      if( lon.type != GeoCoord::LONGITUDE )
-        continue;
-
-      Projection::ScreenPosition pos =
-        _projection->worldToScreen(lat.value, lon.value);
-
+      double latD = -9999.0, lonD = -9999.0;
+      if (geo_node->isDirty()) {
+        GeoCoord lat = parseGeoCoord(geo_node->getLat());
+        if( lat.type != GeoCoord::LATITUDE )
+          continue;
+        
+        GeoCoord lon = parseGeoCoord(geo_node->getLon());
+        if( lon.type != GeoCoord::LONGITUDE )
+          continue;
+        
+        // save the parsed values so we can re-use them if only projection
+        // is changed (very common case for moving vehicle)
+        latD = lat.value;
+        lonD = lon.value;
+        geo_node->setCachedLatLon(std::make_pair(latD, lonD));
+      } else {
+        std::tie(latD, lonD) = geo_node->getCachedLatLon();
+      }
+      
+      Projection::ScreenPosition pos = _projection->worldToScreen(latD, lonD);
       geo_node->setScreenPos(pos.x, pos.y);
 
 //      geo_node->print();
@@ -133,7 +142,7 @@ namespace canvas
     if( strutils::ends_with(child->getNameString(), GEO) )
       // TODO remove from other node
       _geo_nodes.erase(child);
-    else if( parent != _node && child->getName() == HDG )
+    else if( parent != _node && child->getNameString() == HDG )
     {
       _hdg_nodes.erase(child);
 
@@ -301,9 +310,9 @@ namespace canvas
     char* end;
     coord.value = strtod(&val[1], &end);
 
-    if( end != &val[val.length()] )
+    if( end != &val[val.length()] )       // intentional use of... out-of-bounds of array
     {
-      coord.type = GeoCoord::INVALID;
+      coord.type = GeoCoord::INVALID;     // the whole 'val' string was not converted; treat this as an error
       return coord;
     }
 
