@@ -24,7 +24,6 @@
 #include <simgear/misc/strutils.hxx>
 
 #include <osg/Drawable>
-#include <osg/Version>
 
 #include <vg/openvg.h>
 #include <cassert>
@@ -394,6 +393,8 @@ namespace canvas
 
         osg::State* state = renderInfo.getState();
         assert(state);
+        osg::GLExtensions* extensions = state->get<osg::GLExtensions>();
+        assert(extensions);
 
         state->setActiveTextureUnit(0);
         state->setClientActiveTextureUnit(0);
@@ -401,9 +402,14 @@ namespace canvas
 
         bool was_blend_enabled = state->getLastAppliedMode(GL_BLEND);
         bool was_stencil_enabled = state->getLastAppliedMode(GL_STENCIL_TEST);
+        bool was_scissor_enabled = state->getLastAppliedMode(GL_SCISSOR_TEST);
         osg::StateAttribute const* blend_func =
           state->getLastAppliedAttribute(osg::StateAttribute::BLENDFUNC);
 
+        // Set the model view projection matrix
+        osg::Matrixf mvp = state->getModelViewMatrix() * state->getProjectionMatrix();
+        vgSetModelViewProjectionMatSH(mvp.ptr());
+
         // Setup paint
         if( _mode & VG_STROKE_PATH )
         {
@@ -461,7 +467,9 @@ namespace canvas
         //                            better with OpenSceneGraph)
         if( was_blend_enabled )   glEnable(GL_BLEND);
         if( was_stencil_enabled ) glEnable(GL_STENCIL_TEST);
+        if( was_scissor_enabled ) glEnable(GL_SCISSOR_TEST);
         if( blend_func )          blend_func->apply(*state);
+        extensions->glUseProgram(0);
       }
 
       osg::BoundingBox getTransformedBounds(const osg::Matrix& mat) const
@@ -559,12 +567,8 @@ namespace canvas
       /**
        * Compute the bounding box
        */
-      osg::BoundingBox
-#if OSG_VERSION_LESS_THAN(3,3,2)
-      computeBound()
-#else
-      computeBoundingBox()
-#endif
+      osg::BoundingBox computeBoundingBox()
+
       const override
       {
         if( _path == VG_INVALID_HANDLE || (_attributes_dirty & PATH) )
