@@ -1,4 +1,5 @@
-// A text on the Canvas
+///@file
+/// A text on the Canvas
 //
 // Copyright (C) 2012  Thomas Geymayer <tomgey@gmail.com>
 //
@@ -17,6 +18,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 
 #include <simgear_config.h>
+
 #include "CanvasText.hxx"
 #include <simgear/canvas/Canvas.hxx>
 #include <simgear/canvas/CanvasSystemAdapter.hxx>
@@ -51,27 +53,23 @@ namespace canvas
       TextLine lineAt(size_t i) const;
 
       /// Get nearest line to given y-coordinate
+#if OSG_VERSION_LESS_THAN(3,6,5)
       TextLine nearestLine(float pos_y) const;
-
-
       SGVec2i sizeForWidth(int w) const;
-
-      virtual osg::BoundingBox
-#if OSG_VERSION_LESS_THAN(3,3,2)
-      computeBound()
 #else
-      computeBoundingBox()
+      TextLine nearestLine(float pos_y);
+      SGVec2i sizeForWidth(int w);
 #endif
-      const;
+
+      osg::BoundingBox computeBoundingBox() const override;
 
     protected:
-
       friend class TextLine;
 
       canvas::Text *_text_element;
 
-      virtual void computePositions(unsigned int contextID) const;
-  };
+     void computePositionsImplementation() override;
+};
 
   class TextLine
   {
@@ -84,7 +82,7 @@ namespace canvas
       bool empty() const;
 
       osg::Vec2 cursorPos(size_t i) const;
-      osg::Vec2 nearestCursor(float x) const;
+      osg::Vec2i nearestCursor(float x) const;
 
     protected:
       typedef Text::TextOSG::GlyphQuads GlyphQuads;
@@ -121,17 +119,25 @@ namespace canvas
 
     _quads = &text->_textureGlyphQuadMap.begin()->second;
 
-    GlyphQuads::LineNumbers const& line_numbers = _quads->_lineNumbers;
-    GlyphQuads::LineNumbers::const_iterator begin_it =
-      std::lower_bound(line_numbers.begin(), line_numbers.end(), _line);
+    auto refCoords = _text->getCoords();
+    osgText::TextBase::Coords::element_type &coords = *refCoords.get();
+    const auto lineHeight = text->getCharacterHeight() + _text->getLineSpacing();
 
-    if( begin_it == line_numbers.end() || *begin_it != _line )
-      // empty line or past last line
-      return;
-
-    _begin = begin_it - line_numbers.begin();
-    _end = std::upper_bound(begin_it, line_numbers.end(), _line)
-         - line_numbers.begin();
+    bool foundBegin = false;
+    for (size_t i = 0; i <= coords.size(); i += 4) {
+      const auto lineIndex = static_cast<size_t>(floorf(coords[i].y() / lineHeight));
+      if (lineIndex == _line) {
+        if (!foundBegin) {
+          _begin = _end = i / 4;
+          foundBegin = true;
+        } else {
+          _end = i / 4;
+        }
+      } else if (foundBegin) {
+        // next line, we're done
+          break;
+      }
+    }
   }
 
   //----------------------------------------------------------------------------
@@ -149,62 +155,53 @@ namespace canvas
   //----------------------------------------------------------------------------
   osg::Vec2 TextLine::cursorPos(size_t i) const
   {
-    if( !_quads )
-      return osg::Vec2(0, 0);
-
     if( i > size() )
       // Position after last character if out of range (TODO better exception?)
       i = size();
 
     osg::Vec2 pos(0, _text->_offset.y() + _line * _text->lineHeight());
 
-    if( empty() )
+    if( empty() ) {
       return pos;
-#if OSG_VERSION_LESS_THAN(3,3,5)
-    GlyphQuads::Coords2 const& coords = _quads->_coords;
-#else
-    GlyphQuads::Coords2 refCoords = _quads->_coords;
-    GlyphQuads::Coords2::element_type &coords = *refCoords.get();
-#endif
-    size_t global_i = _begin + i;
-
-    if( global_i == _begin )
-      // before first character of line
-      pos.x() = coords[_begin * 4].x();
-    else if( global_i == _end )
-      // After Last character of line
-      pos.x() = coords[(_end - 1) * 4 + 2].x();
-    else
-    {
-      float prev_l = coords[(global_i - 1) * 4].x(),
-            prev_r = coords[(global_i - 1) * 4 + 2].x(),
-            cur_l = coords[global_i * 4].x();
-
-      if( prev_l == prev_r )
-        // If previous character width is zero set to begin of next character
-        // (Happens eg. with spaces)
-        pos.x() = cur_l;
-      else
-        // position at center between characters
-        pos.x() = 0.5 * (prev_r + cur_l);
     }
 
+      osgText::TextBase::Coords refCoords = _text->getCoords();
+      osgText::TextBase::Coords::element_type &coords = *refCoords.get();
+
+      size_t global_i = _begin + i;
+
+      if (global_i == _begin) {
+          // before first character of line
+          pos.x() = coords[_begin * 4].x();
+      } else if (global_i == _end) {
+          // After Last character of line
+          pos.x() = coords[(_end - 1) * 4 + 2].x();
+    } else {
+          float prev_l = coords[(global_i - 1) * 4].x(),
+              prev_r = coords[(global_i - 1) * 4 + 2].x(),
+              cur_l = coords[global_i * 4].x();
+
+          if (prev_l == prev_r)
+              // If previous character width is zero set to begin of next character
+              // (Happens eg. with spaces)
+              pos.x() = cur_l;
+          else
+              // position at center between characters
+              pos.x() = 0.5 * (prev_r + cur_l);
+      }
     return pos;
   }
 
   //----------------------------------------------------------------------------
-  osg::Vec2 TextLine::nearestCursor(float x) const
+  osg::Vec2i TextLine::nearestCursor(float x) const
   {
-    if( empty() )
-      return cursorPos(0);
+    if (empty())
+      return {static_cast<int>(_line), 0};
+
+    osgText::TextBase::Coords refCoords = _text->getCoords();
+    osgText::TextBase::Coords::element_type &coords = *refCoords.get();
 
     GlyphQuads::Glyphs const& glyphs = _quads->_glyphs;
-    #if OSG_VERSION_LESS_THAN(3,3,5)
-    GlyphQuads::Coords2 const& coords = _quads->_coords;
-#else
-    GlyphQuads::Coords2 refCoords = _quads->_coords;
-    GlyphQuads::Coords2::element_type &coords = *refCoords.get();
-#endif
 
     float const HIT_FRACTION = 0.6;
     float const character_width = _text->getCharacterHeight()
@@ -223,7 +220,8 @@ namespace canvas
         break;
     }
 
-    return cursorPos(i - _begin);
+    return {static_cast<int>(_line), 
+      static_cast<int>(i - _begin)};
   }
 
   //----------------------------------------------------------------------------
@@ -302,9 +300,16 @@ namespace canvas
   }
 
   //----------------------------------------------------------------------------
+#if OSG_VERSION_LESS_THAN(3,6,5)
   TextLine Text::TextOSG::nearestLine(float pos_y) const
   {
     osgText::Font const* font = getActiveFont();
+#else
+  TextLine Text::TextOSG::nearestLine(float pos_y)
+  {
+    auto font = getActiveFont();
+#endif
+
     if( !font || lineCount() <= 0 )
       return TextLine(0, this);
 
@@ -326,12 +331,21 @@ namespace canvas
   // simplified version of osgText::Text::computeGlyphRepresentation() to
   // just calculate the size for a given weight. Glpyh calculations/creating
   // is not necessary for this...
+#if OSG_VERSION_LESS_THAN(3,6,5)
   SGVec2i Text::TextOSG::sizeForWidth(int w) const
+#else
+  SGVec2i Text::TextOSG::sizeForWidth(int w)
+#endif
   {
     if( _text.empty() )
       return SGVec2i(0, 0);
 
+#if OSG_VERSION_LESS_THAN(3,6,5)
     osgText::Font* activefont = const_cast<osgText::Font*>(getActiveFont());
+#else
+    auto activefont = getActiveFont();
+#endif
+
     if( !activefont )
       return SGVec2i(-1, -1);
 
@@ -611,104 +625,19 @@ namespace canvas
   }
 
   //----------------------------------------------------------------------------
-  osg::BoundingBox
-#if OSG_VERSION_LESS_THAN(3,3,2)
-  Text::TextOSG::computeBound()
-#else
-  Text::TextOSG::computeBoundingBox()
-#endif
-  const
+  osg::BoundingBox Text::TextOSG::computeBoundingBox() const
   {
-    osg::BoundingBox bb =
-#if OSG_VERSION_LESS_THAN(3,3,2)
-      osgText::Text::computeBound();
-#else
-      osgText::Text::computeBoundingBox();
-#endif
-
-#if OSG_VERSION_LESS_THAN(3,1,0)
-    if( bb.valid() )
-    {
-      // TODO bounding box still doesn't seem always right (eg. with center
-      //      horizontal alignment not completely accurate)
-      bb._min.y() += _offset.y();
-      bb._max.y() += _offset.y();
-    }
-#endif
-
+    osg::BoundingBox bb = osgText::Text::computeBoundingBox();
     return bb;
   }
 
-  //----------------------------------------------------------------------------
-  void Text::TextOSG::computePositions(unsigned int contextID) const
+  void Text::TextOSG::computePositionsImplementation()
   {
-    if( _textureGlyphQuadMap.empty() || _layout == VERTICAL )
-      return osgText::Text::computePositions(contextID);
-
-    // TODO check when it can be larger
-    assert( _textureGlyphQuadMap.size() == 1 );
-
-    const GlyphQuads& quads = _textureGlyphQuadMap.begin()->second;
-    const GlyphQuads::Glyphs& glyphs = quads._glyphs;
-#if OSG_VERSION_LESS_THAN(3,3,5)
-    GlyphQuads::Coords2 const& coords = quads._coords;
-#else
-    GlyphQuads::Coords2 refCoords = quads._coords;
-    GlyphQuads::Coords2::element_type &coords = *refCoords.get();
-#endif
-
-    const GlyphQuads::LineNumbers& line_numbers = quads._lineNumbers;
-
-    float wr = _characterHeight / getCharacterAspectRatio();
-
-    size_t cur_line = static_cast<size_t>(-1);
-    for(size_t i = 0; i < glyphs.size(); ++i)
-    {
-      // Check horizontal offsets
-
-      bool first_char = cur_line != line_numbers[i];
-      cur_line = line_numbers[i];
-
-      bool last_char = (i + 1 == glyphs.size())
-                    || (cur_line != line_numbers[i + 1]);
-
-      if( first_char || last_char )
-      {
-        // From osg/src/osgText/Text.cpp:
-        //
-        // osg::Vec2 upLeft = local+osg::Vec2(0.0f-fHorizQuadMargin, ...);
-        // osg::Vec2 lowLeft = local+osg::Vec2(0.0f-fHorizQuadMargin, ...);
-        // osg::Vec2 lowRight = local+osg::Vec2(width+fHorizQuadMargin, ...);
-        // osg::Vec2 upRight = local+osg::Vec2(width+fHorizQuadMargin, ...);
-
-        float left = coords[i * 4].x(),
-              right = coords[i * 4 + 2].x(),
-              width = glyphs[i]->getWidth() * wr;
-
-        // (local + width + fHoriz) - (local - fHoriz) = width + 2*fHoriz | -width
-        float margin = 0.5f * (right - left - width),
-              cursor_x = left + margin
-                       - glyphs[i]->getHorizontalBearing().x() * wr;
-
-        if( first_char )
-        {
-          if( cur_line == 0 || cursor_x < _textBB._min.x() )
-            _textBB._min.x() = cursor_x;
-        }
-
-        if( last_char )
-        {
-          float cursor_w = cursor_x + glyphs[i]->getHorizontalAdvance() * wr;
-
-          if( cur_line == 0 || cursor_w > _textBB._max.x() )
-            _textBB._max.x() = cursor_w;
-        }
-      }
-    }
-
-    return osgText::Text::computePositions(contextID);
+    TextBase::computePositionsImplementation();
   }
 
+ //----------------------------------------------------------------------------
+
   //----------------------------------------------------------------------------
   const std::string Text::TYPE_NAME = "text";
 
@@ -775,21 +704,20 @@ namespace canvas
   }
 
   //----------------------------------------------------------------------------
-  void Text::setText(const char* text)
+  void Text::setText(const std::string &text)
   {
     _text->setText(text, osgText::String::ENCODING_UTF8);
   }
 
   //----------------------------------------------------------------------------
-  void Text::setFont(const char* name)
+  void Text::setFont(const std::string& name)
   {
     _text->setFont( Canvas::getSystemAdapter()->getFont(name) );
   }
 
   //----------------------------------------------------------------------------
-  void Text::setAlignment(const char* align)
+  void Text::setAlignment(const std::string& align_string)
   {
-    const std::string align_string(align);
     if( 0 ) return;
 #define ENUM_MAPPING(enum_val, string_val) \
     else if( align_string == string_val )\
@@ -851,7 +779,7 @@ namespace canvas
   }
 
   //----------------------------------------------------------------------------
-  osg::Vec2 Text::getNearestCursor(const osg::Vec2& pos) const
+  osg::Vec2i Text::getNearestCursor(const osg::Vec2& pos) const
   {
     return _text->nearestLine(pos.y()).nearestCursor(pos.x());
   }
