@@ -16,6 +16,7 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 
+#include <simgear_config.h>
 #include "LayoutItem.hxx"
 #include <simgear/canvas/Canvas.hxx>
 
@@ -85,12 +86,6 @@ namespace canvas
     invalidate();
   }
 
-  //----------------------------------------------------------------------------
-  LayoutItem::~LayoutItem()
-  {
-
-  }
-
   //----------------------------------------------------------------------------
   void LayoutItem::setContentsMargins(const Margins& margins)
   {
@@ -106,6 +101,24 @@ namespace canvas
     _margins.b = bottom;
   }
 
+  //----------------------------------------------------------------------------
+  SGVec2i LayoutItem::gridLocation() const
+  {
+      return _gridLocation;
+  }
+
+  //----------------------------------------------------------------------------
+  SGVec2i LayoutItem::gridSpan() const
+  {
+      return _span;
+  }
+
+  SGVec2i LayoutItem::gridEnd() const
+  {
+      return _gridLocation + _span + SGVec2i{-1, -1};
+  }
+
+
   //----------------------------------------------------------------------------
   void LayoutItem::setContentsMargin(int margin)
   {
@@ -300,6 +313,18 @@ namespace canvas
     return SGRecti(pos, pos + size);
   }
 
+  //----------------------------------------------------------------------------
+  void LayoutItem::setGridLocation(const SGVec2i& loc)
+  {
+      _gridLocation = loc;
+  }
+
+  //----------------------------------------------------------------------------
+  void LayoutItem::setGridSpan(const SGVec2i& span)
+  {
+      _span = span;
+  }
+
   //----------------------------------------------------------------------------
   void LayoutItem::setCanvas(const CanvasWeakPtr& canvas)
   {
