@@ -1,128 +1,192 @@
-// Stream based logging mechanism.
-//
-// Written by Bernie Bright, 1998
-//
-// Copyright (C) 1998  Bernie Bright - bbright@c031.aone.net.au
-//
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+// SPDX-FileName: logstream.cxx
+// SPDX-License-Identifier: LGPL-2.1-or-later
+// SPDX-FileComment: Stream based logging mechanism.
+// SPDX-FileCopyrightText: Copyright (C) 1998  Bernie Bright - bbright@c031.aone.net.au
+// SPDX-FileContributor: James Turner
 
 #include <simgear_config.h>
 
+#include "logdelta.hxx"
 #include "logstream.hxx"
 
+#include <cstring>
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <algorithm>
-
-#include <boost/foreach.hpp>
+#include <mutex>
 
 #include <simgear/sg_inlines.h>
-#include <simgear/threads/SGThread.hxx>
+#include <simgear/structure/exception.hxx>
 #include <simgear/threads/SGQueue.hxx>
-#include <simgear/threads/SGGuard.hxx>
+#include <simgear/threads/SGThread.hxx>
 
+#include "LogCallback.hxx"
 #include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/misc/strutils.hxx>
+#include <simgear/timing/timestamp.hxx>
 
 #if defined (SG_WINDOWS)
 // for AllocConsole, OutputDebugString
-    #include <windows.h>
-    #include <fcntl.h>
-    #include <io.h>
+#include <windows.h>
+#include <fcntl.h>
+#include <io.h>
 #endif
 
-
-
 //////////////////////////////////////////////////////////////////////////////
 
-namespace simgear
-{
+namespace {
 
-LogCallback::LogCallback(sgDebugClass c, sgDebugPriority p) :
-	m_class(c),
-	m_priority(p)
-{
-}
+static const std::vector<std::string> global_priorityNames = {
+    "UNKN",
+    "BULK",
+    "DBUG",
+    "INFO",
+    "WARN",
+    "ALRT",
+    "POPU",
+    "WARN",
+    "ALRT",
+    "INFO"};
 
-bool LogCallback::shouldLog(sgDebugClass c, sgDebugPriority p) const
-{
-	 return ((c & m_class) != 0 && p >= m_priority);
-}
+struct LogClassMapping {
+    const sgDebugClass c;
+    const std::string name;
+    const std::vector<std::string> aliases;
 
-void LogCallback::setLogLevels( sgDebugClass c, sgDebugPriority p )
-{
-	m_priority = p;
-	m_class = c;
-}
+    LogClassMapping(sgDebugClass cc, const std::string& n, const std::vector<std::string>& al = {}) : c(cc),
+                                                                                                      name(n),
+                                                                                                      aliases(al){};
+};
 
-const char* LogCallback::debugClassToString(sgDebugClass c)
+const std::initializer_list<LogClassMapping> log_class_mappings = {
+    LogClassMapping(SG_NONE, "none"),
+    LogClassMapping(SG_TERRAIN, "terrain"),
+    LogClassMapping(SG_ASTRO, "astro"),
+    LogClassMapping(SG_FLIGHT, "flight"),
+    LogClassMapping(SG_INPUT, "input"),
+    LogClassMapping(SG_GL, "gl", {"opengl"}),
+    LogClassMapping(SG_VIEW, "view"),
+    LogClassMapping(SG_COCKPIT, "cockpit"),
+    LogClassMapping(SG_GENERAL, "general"),
+    LogClassMapping(SG_MATH, "math"),
+    LogClassMapping(SG_EVENT, "event"),
+    LogClassMapping(SG_AIRCRAFT, "aircraft"),
+    LogClassMapping(SG_AUTOPILOT, "autopilot"),
+    LogClassMapping(SG_IO, "io"),
+    LogClassMapping(SG_CLIPPER, "clipper"),
+    LogClassMapping(SG_NETWORK, "network"),
+    LogClassMapping(SG_INSTR, "instrumentation", {"instruments"}),
+    LogClassMapping(SG_ATC, "atc"),
+    LogClassMapping(SG_NASAL, "nasal"),
+    LogClassMapping(SG_SYSTEMS, "systems"),
+    LogClassMapping(SG_AI, "ai"),
+    LogClassMapping(SG_ENVIRONMENT, "environment"),
+    LogClassMapping(SG_SOUND, "sound"),
+    LogClassMapping(SG_NAVAID, "navaid"),
+    LogClassMapping(SG_GUI, "gui"),
+    LogClassMapping(SG_TERRASYNC, "terrasync"),
+    LogClassMapping(SG_PARTICLES, "particles"),
+    LogClassMapping(SG_HEADLESS, "headless"),
+    LogClassMapping(SG_OSG, "osg", {"openscenegraph"}),
+    LogClassMapping(SG_UNDEFD, "")};
+
+} // namespace
+
+
+const std::string& debugClassToString(sgDebugClass c)
 {
-    switch (c) {
-        case SG_NONE:       return "none";
-        case SG_TERRAIN:    return "terrain";
-        case SG_ASTRO:      return "astro";
-        case SG_FLIGHT:     return "flight";
-        case SG_INPUT:      return "input";
-        case SG_GL:         return "opengl";
-        case SG_VIEW:       return "view";
-        case SG_COCKPIT:    return "cockpit";
-        case SG_GENERAL:    return "general";
-        case SG_MATH:       return "math";
-        case SG_EVENT:      return "event";
-        case SG_AIRCRAFT:   return "aircraft";
-        case SG_AUTOPILOT:  return "autopilot";
-        case SG_IO:         return "io";
-        case SG_CLIPPER:    return "clipper";
-        case SG_NETWORK:    return "network";
-        case SG_ATC:        return "atc";
-        case SG_NASAL:      return "nasal";
-        case SG_INSTR:      return "instruments";
-        case SG_SYSTEMS:    return "systems";
-        case SG_AI:         return "ai";
-        case SG_ENVIRONMENT:return "environment";
-        case SG_SOUND:      return "sound";
-        case SG_NAVAID:     return "navaid";
-        case SG_GUI:        return "gui";
-        case SG_TERRASYNC:  return "terrasync";
-        case SG_PARTICLES:  return "particles";
-        default:            return "unknown";
+    auto it = std::find_if(log_class_mappings.begin(), log_class_mappings.end(), [c](const LogClassMapping& lm) {
+        return lm.c == c;
+    });
+
+    // return 'none'
+    if (it == log_class_mappings.end()) {
+        return log_class_mappings.begin()->name;
     }
+
+    return it->name;
 }
 
-} // of namespace simgear
+const std::string& debugPriorityToString(sgDebugPriority p)
+{
+    if (static_cast<int>(p) >= global_priorityNames.size()) {
+        return global_priorityNames.at(0);
+    }
 
-//////////////////////////////////////////////////////////////////////////////
+    return global_priorityNames.at(static_cast<int>(p));
+}
+
+const sgDebugClass debugClassFromString(const std::string& s)
+{
+    using namespace simgear::strutils;
+
+    auto cleaned = lowercase(strip(s));
+    auto it = std::find_if(log_class_mappings.begin(), log_class_mappings.end(), [cleaned](const LogClassMapping& lm) {
+        if (lm.name == cleaned)
+            return true;
+
+        // check alises as well
+        auto it2 = std::find(lm.aliases.begin(), lm.aliases.end(), cleaned);
+        if (it2 != lm.aliases.end()) {
+            return true;
+        }
+
+        return false;
+    });
+
+    if (it == log_class_mappings.end()) {
+        throw sg_format_exception("Couldn't parse '" + s + "' as debug-class", s, "debugClassFromString", false);
+    }
+
+    return it->c;
+}
 
 class FileLogCallback : public simgear::LogCallback
 {
 public:
+    SGTimeStamp logTimer;
     FileLogCallback(const SGPath& aPath, sgDebugClass c, sgDebugPriority p) :
 	    simgear::LogCallback(c, p)
     {
         m_file.open(aPath, std::ios_base::out | std::ios_base::trunc);
+        logTimer.stamp();
     }
 
-    virtual void operator()(sgDebugClass c, sgDebugPriority p,
-        const char* file, int line, const std::string& message)
+    void operator()(sgDebugClass c, sgDebugPriority p,
+                    const char* file, int line, const std::string& message) override
     {
         if (!shouldLog(c, p)) return;
-        m_file << debugClassToString(c) << ":" << (int) p
-            << ":" << file << ":" << line << ":" << message << std::endl;
+
+
+//        fprintf(stderr, "%7.2f [%.8s]:%-10s %s\n", logTimer.elapsedMSec() / 1000.0, debugPriorityToString(p), debugClassToString(c), aMessage.c_str());
+        m_file
+            << std::fixed
+            << std::setprecision(2)
+            << std::setw(8)
+            << std::right
+            << (logTimer.elapsedMSec() / 1000.0)
+            << std::setw(8)
+            << std::left
+            << " [" + debugPriorityToString(p) + "]:"
+            << std::setw(10)
+            << std::left
+            << debugClassToString(c);
+        if (file) {
+            /* <line> can be -ve to indicate that m_fileLine was false, but we
+            want to show file:line information regardless of m_fileLine. */
+            m_file
+                << file
+                << ":"
+                << abs(line)
+                << ": "
+                ;
+            }
+        m_file
+            << message << std::endl;
+        //m_file << debugClassToString(c) << ":" << (int)p
+        //    << ":" << file << ":" << line << ":" << message << std::endl;
     }
 private:
     sg_ofstream m_file;
@@ -131,9 +195,12 @@ private:
 class StderrLogCallback : public simgear::LogCallback
 {
 public:
+    SGTimeStamp logTimer;
+
     StderrLogCallback(sgDebugClass c, sgDebugPriority p) :
 		simgear::LogCallback(c, p)
     {
+        logTimer.stamp();
     }
 
 #if defined (SG_WINDOWS)
@@ -143,12 +210,19 @@ public:
     }
 #endif
 
-    virtual void operator()(sgDebugClass c, sgDebugPriority p,
-        const char* file, int line, const std::string& aMessage)
+    void operator()(sgDebugClass c, sgDebugPriority p,
+                    const char* file, int line, const std::string& aMessage) override
     {
         if (!shouldLog(c, p)) return;
+        //fprintf(stderr, "%s\n", aMessage.c_str());
 
-        fprintf(stderr, "%s\n", aMessage.c_str());
+        if (file && line > 0) {
+            fprintf(stderr, "%8.2f %s:%i: [%.8s]:%-10s %s\n", logTimer.elapsedMSec() / 1000.0, file, line, debugPriorityToString(p).c_str(), debugClassToString(c).c_str(), aMessage.c_str());
+        }
+        else {
+            fprintf(stderr, "%8.2f [%.8s]:%-10s %s\n", logTimer.elapsedMSec() / 1000.0, debugPriorityToString(p).c_str(), debugClassToString(c).c_str(), aMessage.c_str());
+        }
+        //    file, line, aMessage.c_str());
         //fprintf(stderr, "%s:%d:%s:%d:%s\n", debugClassToString(c), p,
         //    file, line, aMessage.c_str());
         fflush(stderr);
@@ -166,8 +240,8 @@ public:
     {
     }
 
-    virtual void operator()(sgDebugClass c, sgDebugPriority p,
-        const char* file, int line, const std::string& aMessage)
+    void operator()(sgDebugClass c, sgDebugPriority p,
+                    const char* file, int line, const std::string& aMessage) override
     {
         if (!shouldLog(c, p)) return;
 
@@ -182,28 +256,6 @@ public:
 class logstream::LogStreamPrivate : public SGThread
 {
 private:
-    /**
-     * storage of a single log entry. This is used to pass log entries from
-     * the various threads to the logging thread, and also to store the startup
-     * entries
-     */
-    class LogEntry
-    {
-    public:
-        LogEntry(sgDebugClass c, sgDebugPriority p,
-            const char* f, int l, const std::string& msg) :
-            debugClass(c), debugPriority(p), file(f), line(l),
-                message(msg)
-        {
-        }
-
-        const sgDebugClass debugClass;
-        const sgDebugPriority debugPriority;
-        const char* file;
-        const int line;
-        const std::string message;
-    };
-
     /**
      * RAII object to pause the logging thread if it's running, and restart it.
      * used to safely make configuration changes.
@@ -242,10 +294,10 @@ public:
          *    window; stdout/stderr will not appear (except in logfiles as they do now)
          * 4) When started from the Console (with --console) open a new console window
          * 5) Ensure that IO redirection still works when started from the console
-         * 
+         *
          * Notes:
          * 1) fgfs needs to be a GUI subsystem app - which it already is
-         * 2) What can't be done is to make the cmd prompt run fgfs synchronously; 
+         * 2) What can't be done is to make the cmd prompt run fgfs synchronously;
          * this is only something that can be done via "start /wait fgfs".
          */
 
@@ -269,7 +321,7 @@ public:
 			}
         } else {
             /*
-            * Attempt to attach to the console process of the parent process; when launched from cmd.exe this should be the console, 
+            * Attempt to attach to the console process of the parent process; when launched from cmd.exe this should be the console,
             * when launched via the RUN menu explorer, or another GUI app that wasn't started from the console this will fail.
             * When it fails we will redirect to the NUL device. This is to ensure that we have valid streams.
             * Later on in the initialisation sequence the --console option will be processed and this will cause the requestConsole() to
@@ -299,7 +351,7 @@ public:
             if (!stdout_isNull){
                 if (!m_stdout_isRedirectedAlready)
                     freopen("conout$", "w", stdout);
-                else 
+                else
                     /*
                     * for already redirected streams we need to attach the stream to the OS handle that is open.
                     * - this comes from part of the answer http://stackoverflow.com/a/13841522
@@ -318,7 +370,7 @@ public:
             }
         }
         //http://stackoverflow.com/a/25927081
-        //Clear the error state for each of the C++ standard stream objects. 
+        //Clear the error state for each of the C++ standard stream objects.
         std::wcout.clear();
         std::cout.clear();
         std::wcerr.clear();
@@ -327,22 +379,38 @@ public:
 
         m_callbacks.push_back(new StderrLogCallback(m_logClass, m_logPriority));
         m_consoleCallbacks.push_back(m_callbacks.back());
-#if defined (SG_WINDOWS) && !defined(NDEBUG)
-		m_callbacks.push_back(new WinDebugLogCallback(m_logClass, m_logPriority));
-		m_consoleCallbacks.push_back(m_callbacks.back());
+        
+#if defined (SG_WINDOWS)
+        const char* winDebugEnv = ::getenv("SG_WINDEBUG");
+        const bool b = winDebugEnv ? simgear::strutils::to_bool(std::string{winDebugEnv}) : false;
+        if (b) {
+            m_callbacks.push_back(new WinDebugLogCallback(m_logClass, m_logPriority));
+            m_consoleCallbacks.push_back(m_callbacks.back());
+        }
 #endif
+        const char* fl = getenv("SG_LOG_FILE_LINE");
+        if (fl && !strcmp(fl, "1")) {
+            m_fileLine = true;
+        }
     }
 
     ~LogStreamPrivate()
     {
         removeCallbacks();
+
+        // house-keeping, avoid leak warnings if we exit before disabling
+        // startup logging
+        {
+            std::lock_guard<std::mutex> g(m_lock);
+            clearStartupEntriesLocked();
+        }
     }
 
-    SGMutex m_lock;
-    SGBlockingQueue<LogEntry> m_entries;
+    std::mutex m_lock;
+    SGBlockingQueue<simgear::LogEntry> m_entries;
 
     // log entries posted during startup
-    std::vector<LogEntry> m_startupEntries;
+    std::vector<simgear::LogEntry> m_startupEntries;
     bool m_startupLogging = false;
 
     typedef std::vector<simgear::LogCallback*> CallbackVec;
@@ -360,13 +428,16 @@ public:
     bool m_stdout_isRedirectedAlready = false;
 #endif
     bool m_developerMode = false;
+    bool m_fileLine = false;
 
     // test suite mode.
     bool m_testMode = false;
 
+    std::vector<std::string> _popupMessages;
+
     void startLog()
     {
-        SGGuard<SGMutex> g(m_lock);
+        std::lock_guard<std::mutex> g(m_lock);
         if (m_isRunning) return;
         m_isRunning = true;
         start();
@@ -378,44 +449,54 @@ public:
             return;
         }
 
-        m_startupLogging = on;
+        {
+            std::lock_guard<std::mutex> g(m_lock);
+            m_startupLogging = on;
+            clearStartupEntriesLocked();
+        }
+    }
+
+    void clearStartupEntriesLocked()
+    {
         m_startupEntries.clear();
     }
 
-    virtual void run()
+    void run() override
     {
         while (1) {
-            LogEntry entry(m_entries.pop());
+            simgear::LogEntry entry(m_entries.pop());
             // special marker entry detected, terminate the thread since we are
             // making a configuration change or quitting the app
-            if ((entry.debugClass == SG_NONE) && !strcmp(entry.file, "done")) {
+            if ((entry.debugClass == SG_NONE) && entry.file && !strcmp(entry.file, "done")) {
                 return;
             }
-
-            if (m_startupLogging) {
-                // save to the startup list for not-yet-added callbacks to
-                // pull down on startup
-                m_startupEntries.push_back(entry);
+            {
+                std::lock_guard<std::mutex> g(m_lock);
+                if (m_startupLogging) {
+                    // save to the startup list for not-yet-added callbacks to
+                    // pull down on startup
+                    m_startupEntries.push_back(entry);
+                }
             }
-
             // submit to each installed callback in turn
             for (simgear::LogCallback* cb : m_callbacks) {
-                (*cb)(entry.debugClass, entry.debugPriority,
-                    entry.file, entry.line, entry.message);
+                cb->processEntry(entry);
             }
         } // of main thread loop
     }
 
     bool stop()
     {
-        SGGuard<SGMutex> g(m_lock);
-        if (!m_isRunning) {
-            return false;
-        }
+        {
+            std::lock_guard<std::mutex> g(m_lock);
+            if (!m_isRunning) {
+                return false;
+            }
 
-        // log a special marker value, which will cause the thread to wakeup,
-        // and then exit
-        log(SG_NONE, SG_ALERT, "done", -1, "");
+            // log a special marker value, which will cause the thread to wakeup,
+            // and then exit
+            log(SG_NONE, SG_ALERT, "done", -1, "", "", false);
+        }
         join();
 
         m_isRunning = false;
@@ -429,9 +510,8 @@ public:
 
         // we clear startup entries not using this, so always safe to run
         // this code, container will simply be empty
-        for (auto entry : m_startupEntries) {
-            (*cb)(entry.debugClass, entry.debugPriority,
-               entry.file, entry.line, entry.message);
+        for (const auto& entry : m_startupEntries) {
+            cb->processEntry(entry);
         }
     }
 
@@ -459,37 +539,52 @@ public:
         PauseThread pause(this);
         m_logPriority = p;
         m_logClass = c;
-		BOOST_FOREACH(simgear::LogCallback* cb, m_consoleCallbacks) {
-	        cb->setLogLevels(c, p);
-		}
+        for (auto cb : m_consoleCallbacks) {
+            cb->setLogLevels(c, p);
+        }
     }
 
-    bool would_log( sgDebugClass c, sgDebugPriority p ) const
+    bool would_log( sgDebugClass c, sgDebugPriority p,
+            const char* file, int line, const char* function,
+            bool freeFilename ) const
     {
         // Testing mode, so always log.
         if (m_testMode) return true;
 
-        p = translatePriority(p);
+        // SG_OSG (OSG notify) - will always be displayed regardless of FG log settings as OSG log level is configured
+        // separately and thus it makes more sense to allow these message through.
+        if (static_cast<unsigned>(p) == static_cast<unsigned>(SG_OSG)) return true;
+
+        p = translatePriority(p, file, line, function, freeFilename);
         if (p >= SG_INFO) return true;
         return ((c & m_logClass) != 0 && p >= m_logPriority);
     }
 
     void log( sgDebugClass c, sgDebugPriority p,
-            const char* fileName, int line, const std::string& msg)
+            const char* fileName, int line, const char* function,
+            const std::string& msg, bool freeFilename)
     {
-        p = translatePriority(p);
-        LogEntry entry(c, p, fileName, line, msg);
+        auto tp = translatePriority(p, fileName, line, function, freeFilename);
+        if (!m_fileLine) {
+            /* This prevents output of file:line in StderrLogCallback. */
+            line = -line;
+        }
+
+        simgear::LogEntry entry(c, tp, p, fileName, line, function, msg, freeFilename);
         m_entries.push(entry);
     }
 
-    sgDebugPriority translatePriority(sgDebugPriority in) const
+    sgDebugPriority translatePriority(sgDebugPriority in,
+            const char* filename, int line, const char* function,
+            bool freeFilename) const
     {
+        in = logDeltaAdd(in, filename, line, function, freeFilename);
         if (in == SG_DEV_WARN) {
             return m_developerMode ? SG_WARN : SG_DEBUG;
         }
 
         if (in == SG_DEV_ALERT) {
-            return m_developerMode ? SG_POPUP : SG_WARN;
+            return m_developerMode ? SG_ALERT : SG_WARN;
         }
 
         return in;
@@ -499,7 +594,7 @@ public:
 /////////////////////////////////////////////////////////////////////////////
 
 static std::unique_ptr<logstream> global_logstream;
-static SGMutex global_logStreamLock;
+static std::mutex global_logStreamLock;
 
 logstream::logstream()
 {
@@ -509,8 +604,8 @@ logstream::logstream()
 
 logstream::~logstream()
 {
-    popup_msgs.clear();
     d->stop();
+    d.reset();
 }
 
 void
@@ -524,6 +619,16 @@ void logstream::setDeveloperMode(bool devMode)
     d->m_developerMode = devMode;
 }
 
+bool logstream::inDeveloperMode() const
+{
+    return d->m_developerMode;
+}
+
+
+void logstream::setFileLine(bool fileLine)
+{
+    d->m_fileLine = fileLine;
+}
 
 void
 logstream::addCallback(simgear::LogCallback* cb)
@@ -539,13 +644,24 @@ logstream::removeCallback(simgear::LogCallback* cb)
 
 void
 logstream::log( sgDebugClass c, sgDebugPriority p,
-        const char* fileName, int line, const std::string& msg)
+        const char* fileName, int line, const char* function,
+        const std::string& msg)
 {
-    d->log(c, p, fileName, line, msg);
+    d->log(c, p, fileName, line, function, msg, false);
+}
+
+void
+logstream::logCopyingFilename( sgDebugClass c, sgDebugPriority p,
+         const char* fileName, int line, const char* function,
+         const std::string& msg)
+{
+    d->log(c, p, strdup(fileName), line, strdup(function), msg, true);
 }
 
 
-void logstream::hexdump(sgDebugClass c, sgDebugPriority p, const char* fileName, int line, const void *mem, unsigned int len, unsigned int columns)
+void logstream::hexdump(sgDebugClass c, sgDebugPriority p,
+        const char* fileName, int line, const char* function,
+        const void *mem, unsigned int len, unsigned int columns)
 {
     unsigned int i, j;
     char temp[3000], temp1[3000];
@@ -557,14 +673,14 @@ void logstream::hexdump(sgDebugClass c, sgDebugPriority p, const char* fileName,
         /* print offset */
         if (i % columns == 0)
         {
-            sprintf(temp1, "0x%06x: ", i);
+            snprintf(temp1, 3000, "0x%06x: ", i);
             strcat(temp, temp1);
         }
 
         /* print hex data */
         if (i < len)
         {
-            sprintf(temp1, "%02x ", 0xFF & ((char*)mem)[i]);
+            snprintf(temp1, 3000, "%02x ", 0xFF & ((char*)mem)[i]);
             strcat(temp, temp1);
         }
         else /* end of block, just aligning for ASCII dump */
@@ -593,7 +709,7 @@ void logstream::hexdump(sgDebugClass c, sgDebugPriority p, const char* fileName,
                     strcat(temp, ".");
                 }
             }
-            log(c, p, fileName, line, temp );
+            log(c, p, fileName, line, function, temp );
             *temp = 0;
         }
     }
@@ -602,17 +718,18 @@ void logstream::hexdump(sgDebugClass c, sgDebugPriority p, const char* fileName,
 void
 logstream::popup( const std::string& msg)
 {
-    popup_msgs.push_back(msg);
+    std::lock_guard<std::mutex> g(d->m_lock);
+    d->_popupMessages.push_back(msg);
 }
 
 std::string
 logstream::get_popup()
 {
-    std::string rv = "";
-    if (!popup_msgs.empty())
-    {
-        rv = popup_msgs.front();
-        popup_msgs.erase(popup_msgs.begin());
+    std::string rv;
+    std::lock_guard<std::mutex> g(d->m_lock);
+    if (!d->_popupMessages.empty()) {
+        rv = d->_popupMessages.front();
+        d->_popupMessages.erase(d->_popupMessages.begin());
     }
     return rv;
 }
@@ -620,13 +737,16 @@ logstream::get_popup()
 bool
 logstream::has_popup()
 {
-    return (popup_msgs.size() > 0) ? true : false;
+    std::lock_guard<std::mutex> g(d->m_lock);
+    return !d->_popupMessages.empty();
 }
 
 bool
-logstream::would_log( sgDebugClass c, sgDebugPriority p ) const
+logstream::would_log( sgDebugClass c, sgDebugPriority p,
+        const char* file, int line, const char* function,
+        bool freeFilename ) const
 {
-    return d->would_log(c,p);
+    return d->would_log(c, p, file, line, function, freeFilename);
 }
 
 sgDebugClass
@@ -653,6 +773,77 @@ logstream::set_log_classes( sgDebugClass c)
     d->setLogLevels(c, d->m_logPriority);
 }
 
+sgDebugPriority logstream::priorityFromString(const std::string& s)
+{
+    if (s == "bulk") return SG_BULK;
+    if (s == "debug") return SG_DEBUG;
+    if (s == "info") return SG_INFO;
+    if (s == "warn") return SG_WARN;
+    if (s == "alert") return SG_ALERT;
+
+    throw std::invalid_argument("Couldn't parse log priority:" + s);
+}
+
+void logstream::parseLogClasses(const std::string& logClassesSpecification)
+{
+    using namespace simgear::strutils;
+    int classes = SG_NONE;
+    const auto lowerCaseSpec = lowercase(logClassesSpecification);
+
+    try {
+        if (logClassesSpecification.empty() || (lowerCaseSpec) == "all") {
+            classes = SG_ALL;
+        } else if (lowerCaseSpec == "none") {
+            classes = SG_NONE;
+        } else {
+            const auto pieces = split_on_any_of(lowerCaseSpec, "|,");
+            for (auto p : pieces) {
+                classes |= debugClassFromString(p);
+            }
+        }
+
+        d->setLogLevels(static_cast<sgDebugClass>(classes), d->m_logPriority);
+    } catch (std::exception& e) {
+        SG_LOG(SG_GENERAL, SG_WARN, "error parsing log classes '" << logClassesSpecification << "':" << e.what());
+    }
+}
+
+void logstream::addLogClass(const std::string& s)
+{
+    try {
+        const auto c = debugClassFromString(s);
+        d->setLogLevels(static_cast<sgDebugClass>(d->m_logClass | c), d->m_logPriority);
+    } catch (std::exception& e) {
+        SG_LOG(SG_GENERAL, SG_WARN, "error adding log class '" << s + "':" << e.what());
+    }
+}
+
+std::string logstream::getLogClassesAsString() const
+{
+    const auto classes = get_log_classes();
+    if (classes == SG_ALL) {
+        return "all";
+    }
+
+    if (classes == SG_NONE) {
+        return "none";
+    }
+
+    std::string result;
+    for (auto mapping : log_class_mappings) {
+        if ((classes & mapping.c) == 0) {
+            continue;
+        }
+
+        if (!result.empty()) {
+            result.append(",");
+        }
+
+        result.append(mapping.name);
+    }
+    return result;
+}
+
 
 logstream&
 sglog()
@@ -663,7 +854,7 @@ sglog()
     // http://www.aristeia.com/Papers/DDJ_Jul_Aug_2004_revised.pdf
     // in the absence of portable memory barrier ops in Simgear,
     // let's keep this correct & safe
-    SGGuard<SGMutex> g(global_logStreamLock);
+    std::lock_guard<std::mutex> g(global_logStreamLock);
 
     if( !global_logstream )
         global_logstream.reset(new logstream);
@@ -681,7 +872,7 @@ void logstream::setStartupLoggingEnabled(bool enabled)
     d->setStartupLoggingEnabled(enabled);
 }
 
-void logstream::requestConsole()
+void logstream::requestConsole(bool ignoreErrors)
 {
 #if defined (SG_WINDOWS)
     const bool stderrAlreadyRedirected = d->m_stderr_isRedirectedAlready;
@@ -720,7 +911,8 @@ void logstream::requestConsole()
             std::cerr.clear();
         }
     } else {
-        MessageBox(0, "--console ignored because stdout or stderr redirected with > or 2>", "Simgear Error", MB_OK | MB_ICONERROR);
+        if (!ignoreErrors)
+            MessageBox(0, "--console ignored because stdout or stderr redirected with > or 2>", "Simgear Error", MB_OK | MB_ICONERROR);
     }
 #endif
 }
@@ -736,15 +928,15 @@ logstream::setTestingMode( bool testMode )
 namespace simgear
 {
 
-void requestConsole()
+void requestConsole(bool ignoreErrors)
 { 
-    sglog().requestConsole();
+    sglog().requestConsole(ignoreErrors);
 }
 
 
 void shutdownLogging()
 {
-    SGGuard<SGMutex> g(global_logStreamLock);
+    std::lock_guard<std::mutex> g(global_logStreamLock);
     global_logstream.reset();
 }
 
