@@ -38,16 +38,22 @@
 #  include <simgear_config.h>
 #endif
 
+#include <iostream>
+#include <iomanip>
 #include <string>
 #include <time.h>
 #include <cstring>
+#include <ostream>
+#include <sstream>
 
 #include <simgear/debug/logstream.hxx>
+#include <simgear/math/sg_random.hxx>
 #include <simgear/structure/exception.hxx>
 
 #include "metar.hxx"
 
 #define NaN SGMetarNaN
+#define TRACE 0
 
 using std::string;
 using std::map;
@@ -71,7 +77,6 @@ using std::vector;
  * @endcode
  */
 SGMetar::SGMetar(const string& m) :
-	_grpcount(0),
 	_x_proxy(false),
 	_year(-1),
 	_month(-1),
@@ -92,14 +97,14 @@ SGMetar::SGMetar(const string& m) :
 	_snow(false),
 	_cavok(false)
 {
-	_data = new char[m.length() + 2];	// make room for " \0"
-	strcpy(_data, m.c_str());
-	_url = _data;
-		
-	normalizeData();
+    _data.resize(m.length() + 2); // make room for " \0"
+    strcpy(_data.data(), m.c_str());
+    _url = m;
 
-	_m = _data;
-	_icao[0] = '\0';
+    normalizeData();
+
+    _m = _data.data();
+    _icao[0] = '\0';
 
 	// NOAA preample
 	if (!scanPreambleDate())
@@ -109,37 +114,37 @@ SGMetar::SGMetar(const string& m) :
 	// METAR header
 	scanType();
 	if (!scanId() || !scanDate()) {
-		delete[] _data;
 		throw sg_io_exception("metar data bogus ", sg_location(_url));
 	}
-	scanModifier();
+	while (scanModifier()) ;
 
 	// base set
 	scanWind();
 	scanVariability();
-	while (scanVisibility()) ;
-	while (scanRwyVisRange()) ;
-	while (scanWeather()) ;
-	while (scanSkyCondition()) ;
-	scanTemperature();
-	scanPressure();
-	while (scanSkyCondition()) ;
-	while (scanRunwayReport()) ;
+	while (scanVisibility()) { /* empty loop body */}
+	while (scanRwyVisRange()) { /* empty loop body */}
+	while (scanWeather()) { /* empty loop body */}
+	while (scanSkyCondition()) { /* empty loop body */}
+
+    if (!scanTemperature()) {
+        throw sg_io_exception("metar temperature data malformed or missing ", sg_location(_url));
+    }
+    if (!scanPressure()) {
+        throw sg_io_exception("metar pressure data malformed or missing ", sg_location(_url));
+    }
+
+	while (scanSkyCondition()) { /* empty loop body */}
+	while (scanRunwayReport()) { /* empty loop body */}
 	scanWindShear();
 
 	// appendix
-	while (scanColorState()) ;
+	while (scanColorState()) { /* empty loop body */}
 	scanTrendForecast();
-	while (scanRunwayReport()) ;
+	while (scanRunwayReport()) { /* empty loop body */}
 	scanRemainder();
 	scanRemark();
 
-	if (_grpcount < 4) {
-		delete[] _data;
-		throw sg_io_exception("metar data incomplete ", sg_location(_url));
-	}
-
-	_url = "";
+    _url.clear();
 }
 
 
@@ -151,10 +156,323 @@ SGMetar::~SGMetar()
 	_clouds.clear();
 	_runways.clear();
 	_weather.clear();
-	delete[] _data;
 }
 
 
+static const char *azimuthName(double d)
+{
+	const char *dir[] = {
+		"N", "NNE", "NE", "ENE",
+		"E", "ESE", "SE", "SSE",
+		"S", "SSW", "SW", "WSW",
+		"W", "WNW", "NW", "NNW"
+	};
+	d += 11.25;
+	while (d < 0)
+		d += 360;
+	while (d >= 360)
+		d -= 360;
+	return dir[int(d / 22.5)];
+}
+
+
+// round double to 10^g
+static double rnd(double r, int g = 0)
+{
+	double f = pow(10.0, g);
+	return f * floor(r / f + 0.5);
+}
+
+
+/* A manipulator that can use spaces to emulate tab characters. */
+struct Tab
+{
+    /* If <stops> is 0, we simply insert tab characters. Otherwise we insert
+    spaces to align with the next column at multiple of <stops>. */
+    explicit Tab(int stops)
+    :
+    _stops(stops)
+    {}
+    int _stops;
+};
+
+std::ostream& operator << (std::ostream& out, const Tab& t)
+{
+    if (t._stops == 0) {
+        return out << '\t';
+    }
+    
+    std::ostringstream& out2 = *(std::ostringstream*) &out;
+    std::string s = out2.str();
+    
+    if (t._stops < 0) {
+        if (!s.size() || s[s.size()-1] != ' ') {
+            out << ' ';
+        }
+        return out;
+    }
+    
+    auto nl = s.rfind('\n');
+    if (nl == std::string::npos) nl = 0;
+    int column = 0;
+    for (auto i = nl+1; i != s.size(); ++i) {
+        if (s[i] == '\t')
+            column = (column + t._stops) / t._stops * t._stops;
+        else
+            column += 1;
+    }
+    int column2 = (column + t._stops) / t._stops * t._stops;
+    for (int i=column; i<column2; ++i) {
+        out << ' ';
+    }
+    return out;
+}
+
+/* Manipulator for SGMetarVisibility using a Tab. */
+struct SGMetarVisibilityManip
+{
+    explicit SGMetarVisibilityManip(const SGMetarVisibility& v, const Tab& tab)
+    :
+    _v(v),
+    _tab(tab)
+    {}
+    const SGMetarVisibility&    _v;
+    const Tab&                  _tab;
+};
+
+std::ostream& operator << (std::ostream& out, const SGMetarVisibilityManip& v)
+{
+	int m = v._v.getModifier();
+	const char *mod;
+	if (m == SGMetarVisibility::GREATER_THAN)
+		mod = ">=";
+	else if (m == SGMetarVisibility::LESS_THAN)
+		mod = "<";
+	else
+		mod = "";
+	out << mod;
+
+	double dist = rnd(v._v.getVisibility_m(), 1);
+	if (dist < 1000.0)
+		out << rnd(dist, 1) << " m";
+	else
+		out << rnd(dist / 1000.0, -1) << " km";
+
+	const char *dir = "";
+	int i;
+	if ((i = v._v.getDirection()) != -1) {
+		dir = azimuthName(i);
+		out << " " << dir;
+	}
+        out << v._tab << v._tab << v._tab << v._tab << v._tab;
+        out << mod << rnd(v._v.getVisibility_sm(), -1) << " US-miles " << dir;
+        return out;
+}
+
+
+std::string SGMetar::getDescription(int tabstops) const
+{
+    std::ostringstream  out;
+	const char *s;
+	char buf[256];
+	double d;
+	int i, lineno;
+        Tab tab(tabstops);
+        
+	if ((i = getReportType()) == SGMetar::AUTO)
+		out << "(METAR automatically generated)\n";
+	else if (i == SGMetar::COR)
+		out << "(METAR manually corrected)\n";
+	else if (i == SGMetar::RTD)
+		out << "(METAR routine delayed)\n";
+        
+	out << "Airport-Id:" << tab << tab << getId() << "\n";
+
+	// date/time
+	int year = getYear();
+	int month = getMonth();
+	out << "Report time:" << tab << tab << year << '/' << month << '/' << getDay();
+	out << ' ' << getHour() << ':';
+	out << std::setw(2) << std::setfill('0') << getMinute() << " UTC\n";
+
+
+	// visibility
+	SGMetarVisibility minvis = getMinVisibility();
+	SGMetarVisibility maxvis = getMaxVisibility();
+	double min = minvis.getVisibility_m();
+	double max = maxvis.getVisibility_m();
+	if (min != NaN) {
+		if (max != NaN) {
+			out << "min. Visibility:" << tab << SGMetarVisibilityManip(minvis, tab) << "\n";
+			out << "max. Visibility:" << tab << SGMetarVisibilityManip(maxvis, tab) << "\n";
+		} else {
+			out << "Visibility:" << tab << tab << SGMetarVisibilityManip(minvis, tab) << "\n";
+                }
+	}
+
+
+	// directed visibility
+	const SGMetarVisibility *dirvis = getDirVisibility();
+	for (i = 0; i < 8; i++, dirvis++)
+		if (dirvis->getVisibility_m() != NaN)
+			out << tab << tab << tab << SGMetarVisibilityManip(*dirvis, tab) << "\n";
+
+
+	// vertical visibility
+	SGMetarVisibility vertvis = getVertVisibility();
+	if ((d = vertvis.getVisibility_ft()) != NaN)
+		out << "Vert. visibility:" << tab << SGMetarVisibilityManip(vertvis, tab) << "\n";
+	else if (vertvis.getModifier() == SGMetarVisibility::NOGO)
+		out << "Vert. visibility:" << tab << "impossible to determine" << "\n";
+
+
+	// wind
+	d = getWindSpeed_kmh();
+	out << "Wind:" << tab << tab << tab;
+	if (d < .1)
+		out << "none" << "\n";
+	else {
+		if ((i = getWindDir()) == -1)
+			out << "from variable directions";
+		else
+			out << "from the " << azimuthName(i) << " (" << i << " deg)";
+		out << " at " << rnd(d, -1) << " km/h";
+
+		out << tab << tab << rnd(getWindSpeed_kt(), -1) << " kt";
+		out << " = " << rnd(getWindSpeed_mph(), -1) << " mph";
+		out << " = " << rnd(getWindSpeed_mps(), -1) << " m/s";
+		out << "\n";
+
+		d = getGustSpeed_kmh();
+                if (d != NaN && d != 0) {
+			out << tab << tab << tab << "with gusts at " << rnd(d, -1) << " km/h";
+			out << tab << tab << tab << rnd(getGustSpeed_kt(), -1) << " kt";
+			out << " = " << rnd(getGustSpeed_mph(), -1) << " mph";
+			out << " = " << rnd(getGustSpeed_mps(), -1) << " m/s";
+			out << "\n";
+		}
+
+		int from = getWindRangeFrom();
+		int to = getWindRangeTo();
+		if (from != to) {
+			out << tab << tab << tab << "variable from " << azimuthName(from);
+			out << " to " << azimuthName(to);
+			out << " (" << from << "deg --" << to << " deg)" << "\n";
+		}
+	}
+
+
+	// temperature/humidity/air pressure
+	if ((d = getTemperature_C()) != NaN) {
+		out << "Temperature:" << tab << tab << d << " C" << tab << tab << tab << tab << tab;
+		out << rnd(getTemperature_F(), -1) << " F" << "\n";
+
+		if ((d = getDewpoint_C()) != NaN) {
+			out << "Dewpoint:" << tab << tab << d << " C" << tab << tab << tab << tab << tab;
+			out << rnd(getDewpoint_F(), -1) << " F"  << "\n";
+			out << "Rel. Humidity: " << tab << tab << rnd(getRelHumidity()) << " %" << "\n";
+		}
+	}
+	if ((d = getPressure_hPa()) != NaN) {
+		out << "Pressure:" << tab << tab << rnd(d) << " hPa" << tab << tab << tab << tab;
+		out << rnd(getPressure_inHg(), -2) << " in. Hg" << "\n";
+	}
+
+
+	// weather phenomena
+	vector<string> wv = getWeather();
+	vector<string>::iterator weather;
+	for (i = 0, weather = wv.begin(); weather != wv.end(); weather++, i++) {
+		out << (i ? ", " : "Weather:") << tab << tab << weather->c_str();
+	}
+	if (i)
+		out << "\n";
+
+
+	// cloud layers
+	const char *coverage_string[5] = {
+		"clear skies", "few clouds", "scattered clouds", "broken clouds", "sky overcast"
+	};
+	vector<SGMetarCloud> cv = getClouds();
+	vector<SGMetarCloud>::iterator cloud;
+	for (lineno = 0, cloud = cv.begin(); cloud != cv.end(); cloud++, lineno++) {
+                if (lineno) out << tab << tab << tab;
+                else    out << "Sky condition:" << tab << tab;
+
+		if ((i = cloud->getCoverage()) != -1)
+			out << coverage_string[i];
+		if ((d = cloud->getAltitude_ft()) != NaN)
+			out << " at " << rnd(d, 1) << " ft";
+		if ((s = cloud->getTypeLongString()))
+			out << " (" << s << ')';
+		if (d != NaN)
+			out << tab << tab << tab << rnd(cloud->getAltitude_m(), 1) << " m";
+		out << "\n";
+	}
+
+
+	// runways
+	map<string, SGMetarRunway> rm = getRunways();
+	map<string, SGMetarRunway>::iterator runway;
+	for (runway = rm.begin(); runway != rm.end(); runway++) {
+		lineno = 0;
+		if (!strcmp(runway->first.c_str(), "ALL"))
+			out << "All runways:" << tab << tab;
+		else
+			out << "Runway " << runway->first << ":" << tab << tab;
+		SGMetarRunway rwy = runway->second;
+
+		// assemble surface string
+		vector<string> surface;
+		if ((s = rwy.getDepositString()) && strlen(s))
+			surface.push_back(s);
+		if ((s = rwy.getExtentString()) && strlen(s))
+			surface.push_back(s);
+		if ((d = rwy.getDepth()) != NaN) {
+			snprintf(buf, 256, "%.1lf mm", d * 1000.0);
+			surface.push_back(buf);
+		}
+		if ((s = rwy.getFrictionString()) && strlen(s))
+			surface.push_back(s);
+		if ((d = rwy.getFriction()) != NaN) {
+			snprintf(buf, 256, "friction: %.2lf", d);
+			surface.push_back(buf);
+		}
+
+		if (! surface.empty()) {
+			vector<string>::iterator rwysurf = surface.begin();
+			for (i = 0; rwysurf != surface.end(); rwysurf++, i++) {
+				if (i)
+					out << ", ";
+				out << *rwysurf;
+			}
+			lineno++;
+		}
+
+		// assemble visibility string
+		SGMetarVisibility minvis = rwy.getMinVisibility();
+		SGMetarVisibility maxvis = rwy.getMaxVisibility();
+		if ((d = minvis.getVisibility_m()) != NaN) {
+			if (lineno++)
+				out << "\n" << tab << tab << tab;
+			out << SGMetarVisibilityManip(minvis, tab);
+		}
+		if (maxvis.getVisibility_m() != d) {
+			out << "\n" << tab << tab << tab << SGMetarVisibilityManip(maxvis, tab) << "\n";
+			lineno++;
+		}
+
+		if (rwy.getWindShear()) {
+			if (lineno++)
+				out << "\n" << tab << tab << tab;
+			out << "critical wind shear" << "\n";
+		}
+		out << "\n";
+	}
+	out << "\n";
+        return out.str();
+}
+
 void SGMetar::useCurrentDate()
 {
 	struct tm now;
@@ -168,19 +486,37 @@ void SGMetar::useCurrentDate()
 	_month = now.tm_mon + 1;
 }
 
+std::string SGMetar::getDataString() const
+{
+    return std::string{_data.data()};
+}
+
+std::string SGMetar::getUnparsedData() const
+{
+    if (!_m || (strlen(_m) < 1))
+        return {};
+
+    return std::string{_m};
+}
 /**
   * Replace any number of subsequent spaces by just one space, and add
   * a trailing space. This makes scanning for things like "ALL RWY" easier.
   */
 void SGMetar::normalizeData()
 {
-	char *src, *dest;
-	for (src = dest = _data; (*dest++ = *src++); )
-		while (*src == ' ' && src[1] == ' ')
-			src++;
-	for (dest--; isspace(*--dest); ) ;
-	*++dest = ' ';
-	*++dest = '\0';
+    // TODO : use strutils::simplify here
+    char* src = _data.data();
+    char* dest = src;
+    for (; (*dest++ = *src++);) {
+        while (*src == ' ' && src[1] == ' ')
+            src++;
+    }
+
+    for (dest--; isspace(*--dest); )
+        ;
+
+    *++dest = ' ';
+    *++dest = '\0';
 }
 
 
@@ -235,7 +571,6 @@ bool SGMetar::scanType()
 	if (strncmp(_m, "METAR ", 6) && strncmp(_m, "SPECI ", 6))
 		return false;
 	_m += 6;
-	_grpcount++;
 	return true;
 }
 
@@ -252,7 +587,6 @@ bool SGMetar::scanId()
 	strncpy(_icao, _m, 4);
 	_icao[4] = '\0';
 	_m = m;
-	_grpcount++;
 	return true;
 }
 
@@ -260,24 +594,29 @@ bool SGMetar::scanId()
 // \d{6}Z
 bool SGMetar::scanDate()
 {
-	char *m = _m;
-	int day, hour, minute;
-	if (!scanNumber(&m, &day, 2))
-		return false;
-	if (!scanNumber(&m, &hour, 2))
-		return false;
-	if (!scanNumber(&m, &minute, 2))
-		return false;
-	if (*m++ != 'Z')
-		return false;
-	if (!scanBoundary(&m))
-		return false;
-	_day = day;
-	_hour = hour;
-	_minute = minute;
-	_m = m;
-	_grpcount++;
-	return true;
+    char* m = _m;
+    int day, hour, minute;
+
+    if (!scanNumber(&m, &day, 2))
+        return false;
+    if (!scanNumber(&m, &hour, 2))
+        return false;
+    if (!scanNumber(&m, &minute, 2))
+        return false;
+
+    if (*m == 'Z')
+        m++;
+    else if (*m != ' ')
+        return false;
+
+    if (!scanBoundary(&m))
+        return false;
+
+    _day = day;
+    _hour = hour;
+    _minute = minute;
+    _m = m;
+    return true;
 }
 
 
@@ -294,6 +633,8 @@ bool SGMetar::scanModifier()
 		m += 4, type = AUTO;
 	else if (!strncmp(m, "COR", 3))			// manually corrected
 		m += 3, type = COR;
+	else if (!strncmp(m, "CC", 2) && *(m+2) >= 'A' && *(m+2) <= 'B')    // correction
+		m += 3, type = COR;
 	else if (!strncmp(m, "RTD", 3))			// routine delayed
 		m += 3, type = RTD;
 	else
@@ -302,7 +643,6 @@ bool SGMetar::scanModifier()
 		return false;
 	_report_type = type;
 	_m = m;
-	_grpcount++;
 	return true;
 }
 
@@ -310,86 +650,135 @@ bool SGMetar::scanModifier()
 // (\d{3}|VRB)\d{1,3}(G\d{2,3})?(KT|KMH|MPS)
 bool SGMetar::scanWind()
 {
-	char *m = _m;
-	int dir;
-	if (!strncmp(m, "VRB", 3))
-		m += 3, dir = -1;
-	else if (!scanNumber(&m, &dir, 3))
-		return false;
+#if TRACE
+    std::cout << "metar wind: " << _m << std::endl;
+#endif
 
-	int i;
-	if (!scanNumber(&m, &i, 2, 3))
-		return false;
-	double speed = i;
+    char* m = _m;
+    int dir;
 
-	double gust = NaN;
-	if (*m == 'G') {
-		m++;
-		if (!scanNumber(&m, &i, 2, 3))
-			return false;
-		gust = i;
-	}
-	double factor;
-	if (!strncmp(m, "KT", 2))
-		m += 2, factor = SG_KT_TO_MPS;
-	else if (!strncmp(m, "KMH", 3))
-		m += 3, factor = SG_KMH_TO_MPS;
-	else if (!strncmp(m, "KPH", 3))		// ??
-		m += 3, factor = SG_KMH_TO_MPS;
-	else if (!strncmp(m, "MPS", 3))
-		m += 3, factor = 1.0;
-	else
-		return false;
-	if (!scanBoundary(&m))
-		return false;
-	_m = m;
-	_wind_dir = dir;
-	_wind_speed = speed * factor;
-	if (gust != NaN)
-		_gust_speed = gust * factor;
-	_grpcount++;
-	return true;
+    if (*m == '?' || *m == 'E') // not spec compliant
+        m++;
+    if (!strncmp(m, "VRB", 3))
+        m += 3, dir = -1;
+    else if (!strncmp(m, "///", 3)) // direction not measurable
+        m += 3, dir = -1;
+    else if (!scanNumber(&m, &dir, 3))
+        dir = -1;
+
+    if (*m == '/' && *(m + 1) != '/') // ignore single slash
+        m++;
+
+    int i;
+    if (!strncmp(m, "//", 2)) // speed not measurable
+        m += 2, i = -1;
+    else if (!scanNumber(&m, &i, 1, 3))
+        i = -1; // not spec compliant
+    double speed = i;
+
+    double gust = NaN;
+    if (*m == ' ' && *(m + 1) == 'G') // space between direction/velocity and gusts - not spec compliant
+        m++;
+    if (*m == 'G') {
+        m++;
+        if (!strncmp(m, "//", 2)) // speed not measurable
+            m += 2, i = -1;
+        else if (!scanNumber(&m, &i, 2, 3))
+            return false;
+
+        if (i != -1)
+            gust = i;
+    }
+
+    double factor;
+    if (!strncmp(m, "KT", 2))
+        m += 2, factor = SG_KT_TO_MPS;
+    else if (!strncmp(m, "KMH", 3)) // invalid Km/h
+        m += 3, factor = SG_KMH_TO_MPS;
+    else if (!strncmp(m, "KPH", 3)) // invalid Km/h
+        m += 3, factor = SG_KMH_TO_MPS;
+    else if (!strncmp(m, "MPS", 3))
+        m += 3, factor = 1.0;
+    else if (!strncmp(m, " ", 1)) // default to Knots
+        factor = SG_KT_TO_MPS;
+    else
+        return false;
+
+    if (!scanBoundary(&m))
+        return false;
+
+    _m = m;
+    _wind_dir = dir == -1 ? 0 : dir;
+    _wind_speed = speed < 0.0 ? 0.0 : speed * factor;
+    if (gust != NaN)
+        _gust_speed = gust * factor;
+    return true;
 }
 
 
 // \d{3}V\d{3}
 bool SGMetar::scanVariability()
 {
-	char *m = _m;
-	int from, to;
-	if (!scanNumber(&m, &from, 3))
-		return false;
-	if (*m++ != 'V')
-		return false;
-	if (!scanNumber(&m, &to, 3))
-		return false;
-	if (!scanBoundary(&m))
-		return false;
-	_m = m;
-	_wind_range_from = from;
-	_wind_range_to = to;
-	_grpcount++;
-	return true;
+#if TRACE
+    std::cout << "metar variability: " << _m << std::endl;
+#endif
+
+    char* m = _m;
+    int from, to;
+
+    if (!strncmp(m, "///", 3)) // direction not measurable
+        m += 3, from = -1;
+    else if (!scanNumber(&m, &from, 1, 3))
+        return false;
+
+    if (*m++ != 'V')
+        return false;
+
+    if (!strncmp(m, "///", 3)) // direction not measurable
+        m += 3, to = -1;
+    else if (!scanNumber(&m, &to, 1, 3))
+        return false;
+
+    if (!scanBoundary(&m))
+        return false;
+
+    _m = m;
+    _wind_range_from = from;
+    _wind_range_to = to;
+
+    return true;
 }
 
 
 bool SGMetar::scanVisibility()
 // TODO: if only directed vis are given, do still set min/max
 {
-	if (!strncmp(_m, "//// ", 5)) {         // spec compliant?
-		_m += 5;
-		_grpcount++;
-		return true;
-	}
+#if TRACE
+    std::cout << "metar visibility: " << _m << std::endl;
+#endif
 
-	char *m = _m;
+    if (!strncmp(_m, "/////", 5))
+        return false;       // not a visibility element (e.g. temperature)
+
+    if (!strncmp(_m, "////", 4)) {
+        _m += 4;
+
+        if (!strncmp(_m, "SM", 2))
+            _m += 2;
+        else if (!strncmp(_m, "KM", 2))
+            _m += 2;
+
+        return scanBoundary(&_m);
+    }
+
+    char *m = _m;
 	double distance;
 	int i, dir = -1;
 	int modifier = SGMetarVisibility::EQUALS;
 // \d{4}(N|NE|E|SE|S|SW|W|NW)?
 	if (scanNumber(&m, &i, 4)) {
 		if( strncmp( m, "NDV",3 ) == 0 ) {
-			m+=3; // tolerate NDV (no directional validation)
+			m+=3; // tolerate NDV (non-directional variation)
 		} else if (*m == 'E') {
 			m++, dir = 90;
 		} else if (*m == 'W') {
@@ -421,7 +810,7 @@ bool SGMetar::scanVisibility()
 		if (*m == 'M')
 			m++, modifier = SGMetarVisibility::LESS_THAN;
 
-		if (!scanNumber(&m, &i, 1, 2))
+		if (!scanNumber(&m, &i, 1, 3))
 			return false;
 		distance = i;
 
@@ -439,7 +828,8 @@ bool SGMetar::scanVisibility()
 				return false;
 			if (!scanNumber(&m, &denom, 1, 2))
 				return false;
-			distance += (double)i / denom;
+			if (denom != 0)
+				distance += (double)i / denom;
 		}
 
 		if (!strncmp(m, "SM", 2))
@@ -464,7 +854,6 @@ bool SGMetar::scanVisibility()
 	v->_modifier = modifier;
 	v->_direction = dir;
 	_m = m;
-	_grpcount++;
 	return true;
 }
 
@@ -472,6 +861,10 @@ bool SGMetar::scanVisibility()
 // R\d\d[LCR]?/([PM]?\d{4}V)?[PM]?\d{4}(FT)?[DNU]?
 bool SGMetar::scanRwyVisRange()
 {
+#if TRACE
+    std::cout << "metar runway range: " << _m << std::endl;
+#endif
+
 	char *m = _m;
 	int i;
 	SGMetarRunway r;
@@ -490,6 +883,12 @@ bool SGMetar::scanRwyVisRange()
 	if (*m++ != '/')
 		return false;
 
+	if (!strncmp(m, "////", 4)) {
+		// sensor failure... ignore
+		_m = m + 4;
+		return scanBoundary(&_m);
+	}
+
 	int from, to;
 	if (*m == 'P')
 		m++, r._min_visibility._modifier = SGMetarVisibility::GREATER_THAN;
@@ -531,7 +930,6 @@ bool SGMetar::scanRwyVisRange()
 
 	_runways[id]._min_visibility = r._min_visibility;
 	_runways[id]._max_visibility = r._max_visibility;
-	_grpcount++;
 	return true;
 }
 
@@ -553,6 +951,7 @@ static const struct Token description[] = {
 	{ "FZ", "freezing" },
 	{ "MI", "shallow" },
 	{ "PR", "partial" },
+	{ "RE", "recent" },
 	{ 0, 0 }
 };
 
@@ -563,6 +962,7 @@ static const struct Token phenomenon[] = {
 	{ "GS",   "small hail and/or snow pellets" },
 	{ "IC",   "ice crystals" },
 	{ "PE",   "ice pellets" },
+	{ "PL",   "ice pellets" },
 	{ "RA",   "rain" },
 	{ "SG",   "snow grains" },
 	{ "SN",   "snow" },
@@ -576,12 +976,11 @@ static const struct Token phenomenon[] = {
 	{ "PY",   "spray" },
 	{ "SA",   "sand" },
 	{ "VA",   "volcanic ash" },
-	{ "DS",   "duststorm" },
+	{ "DS",   "dust storm" },
 	{ "FC",   "funnel cloud/tornado waterspout" },
 	{ "PO",   "well-developed dust/sand whirls" },
 	{ "SQ",   "squalls" },
 	{ "SS",   "sandstorm" },
-	{ "UP",   "unknown" },	// ... due to failed automatic acquisition
 	{ 0, 0 }
 };
 
@@ -589,6 +988,10 @@ static const struct Token phenomenon[] = {
 // (+|-|VC)?(NSW|MI|PR|BC|DR|BL|SH|TS|FZ)?((DZ|RA|SN|SG|IC|PE|GR|GS|UP){0,3})(BR|FG|FU|VA|DU|SA|HZ|PY|PO|SQ|FC|SS|DS){0,3}
 bool SGMetar::scanWeather()
 {
+#if TRACE
+    std::cout << "metar weather: " << _m << std::endl;
+#endif
+
 	char *m = _m;
 	string weather;
 	const struct Token *a;
@@ -597,10 +1000,15 @@ bool SGMetar::scanWeather()
 	// Denotes a temporary failure of the sensor
 	if (!strncmp(m, "// ", 3)) {
 		_m += 3;
-		_grpcount++;
 		return false;
 	}
 
+    // maintenance flag
+    if (*m == 'M' && *(m+1) == ' ') {
+        _m++;
+        return scanBoundary(&_m);
+    }
+
 	if ((a = scanToken(&m, special))) {
 		if (!scanBoundary(&m))
 			return false;
@@ -635,6 +1043,8 @@ bool SGMetar::scanWeather()
 		weather += string(a->text) + " ";
 		if (!strcmp(a->id, "RA"))
 			_rain = w.intensity;
+		else if (!strcmp(a->id, "DZ"))
+			_rain = LIGHT;
 		else if (!strcmp(a->id, "HA"))
 			_hail = w.intensity;
 		else if (!strcmp(a->id, "SN"))
@@ -651,7 +1061,6 @@ bool SGMetar::scanWeather()
     if( ! w.phenomena.empty() ) {
         _weather2.push_back( w );
     }
-    _grpcount++;
     return true;
 }
 
@@ -683,16 +1092,21 @@ static const struct Token cloud_types[] = {
 // (FEW|SCT|BKN|OVC|SKC|CLR|CAVOK|VV)([0-9]{3}|///)?[:cloud_type:]?
 bool SGMetar::scanSkyCondition()
 {
+#if TRACE
+    std::cout << "metar sky condition: " << _m << std::endl;
+#endif
+
 	char *m = _m;
 	int i;
 	SGMetarCloud cl;
+    static SGMetarCloud::Coverage priorCoverage;
 
 	if (!strncmp(m, "//////", 6)) {
-		m += 6;
-		if (!scanBoundary(&m))
-			return false;
-		_m = m;
-		return true;
+		char* m2 = m+6;
+		if (scanBoundary(&m2)) {
+			_m = m2;
+			return true;
+		}
 	}
 
 	if (!strncmp(m, "CLR", i = 3)				// clear
@@ -714,31 +1128,50 @@ bool SGMetar::scanSkyCondition()
 		return true;
 	}
 
-	if (!strncmp(m, "VV", i = 2))				// vertical visibility
-		;
-	else if (!strncmp(m, "FEW", i = 3))
+    bool verticalVisibility = false;
+    if (!strncmp(m, "VV", i = 2)) {             // vertical visibility
+        verticalVisibility = true;
+    } else if (!strncmp(m, "FEW", i = 3)) {
         cl._coverage = SGMetarCloud::COVERAGE_FEW;
-	else if (!strncmp(m, "SCT", i = 3))
+        priorCoverage = cl._coverage;
+    } else if (!strncmp(m, "SCT", i = 3)) {
         cl._coverage = SGMetarCloud::COVERAGE_SCATTERED;
-	else if (!strncmp(m, "BKN", i = 3))
+        priorCoverage = cl._coverage;
+    } else if (!strncmp(m, "BKN", i = 3)) {
         cl._coverage = SGMetarCloud::COVERAGE_BROKEN;
-	else if (!strncmp(m, "OVC", i = 3))
+        priorCoverage = cl._coverage;
+    } else if (!strncmp(m, "OVC", i = 3)) {
         cl._coverage = SGMetarCloud::COVERAGE_OVERCAST;
-	else
-		return false;
-	m += i;
+        priorCoverage = cl._coverage;
+    } else if (!strncmp(m, "///", i = 3))
+        cl._coverage = SGMetarCloud::COVERAGE_NIL; // should we add 'unknown'?
+    else {
+        // check for an implied coverage element
+        if (*m >= '0' && *m <= '9' &&
+            *(m+1) >= '0' && *(m+1) <= '9' &&
+            *(m+2) >= '0' && *(m+2) <= '9' &&
+            *(m+3) == ' ') {
+            cl._coverage = priorCoverage;
+            i = 0;
+        } else
+            return false;
+    }
+    m += i;
 
-	if (!strncmp(m, "///", 3))	// vis not measurable (e.g. because of heavy snowing)
+    if (!strncmp(m, "///", 3)) { // vis not measurable (e.g. because of heavy snowing)
 		m += 3, i = -1;
-	else if (scanBoundary(&m)) {
+		sg_srandom_time();
+		// randomize the base height to avoid the black sky issue
+		i = 50 + static_cast<int>(sg_random() * 250.0);		// range [5,000, 30,000]
+	} else if (scanBoundary(&m)) {
 		_m = m;
 		return true;				// ignore single OVC/BKN/...
 	} else if (!scanNumber(&m, &i, 3))
 		i = -1;
 
-	if (cl._coverage == SGMetarCloud::COVERAGE_NIL) {
-		if (!scanBoundary(&m))
-			return false;
+	if (verticalVisibility) {
+		 if (!scanBoundary(&m))
+		 	return false;
 		if (i == -1)			// 'VV///'
 			_vert_visibility._modifier = SGMetarVisibility::NOGO;
 		else
@@ -762,10 +1195,12 @@ bool SGMetar::scanSkyCondition()
 		m += 3;
 	if (!scanBoundary(&m))
 		return false;
-	_clouds.push_back(cl);
+
+	// require known coverage and base-height
+	if (i != -1 && cl._coverage != SGMetarCloud::COVERAGE_NIL)
+		_clouds.push_back(cl);
 
 	_m = m;
-	_grpcount++;
 	return true;
 }
 
@@ -774,29 +1209,58 @@ bool SGMetar::scanSkyCondition()
 // (M?[0-9]{2}|XX)/(M?[0-9]{2}|XX)?    (Namibia)
 bool SGMetar::scanTemperature()
 {
+#if TRACE
+    std::cout << "metar temp: " << _m << std::endl;
+#endif
+
 	char *m = _m;
 	int sign = 1, temp, dew;
-	if (!strncmp(m, "XX/XX", 5)) {		// not spec compliant!
+
+    // sniff test to confirm that this is a temperature element
+    std::string s = "M/0123456789";
+    for (int i=0; i<7; ++i) {
+        if (*(m+i) == ' ')
+            break;
+        if (s.find(*(m+i)) == std::string::npos)
+            return true;        // nope, bail
+    }
+
+	if (!strncmp(m, "XX/XX", 5)) {		            // not spec compliant!
+		_m += 5;
+		return scanBoundary(&_m);
+	}
+	if (!strncmp(m, "/////", 5)) {
+		// sensor failure... assume standard temperature
+		_temp = 15.0;
+		_dewp = 3.0;
 		_m += 5;
 		return scanBoundary(&_m);
 	}
 
+    // maintenance flag
+    if (*m == 'M' && *(m+1) == ' ') {
+        _m++;
+        return scanBoundary(&_m);
+    }
+
 	if (*m == 'M')
 		m++, sign = -1;
-	if (!scanNumber(&m, &temp, 2))
+	if (!scanNumber(&m, &temp, 1, 2))
 		return false;
 	temp *= sign;
 
 	if (*m++ != '/')
 		return false;
 	if (!scanBoundary(&m)) {
-		if (!strncmp(m, "XX", 2))	// not spec compliant!
-			m += 2, sign = 0, dew = temp;
+		if (!strncmp(m, "XX", 2))	                // not spec compliant!
+			m += 2, sign = 0, dew = temp - 10;
+		else if (!strncmp(m, "//", 2))	                // sensor failure... assume relative dew point
+			m += 2, sign = 0, dew = temp - 10;
 		else {
 			sign = 1;
 			if (*m == 'M')
 				m++, sign = -1;
-			if (!scanNumber(&m, &dew, 2))
+			if (!scanNumber(&m, &dew, 1, 2))
 				return false;
 		}
 		if (!scanBoundary(&m))
@@ -806,7 +1270,6 @@ bool SGMetar::scanTemperature()
 	}
 	_temp = temp;
 	_m = m;
-	_grpcount++;
 	return true;
 }
 
@@ -825,31 +1288,73 @@ double SGMetar::getRelHumidity() const
 // [AQ]\d{2}(\d{2}|//)   (Namibia)
 bool SGMetar::scanPressure()
 {
+#if TRACE
+    std::cout << "metar pressure: " << _m << std::endl;
+#endif
+
 	char *m = _m;
 	double factor;
+    bool unitProvided = true;
+    bool valueProvided = true;
 	int press, i;
 
-	if (*m == 'A')
-		factor = SG_INHG_TO_PA / 100;
-	else if (*m == 'Q')
-		factor = 100;
+	if (*m == '\0') {
+		_pressure = 101300.0;	// pressure not provided... assume standard pressure
+		return true;
+	}
+
+	if (*m == 'A') {
+		factor = SG_INHG_TO_PA / 100.0;
+        m++;
+    }
+	else if (*m == 'Q') {
+		factor = 100.0;
+        m++;
+    }
 	else
-		return false;
-	m++;
-	if (!scanNumber(&m, &press, 2))
-		return false;
-	press *= 100;
-	if (!strncmp(m, "//", 2))	// not spec compliant!
-		m += 2;
-	else if (scanNumber(&m, &i, 2))
-		press += i;
-	else
-		return false;
-	if (!scanBoundary(&m))
-		return false;
-	_pressure = press * factor;
+        unitProvided = false;
+
+	if (*m == ' ')	// ignore space
+		m++;
+    if (!strncmp(m, "////", 4)) {
+        // sensor failure... assume standard pressure
+        if (*(m - 1) == 'A')
+            press = 2992;
+        else
+            press = 1013;
+        m += 4;
+    } else {
+        if (!scanNumber(&m, &press, 2, 4)) {
+			valueProvided = false;
+
+			// sensor failure... assume standard pressure
+			if (*(m - 1) == 'A')
+				press = 2992;
+			else
+				press = 1013;
+		}
+
+		if (press < 100) {				// 2-digit pressure may have further data following
+        	press *= 100;
+        	if (!strncmp(m, "//", 2)) 	// not spec compliant!
+        	    m += 2;
+        	else if (scanNumber(&m, &i, 2))
+        	    press += i;
+		}
+    }
+
+	if (*m == ',' || *m == '=')	// ignore trailing comma, equals
+		m++;
+
+    if ((unitProvided || valueProvided) && !scanBoundary(&m))
+        return false;
+
+    // derive unit when not explicitly provided
+    if (!unitProvided)
+        factor = (press > 2000 ? SG_INHG_TO_PA / 100.0 : 100.0);
+
+    _pressure = press * factor;
 	_m = m;
-	_grpcount++;
 	return true;
 }
 
@@ -965,7 +1470,6 @@ bool SGMetar::scanRunwayReport()
 	_runways[id]._friction_string = r._friction_string;
 	_runways[id]._comment = r._comment;
 	_m = m;
-	_grpcount++;
 	return true;
 }
 
