@@ -33,13 +33,10 @@
 #include <errno.h>
 #include <map>
 #include <stdexcept>
-
-#include <boost/foreach.hpp>
-#include <boost/algorithm/string/case_conv.hpp>
+#include <mutex>
 
 #include <simgear/simgear_config.h>
 
-#include <curl/multi.h>
 
 #include <simgear/io/sg_netChat.hxx>
 
@@ -49,6 +46,9 @@
 #include <simgear/timing/timestamp.hxx>
 #include <simgear/structure/exception.hxx>
 
+#include "HTTPClient_private.hxx"
+#include "HTTPTestApi_private.hxx"
+
 #if defined( HAVE_VERSION_H ) && HAVE_VERSION_H
 #include "version.h"
 #else
@@ -66,73 +66,33 @@ namespace HTTP
 extern const int DEFAULT_HTTP_PORT = 80;
 const char* CONTENT_TYPE_URL_ENCODED = "application/x-www-form-urlencoded";
 
-class Connection;
-typedef std::multimap<std::string, Connection*> ConnectionDict;
-typedef std::list<Request_ptr> RequestList;
-
-class Client::ClientPrivate
-{
-public:
-    CURLM* curlMulti;
-
-    void createCurlMulti()
-    {
-        curlMulti = curl_multi_init();
-        // see https://curl.haxx.se/libcurl/c/CURLMOPT_PIPELINING.html
-        // we request HTTP 1.1 pipelining
-        curl_multi_setopt(curlMulti, CURLMOPT_PIPELINING, 1 /* aka CURLPIPE_HTTP1 */);
+void Client::ClientPrivate::createCurlMulti() {
+  curlMulti = curl_multi_init();
+  // see https://curl.haxx.se/libcurl/c/CURLMOPT_PIPELINING.html
+  // we request HTTP 1.1 pipelining
+  curl_multi_setopt(curlMulti, CURLMOPT_PIPELINING, 1 /* aka CURLPIPE_HTTP1 */);
 #if (LIBCURL_VERSION_MINOR >= 30)
-        curl_multi_setopt(curlMulti, CURLMOPT_MAX_TOTAL_CONNECTIONS, (long) maxConnections);
-        curl_multi_setopt(curlMulti, CURLMOPT_MAX_PIPELINE_LENGTH,
-                          (long) maxPipelineDepth);
-        curl_multi_setopt(curlMulti, CURLMOPT_MAX_HOST_CONNECTIONS,
-                          (long) maxHostConnections);
+  curl_multi_setopt(curlMulti, CURLMOPT_MAX_TOTAL_CONNECTIONS,
+                    (long)maxConnections);
+  curl_multi_setopt(curlMulti, CURLMOPT_MAX_PIPELINE_LENGTH,
+                    (long)maxPipelineDepth);
+  curl_multi_setopt(curlMulti, CURLMOPT_MAX_HOST_CONNECTIONS,
+                    (long)maxHostConnections);
 #endif
-    }
+}
 
-    typedef std::map<Request_ptr, CURL*> RequestCurlMap;
-    RequestCurlMap requests;
-
-    std::string userAgent;
-    std::string proxy;
-    int proxyPort;
-    std::string proxyAuth;
-    unsigned int maxConnections;
-    unsigned int maxHostConnections;
-    unsigned int maxPipelineDepth;
-
-    RequestList pendingRequests;
-
-    SGTimeStamp timeTransferSample;
-    unsigned int bytesTransferred;
-    unsigned int lastTransferRate;
-    uint64_t totalBytesDownloaded;
-
-    SGPath tlsCertificatePath;
-};
-
-Client::Client() :
-    d(new ClientPrivate)
+Client::Client()
 {
-    d->proxyPort = 0;
-    d->maxConnections = 4;
-    d->maxHostConnections = 4;
-    d->bytesTransferred = 0;
-    d->lastTransferRate = 0;
-    d->timeTransferSample.stamp();
-    d->totalBytesDownloaded = 0;
-    d->maxPipelineDepth = 5;
-    setUserAgent("SimGear-" SG_STRINGIZE(SIMGEAR_VERSION));
-
-    d->tlsCertificatePath = SGPath::fromEnv("SIMGEAR_TLS_CERT_PATH");
-
     static bool didInitCurlGlobal = false;
+    static std::mutex initMutex;
+
+    std::lock_guard<std::mutex> g(initMutex);
     if (!didInitCurlGlobal) {
       curl_global_init(CURL_GLOBAL_ALL);
       didInitCurlGlobal = true;
     }
 
-    d->createCurlMulti();
+    reset();
 }
 
 Client::~Client()
@@ -164,6 +124,28 @@ void Client::setMaxPipelineDepth(unsigned int depth)
 #endif
 }
 
+void Client::reset()
+{
+    if (d.get()) {
+        curl_multi_cleanup(d->curlMulti);
+    }
+
+    d.reset(new ClientPrivate);
+
+    d->proxyPort = 0;
+    d->maxConnections = 4;
+    d->maxHostConnections = 4;
+    d->bytesTransferred = 0;
+    d->lastTransferRate = 0;
+    d->timeTransferSample.stamp();
+    d->totalBytesDownloaded = 0;
+    d->maxPipelineDepth = 5;
+    d->curlPerformActive = false;
+    setUserAgent("SimGear-" SG_STRINGIZE(SIMGEAR_VERSION));
+    d->tlsCertificatePath = SGPath::fromEnv("SIMGEAR_TLS_CERT_PATH");
+    d->createCurlMulti();
+}
+
 void Client::update(int waitTimeout)
 {
     if (d->requests.empty()) {
@@ -174,32 +156,29 @@ void Client::update(int waitTimeout)
     }
 
     int remainingActive, messagesInQueue;
-#if defined(SG_MAC)
-    // Mac 10.8 libCurl lacks this, let's keep compat for now
-    fd_set curlReadFDs, curlWriteFDs, curlErrorFDs;
-    int maxFD;
-    curl_multi_fdset(d->curlMulti,
-                     &curlReadFDs,
-                     &curlWriteFDs,
-                     &curlErrorFDs,
-                     &maxFD);
-    
-    struct timeval timeout;
-    long t;
-    
-    curl_multi_timeout(d->curlMulti, &t);
-    if ((t < 0) || (t > waitTimeout)) {
-        t = waitTimeout;
-    }
-    
-    timeout.tv_sec = t / 1000;
-    timeout.tv_usec = (t % 1000) * 1000;
-    ::select(maxFD, &curlReadFDs, &curlWriteFDs, &curlErrorFDs, &timeout);
-#else
     int numFds;
-    curl_multi_wait(d->curlMulti, NULL, 0, waitTimeout, &numFds);
-#endif
-    curl_multi_perform(d->curlMulti, &remainingActive);
+    CURLMcode mc = curl_multi_wait(d->curlMulti, NULL, 0, waitTimeout, &numFds);
+    if (mc != CURLM_OK) {
+        SG_LOG(SG_IO, SG_WARN, "curl_multi_wait failed:" << curl_multi_strerror(mc));
+        return;
+    }
+
+    d->curlPerformActive = true;
+    mc = curl_multi_perform(d->curlMulti, &remainingActive);
+    if (mc == CURLM_CALL_MULTI_PERFORM) {
+        // we could loop here, but don't want to get blocked
+        // also this shouldn't  ocurr in any modern libCurl
+        curl_multi_perform(d->curlMulti, &remainingActive);
+    } else if (mc != CURLM_OK) {
+        d->curlPerformActive = false;
+        SG_LOG(SG_IO, SG_WARN, "curl_multi_perform failed:" << curl_multi_strerror(mc));
+        return;
+    }
+
+    d->curlPerformActive = false;
+    for (auto r : d->pendingCancelRequests) {
+        cancelRequest(r, "deferred cancellation");
+    };
 
     CURLMsg* msg;
     while ((msg = curl_multi_info_read(d->curlMulti, &messagesInQueue))) {
@@ -224,12 +203,33 @@ void Client::update(int waitTimeout)
           assert(it->second == e);
           d->requests.erase(it);
 
-        if (msg->data.result == 0) {
-          req->responseComplete();
-        } else {
-          SG_LOG(SG_IO, SG_WARN, "CURL Result:" << msg->data.result << " " << curl_easy_strerror(msg->data.result));
-          req->setFailure(msg->data.result, curl_easy_strerror(msg->data.result));
-        }
+          bool doProcess = true;
+          if (d->testsuiteResponseDoneCallback) {
+            doProcess =
+                !d->testsuiteResponseDoneCallback(msg->data.result, req);
+          }
+
+          if (doProcess) {
+            bool ok = false;
+            SG_LOG(SG_IO, SG_DEBUG, "msg->data.result=" << msg->data.result);
+            if (msg->data.result == CURLE_OK) {
+              ok = true;
+            }
+            else if (req->range() != "" && (responseCode == 206 || responseCode == 416)) {
+              SG_LOG(SG_IO, SG_DEBUG, "req->range() is set (to '" << req->range() << "')"
+                      << " so treating as ok: msg->data.result=" << msg->data.result);
+              ok = true;
+            }
+            if (ok) {
+              req->responseComplete();
+            } else {
+              SG_LOG(SG_IO, SG_WARN,
+                     "CURL Result:" << msg->data.result << " "
+                                    << curl_easy_strerror(msg->data.result));
+              req->setFailure(msg->data.result,
+                              curl_easy_strerror(msg->data.result));
+            }
+          }
 
         curl_multi_remove_handle(d->curlMulti, e);
         curl_easy_cleanup(e);
@@ -273,10 +273,23 @@ void Client::makeRequest(const Request_ptr& r)
     curl_easy_setopt(curlRequest, CURLOPT_WRITEDATA, r.get());
     curl_easy_setopt(curlRequest, CURLOPT_HEADERFUNCTION, requestHeaderCallback);
     curl_easy_setopt(curlRequest, CURLOPT_HEADERDATA, r.get());
+    unsigned long bps = r->getMaxBytesPerSec();
+    if (bps != 0) {
+        curl_easy_setopt(curlRequest, CURLOPT_MAX_RECV_SPEED_LARGE, bps);
+    }
 
+#if !defined(CURL_MAX_READ_SIZE)
+	const int CURL_MAX_READ_SIZE = 512 * 1024;
+#endif
+
+    curl_easy_setopt(curlRequest, CURLOPT_BUFFERSIZE, CURL_MAX_READ_SIZE);
     curl_easy_setopt(curlRequest, CURLOPT_USERAGENT, d->userAgent.c_str());
     curl_easy_setopt(curlRequest, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
 
+    if (sglog().would_log(SG_TERRASYNC, SG_DEBUG, __FILE__, __LINE__, __FUNCTION__)) {
+        curl_easy_setopt(curlRequest, CURLOPT_VERBOSE, 1);
+    }
+
     curl_easy_setopt(curlRequest, CURLOPT_FOLLOWLOCATION, 1);
 
     if (!d->tlsCertificatePath.isNull()) {
@@ -294,7 +307,19 @@ void Client::makeRequest(const Request_ptr& r)
       }
     }
 
-    std::string method = boost::to_lower_copy(r->method());
+    std::string range = r->range();
+    if (range != "") {
+        curl_easy_setopt(curlRequest, CURLOPT_RANGE, range.c_str());
+        SG_LOG(SG_GENERAL, SG_DEBUG, "Have added CURLOPT_RANGE: '" << range << "'");
+    }
+    
+    const char* enc = r->getAcceptEncoding();
+    if (enc) {
+        curl_easy_setopt(curlRequest, CURLOPT_ACCEPT_ENCODING, const_cast<char*>(enc));
+        SG_LOG(SG_GENERAL, SG_ALERT, "Have added CURLOPT_ACCEPT_ENCODING: '" << enc << "'");
+    }
+
+    const std::string method = strutils::lowercase (r->method());
     if (method == "get") {
       curl_easy_setopt(curlRequest, CURLOPT_HTTPGET, 1);
     } else if (method == "put") {
@@ -352,9 +377,16 @@ void Client::cancelRequest(const Request_ptr &r, std::string reason)
         return;
     }
 
+    // don't cancel from inside curl_multi_perform, causes
+    // everything to melt. Defer until after it's done
+    if (d->curlPerformActive) {
+        d->pendingCancelRequests.push_back(r);
+        return;
+    }
+
     CURLMcode err = curl_multi_remove_handle(d->curlMulti, it->second);
     if (err != CURLM_OK) {
-      SG_LOG(SG_IO, SG_WARN, "curl_multi_remove_handle failed:" << err);
+        SG_LOG(SG_IO, SG_WARN, "cancelRequest: curl_multi_remove_handle failed:" << err);
     }
 
     // clear the request pointer form the curl-easy object
@@ -485,7 +517,7 @@ bool isRedirectStatus(int code)
 {
     return ((code >= 300) && (code < 400));
 }
-    
+
 size_t Client::requestHeaderCallback(char *rawBuffer, size_t size, size_t nitems, void *userdata)
 {
   size_t byteSize = size * nitems;
@@ -500,7 +532,7 @@ size_t Client::requestHeaderCallback(char *rawBuffer, size_t size, size_t nitems
           return byteSize;
       }
   }
-    
+
   if (req->readyState() == HTTP::Request::OPENED) {
     req->responseStart(h);
     return byteSize;
@@ -527,8 +559,8 @@ size_t Client::requestHeaderCallback(char *rawBuffer, size_t size, size_t nitems
       return byteSize;
   }
 
-  std::string key = strutils::simplify(h.substr(0, colonPos));
-  std::string lkey = boost::to_lower_copy(key);
+  const std::string key = strutils::simplify(h.substr(0, colonPos));
+  const std::string lkey = strutils::lowercase (key);
   std::string value = strutils::strip(h.substr(colonPos + 1));
 
   req->responseHeader(lkey, value);
@@ -551,6 +583,17 @@ void Client::clearAllConnections()
     d->createCurlMulti();
 }
 
+/////////////////////////////////////////////////////////////////////
+
+void TestApi::setResponseDoneCallback(Client *cl, ResponseDoneCallback cb) {
+  cl->d->testsuiteResponseDoneCallback = cb;
+}
+
+void TestApi::markRequestAsFailed(Request_ptr req, int curlCode,
+                                  const std::string &message) {
+  req->setFailure(curlCode, message);
+}
+
 } // of namespace HTTP
 
 } // of namespace simgear
