@@ -7,8 +7,6 @@
 #include <thread>
 #include <atomic>
 
-#include <boost/algorithm/string/case_conv.hpp>
-
 #include <simgear/simgear_config.h>
 
 #include "DNSClient.hxx"
@@ -52,7 +50,7 @@ public:
 private:
     unsigned int _interval;
     std::atomic<unsigned int> _timer;
-    std::atomic<bool> _running;
+    std::atomic<int> _running;
     std::thread _thread;
 
     void loop()
