@@ -41,7 +41,6 @@
 
 #include <simgear/misc/strutils.hxx>
 #include <simgear/debug/logstream.hxx>
-#include <boost/foreach.hpp>
 
 #include <cstring>
 #include <cstdlib>
@@ -115,14 +114,14 @@ Dir Dir::tempDir(const std::string& templ)
     // Mac OS-X / BSD manual says any number of 'X's, but GLibc manual
     // says exactly six, so that's what I'm going with
     p.concat("-XXXXXX");
-    std::string s = p.local8BitStr();
+    std::string s = p.utf8Str();
     ::snprintf(buf, 1024, "%s", s.c_str());
     if (!mkdtemp(buf)) {
         SG_LOG(SG_IO, SG_WARN,
                "mkdtemp failed: " << simgear::strutils::error_string(errno));
         return Dir();
     }
-    
+
     return Dir(SGPath(buf));
 #else
 #if defined(SG_WINDOWS)
@@ -138,7 +137,7 @@ Dir Dir::tempDir(const std::string& templ)
         SG_LOG(SG_IO, SG_WARN, "failed to create temporary directory at " << p);
         return Dir();
     }
-    
+
     return t;
 #endif
 }
@@ -154,7 +153,7 @@ PathList Dir::children(int types, const std::string& nameFilter) const
   if (types == 0) {
     types = TYPE_FILE | TYPE_DIR | NO_DOT_OR_DOTDOT;
   }
-  
+
 #if defined(SG_WINDOWS)
   std::wstring search(_path.wstr());
   if (nameFilter.empty()) {
@@ -162,18 +161,18 @@ PathList Dir::children(int types, const std::string& nameFilter) const
   } else {
     search += simgear::strutils::convertUtf8ToWString("\\*" + nameFilter);
   }
-  
+
   WIN32_FIND_DATAW fData;
   HANDLE find = FindFirstFileW(search.c_str(), &fData);
   if (find == INVALID_HANDLE_VALUE) {
 	  int err = GetLastError();
 	  if (err != ERROR_FILE_NOT_FOUND) {
-		SG_LOG(SG_GENERAL, SG_WARN, "Dir::children: FindFirstFile failed:" << 
+		SG_LOG(SG_GENERAL, SG_WARN, "Dir::children: FindFirstFile failed:" <<
 			_path << " with error:" << err);
 	  }
     return result;
   }
-  
+
   bool done = false;
   for (bool done = false; !done; done = (FindNextFileW(find, &fData) == 0)) {
     if (fData.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) {
@@ -181,14 +180,14 @@ PathList Dir::children(int types, const std::string& nameFilter) const
         continue;
       }
     }
-    
+
 	std::string utf8File = simgear::strutils::convertWStringToUtf8(fData.cFileName);
     if (fData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
-	  if (types & NO_DOT_OR_DOTDOT) {
-		if ((utf8File == ".") || (utf8File == "..")) {
-		  continue;
-		}
-	  }
+        if (types & NO_DOT_OR_DOTDOT) {
+            if ((utf8File == ".") || (utf8File == "..")) {
+                continue;
+            }
+        }
 
       if (!(types & TYPE_DIR)) {
         continue;
@@ -199,6 +198,14 @@ PathList Dir::children(int types, const std::string& nameFilter) const
 		continue; // always ignore device files
     } else if (!(types & TYPE_FILE)) {
        continue;
+    } else {
+        // regular file
+
+        // skip .foo files even on Windows. We use this in terraSync and other places
+        // to tread .dirindex files as hidden
+        if (!(types & INCLUDE_HIDDEN) && (utf8File.front() == '.')) {
+            continue;
+        }
     }
 
     result.push_back(file(utf8File));
@@ -206,38 +213,38 @@ PathList Dir::children(int types, const std::string& nameFilter) const
 
   FindClose(find);
 #else
-    std::string ps = _path.local8BitStr();
+    std::string ps = _path.utf8Str();
   DIR* dp = opendir(ps.c_str());
   if (!dp) {
     SG_LOG(SG_GENERAL, SG_WARN, "Dir::children: opendir failed:" << _path);
     return result;
   }
-  
+
   int filterLen = nameFilter.size();
-  
+
   while (true) {
     struct dirent* entry = readdir(dp);
     if (!entry) {
       break; // done iteration
     }
-    
+
     // skip hidden files (names beginning with '.') unless requested
     if (!(types & INCLUDE_HIDDEN) && (entry->d_name[0] == '.') &&
          strcmp(entry->d_name, ".") && strcmp(entry->d_name, "..")) {
       continue;
     }
-    
+
     SGPath f = file(entry->d_name);
     if (!f.exists()) {
       continue; // stat() failed
     }
-    
+
     if (f.isDir()) {
       // directory handling
       if (!(types & TYPE_DIR)) {
         continue;
       }
-      
+
       if (types & NO_DOT_OR_DOTDOT) {
         if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..")) {
           continue;
@@ -258,17 +265,17 @@ PathList Dir::children(int types, const std::string& nameFilter) const
       if (nameLen < filterLen) {
         continue; // name is shorter than the filter
       }
-    
+
       char* nameSuffix = entry->d_name + (nameLen - filterLen);
       if (strcmp(nameSuffix, nameFilter.c_str())) {
         continue;
       }
     }
-    
+
   // passed all criteria, add to our result vector
     result.push_back(file(entry->d_name));
   }
-  
+
   closedir(dp);
 #endif
 
@@ -287,16 +294,26 @@ bool Dir::isNull() const
 bool Dir::isEmpty() const
 {
 #if defined(SG_WINDOWS)
+    // PathIsDirectoryEmptyW returns false for a missing / non-directory path,
+    // but our POSIX implementation below returns *true* for a non-existing
+    // directory, so let's be consistent.
+    if (!_path.exists()) {
+        return true;
+    }
+
   std::wstring ps = _path.wstr();
   return PathIsDirectoryEmptyW( ps.c_str() );
 #else
-  std::string ps = _path.local8BitStr();
+  std::string ps = _path.utf8Str();
   DIR* dp = opendir( ps.c_str() );
   if (!dp) return true;
 
   int n = 0;
   dirent* d;
-  while( (d = readdir(dp)) !=NULL && (n < 4) ) n++;
+  while (n < 3 && (d = readdir(dp)) != nullptr) {
+    n++;
+  }
+
   closedir(dp);
 
   return (n == 2); // '.' and '..' always exist
@@ -313,7 +330,7 @@ SGPath Dir::file(const std::string& name) const
   SGPath childPath = _path;
   childPath.set_cached(true);
   childPath.append(name);
-  return childPath;  
+  return childPath;
 }
 
 bool Dir::create(mode_t mode)
@@ -321,7 +338,7 @@ bool Dir::create(mode_t mode)
     if (exists()) {
         return false; // already exists
     }
-    
+
 // recursively create parent directories
     Dir pr(parent());
     if (!pr.path().isNull() && !pr.exists()) {
@@ -330,13 +347,13 @@ bool Dir::create(mode_t mode)
             return false;
         }
     }
-    
+
 // finally, create ourselves
 #if defined(SG_WINDOWS)
 	std::wstring ps = _path.wstr();
 	int err = _wmkdir(ps.c_str());
 #else
-    std::string ps = _path.local8BitStr();
+    std::string ps = _path.utf8Str();
     int err = mkdir(ps.c_str(), mode);
 #endif
     if (err) {
@@ -344,7 +361,7 @@ bool Dir::create(mode_t mode)
                "directory creation failed for '" << _path.utf8Str() << "': " <<
                simgear::strutils::error_string(errno));
     }
-    
+
     return (err == 0);
 }
 
@@ -356,20 +373,20 @@ bool Dir::removeChildren() const
 
     bool ok;
     PathList cs = children(NO_DOT_OR_DOTDOT | INCLUDE_HIDDEN | TYPE_FILE | TYPE_DIR);
-    BOOST_FOREACH(SGPath path, cs) {
+    for (auto path : cs) {
         if (path.isDir()) {
             Dir childDir(path);
             ok = childDir.remove(true);
         } else {
             ok = path.remove();
         }
-        
+
         if (!ok) {
             SG_LOG(SG_IO, SG_WARN, "failed to remove:" << path);
             return false;
         }
     } // of child iteration
-    
+
     return true;
 }
 
@@ -379,7 +396,7 @@ bool Dir::remove(bool recursive)
         SG_LOG(SG_IO, SG_WARN, "attempt to remove non-existant dir:" << _path);
         return false;
     }
-    
+
     if (recursive) {
         if (!removeChildren()) {
             SG_LOG(SG_IO, SG_WARN, "Dir at:" << _path << " failed to remove children");
@@ -391,7 +408,7 @@ bool Dir::remove(bool recursive)
 	std::wstring ps = _path.wstr();
     int err = _wrmdir(ps.c_str());
 #else
-	std::string ps = _path.local8BitStr();
+	std::string ps = _path.utf8Str();
     int err = rmdir(ps.c_str());
 #endif
     if (err) {
