@@ -21,38 +21,43 @@
 //
 // $Id$
 
-
 #include <simgear/compiler.h>
-
 #include <simgear_config.h>
-#include <simgear/debug/logstream.hxx>
-#include <simgear/misc/strutils.hxx>
-#include <simgear/io/iostreams/sgstream.hxx>
 
-#include <cstring>
-#include <stdio.h>
-#include <sys/stat.h>
-#include <errno.h>
-#include <fstream>
+#include <cerrno>
+#include <cstdio>
 #include <cstdlib>
+#include <cstring>
+#include <fstream>
+#include <utility>
+
+#include <sys/stat.h>
 
 #if !defined(SG_WINDOWS)
 #  include <sys/types.h>
 #  include <utime.h>
-#endif
-
-#if defined(SG_WINDOWS)
+#  include <unistd.h>
+#else
 #  include <direct.h>
 #  include <sys/utime.h>
 #  include <Shlwapi.h>
 #endif
 
-#include "sg_path.hxx"
+#include <simgear/debug/logstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/sg_dir.hxx>
+#include <simgear/misc/strutils.hxx>
+
+#include "sg_path.hxx"
 
 using std::string;
 using simgear::strutils::starts_with;
 
+// Static data members used by SGPath::addAllowedPathPattern(),
+// SGPath::validate(), etc.
+string_list SGPath::read_allowed_paths;
+string_list SGPath::write_allowed_paths;
+
 /**
  * define directory path separators
  */
@@ -279,6 +284,128 @@ void SGPath::set_cached(bool cached)
   _cached = false;
 }
 
+// ***************************************************************************
+// *                    Access permissions for Nasal code                    *
+// ***************************************************************************
+
+// Static member function
+SGPath::Permissions SGPath::NasalIORulesChecker(const SGPath& path)
+{
+    Permissions perm;
+
+    if (!path.isAbsolute()) {
+        // SGPath caches permissions, which breaks for relative paths if the
+        // current directory changes.
+        SG_LOG(SG_NASAL, SG_ALERT,
+               "SGPath::NasalIORulesChecker(): file operation on '" <<
+               path.utf8Str() << "': access denied (relative paths not "
+               "accepted; use realpath() to obtain an absolute path)");
+    }
+
+    perm.read  = path.isAbsolute() && !path.validate(false).isNull();
+    perm.write = path.isAbsolute() && !path.validate(true).isNull();
+
+    return perm;
+}
+
+// Static member function
+void SGPath::clearListOfAllowedPaths(bool write)
+{
+    string_list& allowed_paths(write ? write_allowed_paths : read_allowed_paths);
+    allowed_paths.clear();
+}
+
+// Static member function
+void SGPath::addAllowedPathPattern(const string& pattern, bool write)
+{
+    string_list& allowed_paths(write ? write_allowed_paths : read_allowed_paths);
+    allowed_paths.push_back(pattern);
+}
+
+// Static member function
+[[nodiscard]]
+bool SGPath::addAllowedPath(const std::string& path, const Permissions& perms)
+{
+    // Normalizes \ to / on Windows, removes any trailing '/', etc.
+    const string normed_path = SGPath(path).realpath().utf8Str();
+
+    if (normed_path.find("*") == string::npos) {
+        if (perms.read) {
+            read_allowed_paths.push_back(normed_path);
+        }
+
+        if (perms.write) {
+            write_allowed_paths.push_back(std::move(normed_path));
+        }
+
+        return true;
+    } else {
+        return false;
+    }
+}
+
+// Static member function
+[[nodiscard]]
+bool SGPath::addAllowedDirectoryHierarchy(const std::string& path,
+                                          const Permissions& perms)
+{
+    const string normed_path = SGPath(path).realpath().utf8Str();
+
+    if (normed_path.find("*") == string::npos) {
+        if (perms.read) {
+            read_allowed_paths.push_back(normed_path);
+            read_allowed_paths.push_back(normed_path + "/*");
+        }
+
+        if (perms.write) {
+            write_allowed_paths.push_back(normed_path);
+            write_allowed_paths.push_back(normed_path + "/*");
+        }
+
+        return true;
+    } else {
+        return false;
+    }
+}
+
+// Static member function
+const string_list& SGPath::getListOfAllowedPaths(bool write)
+{
+    return write ? write_allowed_paths : read_allowed_paths;
+}
+
+SGPath SGPath::validate(bool write) const
+{
+    // Normalize the path (prevents ../../.. or symlink trickery)
+    const string normed_path = realpath().utf8Str();
+
+    const string_list& allowed_paths{
+        write ? write_allowed_paths : read_allowed_paths};
+    string::size_type star_pos;
+
+    // Check against each allowed pattern
+    for (const auto& path: allowed_paths) {
+        star_pos = path.find('*');
+        if (star_pos == string::npos) {
+            if (!(path.compare(normed_path))) {
+                return fromUtf8(normed_path);
+            }
+        } else {
+            if ((path.size()-1 <= normed_path.size()) /* long enough to be a potential match */
+                && !(path.substr(0, star_pos)
+                     .compare(normed_path.substr(0, star_pos))) /* before-star parts match */
+                && !(path.substr(star_pos+1, path.size()-star_pos-1)
+                     .compare(normed_path.substr(star_pos+1+normed_path.size()-path.size(),
+                                                 path.size()-star_pos-1))) /* after-star parts match */) {
+                return fromUtf8(normed_path);
+            }
+        }
+    }
+
+    // No match found
+    return SGPath();
+}
+
 // append another piece to the existing path
 void SGPath::append( const string& p ) {
     if ( path.empty() ) {
@@ -423,7 +550,7 @@ string SGPath::complete_lower_extension() const
 }
 
 //------------------------------------------------------------------------------
-void SGPath::validate() const
+void SGPath::updateCachedAttributes() const
 {
   if (_cached && _cacheEnabled) {
     return;
@@ -509,7 +636,7 @@ void SGPath::checkAccess() const
     return;
   }
 
-  validate();
+  updateCachedAttributes();
   _rwCached = true;
 }
 
@@ -528,7 +655,7 @@ bool SGPath::exists() const
     return _exists;
   }
 #endif
-  validate();
+  updateCachedAttributes();
   return _exists;
 }
 
@@ -548,13 +675,13 @@ bool SGPath::canWrite() const
 
 bool SGPath::isDir() const
 {
-  validate();
+  updateCachedAttributes();
   return _exists && _isDir;
 }
 
 bool SGPath::isFile() const
 {
-  validate();
+  updateCachedAttributes();
   return _exists && _isFile;
 }
 
@@ -746,13 +873,13 @@ bool SGPath::remove()
 
 time_t SGPath::modTime() const
 {
-    validate();
+    updateCachedAttributes();
     return _modTime;
 }
 
 size_t SGPath::sizeInBytes() const
 {
-    validate();
+    updateCachedAttributes();
     return _size;
 }
 
@@ -993,7 +1120,7 @@ SGPath SGPath::realpath() const
     char* buf = ::realpath(path.c_str(), NULL);
 #endif
     if (!buf) // File does not exist: return the realpath it would have if created now
-              // (needed for fgValidatePath security)
+              // (needed for SGPath::validate() security)
     {
         if (path.empty()) {
             return simgear::Dir::current().path();
@@ -1110,3 +1237,15 @@ bool SGPath::touch()
     _cached = false;
     return true;
 }
+
+bool SGPath::makeLink(const std::string& destination)
+{
+    #ifdef SG_WINDOWS
+    return false;
+    #else
+    if (symlink(destination.c_str(), c_str())) {
+        return false;
+    }
+    return true;
+    #endif
+}
