@@ -37,7 +37,7 @@
 #include <simgear/package/md5.h>
 #include <simgear/compiler.h>   // SG_WINDOWS
 #include <simgear/structure/exception.hxx>
-
+#include <simgear/math/SGGeod.hxx>
 
 #if defined(SG_WINDOWS)
 	#include <windows.h>
@@ -267,6 +267,38 @@ namespace simgear {
 	    return do_strip( s, BOTHSTRIP );
 	}
 
+    string makeStringSafeForPropertyName(const std::string& str)
+    {
+        // This function replaces all characters in 'str' that are not
+        // alphanumeric or '-'.
+        // TODO: make the function multibyte safe.
+        string res = str;
+
+        int index = 0;
+        for (char& c : res) {
+            if (!std::isalpha(c) && !std::isdigit(c) && c != '-') {
+                switch (c) {
+                case ' ':
+                case '\t':
+                case '\n':
+                case '\r':
+                case '_':
+                case '.':
+                case '/':
+                case '\\':
+                    res[index] = '-';
+                    break;
+                default:
+                    res[index] = '_';
+                    SG_LOG(SG_GENERAL, SG_WARN, "makeStringSafeForPropertyName: Modified '" << str << "' to '" << res << "'");
+                }
+            }
+            index++;
+        }
+
+        return res;
+    }
+
         void
         stripTrailingNewlines_inplace(string& s)
         {
@@ -587,6 +619,40 @@ namespace simgear {
         // reached end - longer wins
         return v1parts.size() - v2parts.size();
     }
+    
+    bool compareVersionToWildcard(const std::string& aVersion, const std::string& aCandidate)
+    {
+        if (aCandidate == aVersion) {
+            return true;
+        }
+
+        // examine each dot-seperated component in turn, supporting a wildcard
+        // in the versions from the catalog.
+        string_list parts = split(aVersion, ".");
+        string_list candidateParts = split(aCandidate, ".");
+
+        const size_t partCount = parts.size();
+        const size_t candidatePartCount =  candidateParts.size();
+        
+        bool previousCandidatePartWasWildcard = false;
+
+        for (unsigned int p=0; p < partCount; ++p) {
+            // candidate string is too short, can match if it ended with wildcard
+            // part. This allows candidate '2016.*' to match '2016.1.2' and so on
+            if (candidatePartCount <= p) {
+                return previousCandidatePartWasWildcard;
+            }
+
+            if (candidateParts.at(p) == "*") {
+                // always passes
+                previousCandidatePartWasWildcard = true;
+            } else if (parts.at(p) != candidateParts.at(p)) {
+                return false;
+            }
+        }
+
+        return true;
+    }
 
     string join(const string_list& l, const string& joinWith)
     {
@@ -602,27 +668,34 @@ namespace simgear {
         return result;
     }
 
-    string uppercase(const string &s) {
-      string rslt(s);
-      for(string::iterator p = rslt.begin(); p != rslt.end(); p++){
-        *p = toupper(*p);
-      }
-      return rslt;
+    string uppercase(const string& s)
+    {
+        string result;
+        std::transform(s.begin(), s.end(), std::back_inserter(result), ::toupper);
+        return result;
     }
 
-    string lowercase(const string &s) {
-      string rslt(s);
-      for(string::iterator p = rslt.begin(); p != rslt.end(); p++){
-        *p = tolower(*p);
-      }
-      return rslt;
+    string lowercase(const string& s)
+    {
+        string result;
+        std::transform(s.begin(), s.end(), std::back_inserter(result), ::tolower);
+        return result;
     }
 
-    void lowercase(string &s) {
-      for(string::iterator p = s.begin(); p != s.end(); p++){
-        *p = tolower(*p);
-      }
+bool iequals(const std::string& a, const std::string& b)
+{
+    const auto lenA = a.length();
+    const auto lenB = b.length();
+    if (lenA != lenB) return false;
+    
+    const char* aPtr = a.data();
+    const char* bPtr = b.data();
+    for (size_t i = 0; i < lenA; ++i) {
+        if (tolower(*aPtr++) != tolower(*bPtr++)) return false;
     }
+    
+    return true;
+}
 
 #if defined(SG_WINDOWS)
 static std::wstring convertMultiByteToWString(DWORD encoding, const std::string& a)
@@ -656,8 +729,7 @@ static std::string convertWStringToMultiByte(DWORD encoding, const std::wstring&
 std::wstring convertUtf8ToWString(const std::string& a)
 {
 #if defined(SG_WINDOWS)
-    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> ucs2conv;
-    return ucs2conv.from_bytes(a);
+    return convertMultiByteToWString(CP_UTF8, a);
 #else
     assert(sizeof(wchar_t) == 4);
     std::wstring result;
@@ -707,8 +779,7 @@ std::wstring convertUtf8ToWString(const std::string& a)
 std::string convertWStringToUtf8(const std::wstring& w)
 {
 #if defined(SG_WINDOWS)
-    std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> ucs2conv;
-    return ucs2conv.to_bytes(w);
+    return convertWStringToMultiByte(CP_UTF8, w);
 #else
     assert(sizeof(wchar_t) == 4);
     std::string result;
@@ -883,6 +954,69 @@ std::string encodeHex(const unsigned char* rawBytes, unsigned int length)
   return hex;
 }
 
+std::vector<uint8_t> decodeHex(const std::string& input)
+{
+    std::vector<uint8_t> result;
+    char* ptr = const_cast<char*>(input.data());
+    const char* end = ptr + input.length();
+    
+    bool highNibble = true;
+    uint8_t b = 0;
+    
+    while (ptr != end) {
+        const char c = *ptr;
+        char val = 0;
+        
+        if (c == '0') {
+            val = 0;
+            if ((ptr + 1) < end) {
+                const auto peek = *(ptr + 1);
+                if (peek == 'x') {
+                    // tolerate 0x prefixing
+                    highNibble = true;
+                    ptr += 2; // skip both bytes
+                    continue;
+                }
+            }
+        } else if (isdigit(c)) {
+            val =  c - '0';
+        } else if ((c >= 'A') && (c <= 'F')) {
+            val = c - 'A' + 10;
+        } else if ((c >= 'a') && (c <= 'f')) {
+            val = c - 'a' + 10;
+        } else {
+            // any other input: newline, space, tab, comma...
+            if (!highNibble) {
+                // allow a single digit to work, if we have spacing
+                highNibble = true;
+                result.push_back(b >> 4);
+            }
+            
+            ++ptr;
+            continue;
+        }
+        
+        if (highNibble) {
+            highNibble = false;
+            b = val << 4;
+        } else {
+            highNibble = true;
+            b |= val;
+            result.push_back(b);
+        }
+        
+        ++ptr;
+    }
+    
+    // watch for trailing single digit
+    // this is reqquired so a stirng ending in 0x3 is decoded.
+    if (!highNibble) {
+        result.push_back(b >> 4);
+    }
+    
+    return result;
+}
+    
 // Write an octal backslash-escaped respresentation of 'val' to 'buf'.
 //
 // At least 4 write positions must be available at 'buf'. The result is *not*
@@ -991,6 +1125,16 @@ std::string unescape(const char* s)
   }
   return r;
 }
+std::string replace(std::string source, const std::string search, const std::string replacement, std::size_t start_pos)
+{
+    if (start_pos < source.length()) {
+        while ((start_pos = source.find(search, start_pos)) != std::string::npos) {
+            source.replace(start_pos, search.length(), replacement);
+            start_pos += replacement.length();
+        }
+    }
+    return source;
+}
 
 string sanitizePrintfFormat(const string& input)
 {
@@ -1017,7 +1161,7 @@ std::string error_string(int errnum)
   retcode = strerror_s(buf, sizeof(buf), errnum);
 #elif defined(_GNU_SOURCE)
   return std::string(strerror_r(errnum, buf, sizeof(buf)));
-#elif (_POSIX_C_SOURCE >= 200112L) || defined(SG_MAC) || defined(__FreeBSD__)
+#elif (_POSIX_C_SOURCE >= 200112L) || defined(SG_MAC) || defined(__FreeBSD__) || defined(__OpenBSD__)
   int retcode;
   // POSIX.1-2001 and POSIX.1-2008
   retcode = strerror_r(errnum, buf, sizeof(buf));
@@ -1059,6 +1203,13 @@ bool to_bool(const std::string& s)
     return false;
 }
 
+bool is_bool(const std::string& s)
+{
+    const string_list strings = {"yes", "no", "true", "false", "1", "0"};
+    auto it = std::find(strings.begin(), strings.end(), strutils::lowercase(s));
+    return it != strings.end();
+}
+
 enum PropMatchState
 {
     MATCH_LITERAL = 0,
@@ -1133,6 +1284,356 @@ bool matchPropPathToTemplate(const std::string& path, const std::string& templat
 
     // unreachable
 }
+ 
+bool parseStringAsLatLonValue(const std::string& s, double& degrees)
+{
+    try {
+            string ss = simplify(s);
+            auto spacePos = ss.find_first_of(" *");
+        
+            if (spacePos == std::string::npos) {
+                degrees = std::stod(ss);
+            } else {
+                degrees = std::stod(ss.substr(0, spacePos));
+                
+                double minutes = 0.0, seconds = 0.0;
+                
+                // check for minutes marker
+                auto quotePos = ss.find('\'');
+                if (quotePos == std::string::npos) {
+                    const auto minutesStr = ss.substr(spacePos+1);
+                    if (!minutesStr.empty()) {
+                        minutes = std::stod(minutesStr);
+                    }
+                } else {
+                    minutes = std::stod(ss.substr(spacePos+1, quotePos - spacePos));
+                    const auto secondsStr = ss.substr(quotePos+1);
+                    if (!secondsStr.empty()) {
+                        seconds = std::stod(secondsStr);
+                    }
+                }
+                
+                if ((seconds < 0.0) || (minutes < 0.0)) {
+                    // don't allow sign information in minutes or seconds
+                    return false;
+                }
+                
+                double offset = (minutes / 60.0) + (seconds / 3600.0);
+                degrees += (degrees >= 0.0) ? offset : -offset;
+            }
+        
+            // since we simplified, any trailing N/S/E/W must be the last char
+            const char lastChar = ::toupper(ss.back());
+            if ((lastChar == 'W') || (lastChar == 'S')) {
+                degrees = -degrees;
+            }
+    } catch (std::exception&) {
+        // std::stdo can throw
+        return false;
+    }
+    
+    return true;
+}
+        
+namespace {
+    bool isLatString(const std::string &s)
+    {
+        const char lastChar = ::toupper(s.back());
+        return (lastChar == 'N') || (lastChar == 'S');
+    }
+    
+    bool isLonString(const std::string &s)
+    {
+        const char lastChar = ::toupper(s.back());
+        return (lastChar == 'E') || (lastChar == 'W');
+    }
+} // of anonymous namespace
+        
+bool parseStringAsGeod(const std::string& s, SGGeod* result, bool assumeLonLatOrder)
+{
+    if (s.empty())
+        return false;
+    
+    const auto commaPos = s.find(',');
+    if (commaPos == string::npos) {
+        return false;
+    }
+    
+    auto termA = simplify(s.substr(0, commaPos)),
+        termB = simplify(s.substr(commaPos+1));
+    double valueA, valueB;
+    if (!parseStringAsLatLonValue(termA, valueA) || !parseStringAsLatLonValue(termB, valueB)) {
+        return false;
+    }
+    
+    if (result) {
+        // explicit ordering
+        if (isLatString(termA) && isLonString(termB)) {
+            *result = SGGeod::fromDeg(valueB, valueA);
+        } else if (isLonString(termA) && isLatString(termB)) {
+            *result = SGGeod::fromDeg(valueA, valueB);
+        } else {
+            // implicit ordering
+            // SGGeod wants longitude, latitude
+            *result = assumeLonLatOrder ? SGGeod::fromDeg(valueA, valueB)
+                                        : SGGeod::fromDeg(valueB, valueA);
+        }
+    }
+    
+    return true;
+}
+        
+namespace {
+    const char* static_degreeSymbols[] = {
+        "*",
+        " ",
+        "\xB0",     // Latin-1 B0 codepoint
+        "\xC2\xB0"  // UTF-8 equivalent
+    };
+} // of anonymous namespace
+        
+std::string formatLatLonValueAsString(double deg, LatLonFormat format,
+                                      char c,
+                                      DegreeSymbol degreeSymbol)
+{
+    double min, sec;
+    const int sign = deg < 0.0 ? -1 : 1;
+    deg = fabs(deg);
+    char buf[128];
+    const char* degSym = static_degreeSymbols[static_cast<int>(degreeSymbol)];
+    
+    switch (format) {
+    case LatLonFormat::DECIMAL_DEGREES:
+        ::snprintf(buf, sizeof(buf), "%3.6f%c", deg, c);
+        break;
+        
+    case LatLonFormat::DEGREES_MINUTES:
+        // d mm.mmm' (DMM format) -- uses a round-off factor tailored to the
+        // required precision of the minutes field (three decimal places),
+        // preventing minute values of 60.
+        min = (deg - int(deg)) * 60.0;
+        if (min >= 59.9995) {
+            min -= 60.0;
+            deg += 1.0;
+        }
+        snprintf(buf, sizeof(buf), "%d%s%06.3f'%c", int(deg), degSym, fabs(min), c);
+        break;
+        
+    case LatLonFormat::DEGREES_MINUTES_SECONDS:
+        // d mm'ss.s" (DMS format) -- uses a round-off factor tailored to the
+        // required precision of the seconds field (one decimal place),
+        // preventing second values of 60.
+        min = (deg - int(deg)) * 60.0;
+        sec = (min - int(min)) * 60.0;
+        if (sec >= 59.95) {
+            sec -= 60.0;
+            min += 1.0;
+            if (min >= 60.0) {
+                min -= 60.0;
+                deg += 1.0;
+            }
+        }
+        ::snprintf(buf, sizeof(buf), "%d%s%02d'%04.1f\"%c", int(deg), degSym,
+                   int(min), fabs(sec), c);
+        break;
+            
+    case LatLonFormat::SIGNED_DECIMAL_DEGREES:
+        // d.dddddd' (signed DDD format).
+        ::snprintf(buf, sizeof(buf), "%3.6f", sign*deg);
+        break;
+    
+    case LatLonFormat::SIGNED_DEGREES_MINUTES:
+        // d mm.mmm' (signed DMM format).
+        min = (deg - int(deg)) * 60.0;
+        if (min >= 59.9995) {
+            min -= 60.0;
+            deg += 1.0;
+        }
+        if (sign == 1) {
+            snprintf(buf, sizeof(buf), "%d%s%06.3f'", int(deg), degSym, fabs(min));
+        } else {
+            snprintf(buf, sizeof(buf), "-%d%s%06.3f'", int(deg), degSym, fabs(min));
+        }
+        break;
+        
+            
+    case LatLonFormat::SIGNED_DEGREES_MINUTES_SECONDS:
+        // d mm'ss.s" (signed DMS format).
+        min = (deg - int(deg)) * 60.0;
+        sec = (min - int(min)) * 60.0;
+        if (sec >= 59.95) {
+            sec -= 60.0;
+            min += 1.0;
+            if (min >= 60.0) {
+                min -= 60.0;
+                deg += 1.0;
+            }
+        }
+        if (sign == 1) {
+            snprintf(buf, sizeof(buf), "%d%s%02d'%04.1f\"", int(deg), degSym, int(min), fabs(sec));
+        } else {
+            snprintf(buf, sizeof(buf), "-%d%s%02d'%04.1f\"", int(deg), degSym, int(min), fabs(sec));
+        }
+        break;
+            
+    case LatLonFormat::ZERO_PAD_DECIMAL_DEGRESS:
+        // dd.dddddd X, ddd.dddddd X (zero padded DDD format).
+        if (c == 'N' || c == 'S') {
+            snprintf(buf, sizeof(buf), "%09.6f%c", deg, c);
+        } else {
+            snprintf(buf, sizeof(buf), "%010.6f%c", deg, c);
+        }
+        break;
+            
+    case LatLonFormat::ZERO_PAD_DEGREES_MINUTES:
+        // dd mm.mmm' X, ddd mm.mmm' X (zero padded DMM format).
+        min = (deg - int(deg)) * 60.0;
+        if (min >= 59.9995) {
+            min -= 60.0;
+            deg += 1.0;
+        }
+        if (c == 'N' || c == 'S') {
+            snprintf(buf, sizeof(buf), "%02d%s%06.3f'%c", int(deg), degSym, fabs(min), c);
+        } else {
+            snprintf(buf, sizeof(buf), "%03d%s%06.3f'%c", int(deg), degSym, fabs(min), c);
+        }
+        break;
+            
+    case LatLonFormat::ZERO_PAD_DEGREES_MINUTES_SECONDS:
+        // dd mm'ss.s" X, dd mm'ss.s" X (zero padded DMS format).
+        min = (deg - int(deg)) * 60.0;
+        sec = (min - int(min)) * 60.0;
+        if (sec >= 59.95) {
+            sec -= 60.0;
+            min += 1.0;
+            if (min >= 60.0) {
+                min -= 60.0;
+                deg += 1.0;
+            }
+        }
+        if (c == 'N' || c == 'S') {
+            snprintf(buf, sizeof(buf), "%02d%s%02d'%04.1f\"%c", int(deg), degSym, int(min), fabs(sec), c);
+        } else {
+            snprintf(buf, sizeof(buf), "%03d%s%02d'%04.1f\"%c", int(deg), degSym, int(min), fabs(sec), c);
+        }
+        break;
+            
+    case LatLonFormat::TRINITY_HOUSE:
+        // dd* mm'.mmm X, ddd* mm'.mmm X (Trinity House Navigation standard).
+        min = (deg - int(deg)) * 60.0;
+        if (min >= 59.9995) {
+            min -= 60.0;
+            deg += 1.0;
+        }
+        if (c == 'N' || c == 'S') {
+            snprintf(buf, sizeof(buf), "%02d* %02d'.%03d%c", int(deg), int(min), int(SGMisc<double>::round((min-int(min))*1000)), c);
+        } else {
+            snprintf(buf, sizeof(buf), "%03d* %02d'.%03d%c", int(deg), int(min), int(SGMisc<double>::round((min-int(min))*1000)), c);
+        }
+        break;
+            
+    case LatLonFormat::DECIMAL_DEGREES_SYMBOL:
+        ::snprintf(buf, sizeof(buf), "%3.6f%s%c", deg, degSym, c);
+        break;
+            
+    case LatLonFormat::ICAO_ROUTE_DEGREES:
+    {
+        min = (deg - int(deg)) * 60.0;
+        if (min >= 59.9995) {
+            min -= 60.0;
+            deg += 1.0;
+        }
+        
+        if (static_cast<int>(min) == 0) {
+            // 7-digit mode
+            if (c == 'N' || c == 'S') {
+                snprintf(buf, sizeof(buf), "%02d%c", int(deg), c);
+            } else {
+                snprintf(buf, sizeof(buf), "%03d%c", int(deg), c);
+            }
+        } else {
+            // 11-digit mode
+            if (c == 'N' || c == 'S') {
+                snprintf(buf, sizeof(buf), "%02d%02d%c", int(deg), int(min), c);
+            } else {
+                snprintf(buf, sizeof(buf), "%03d%02d%c", int(deg), int(min), c);
+            }
+        }
+        break;
+    }
+            
+    default:
+        break;
+    }
+ 
+    return std::string(buf);
+}
+
+std::string formatGeodAsString(const SGGeod& geod, LatLonFormat format,
+                               DegreeSymbol degreeSymbol)
+{
+    const char ns = (geod.getLatitudeDeg() > 0.0) ? 'N' : 'S';
+    const char ew = (geod.getLongitudeDeg() > 0.0) ? 'E' : 'W';
+    
+    // no comma seperator
+    if (format == LatLonFormat::ICAO_ROUTE_DEGREES) {
+        return formatLatLonValueAsString(geod.getLatitudeDeg(), format, ns, degreeSymbol) +
+            formatLatLonValueAsString(geod.getLongitudeDeg(), format, ew, degreeSymbol);
+    }
+    
+    return formatLatLonValueAsString(geod.getLatitudeDeg(), format, ns, degreeSymbol) + ","
+        + formatLatLonValueAsString(geod.getLongitudeDeg(), format, ew, degreeSymbol);
+}
+
+PrintfFormatType validatePrintfFormat(const char* f)
+{
+    bool l = false;
+    for (; *f; f++) {
+        if (*f == '%') {
+            if (f[1] == '%')
+                f++;
+            else
+                break;
+        }
+    }
+    if (*f++ != '%')
+        return PrintfFormatType::Invalid;
+
+    PrintfFormatType type = PrintfFormatType::Invalid;
+    while (*f == ' ' || *f == '+' || *f == '-' || *f == '#' || *f == '0')
+        f++;
+    while (*f && isdigit(*f))
+        f++;
+    if (*f == '.') {
+        f++;
+        while (*f && isdigit(*f))
+            f++;
+    }
+
+    if (*f == 'l')
+        l = true, f++;
+
+    if (*f == 'd') {
+        type = l ? PrintfFormatType::Long : PrintfFormatType::Int;
+    } else if (*f == 'f')
+        type = l ? PrintfFormatType::Double : PrintfFormatType::Float;
+    else if (*f == 's') {
+        if (l)
+            return PrintfFormatType::Invalid;
+        type = PrintfFormatType::CharPointer;
+    } else
+        return PrintfFormatType::Invalid;
+
+    for (++f; *f; f++) {
+        if (*f == '%') {
+            if (f[1] == '%')
+                f++;
+            else
+                return PrintfFormatType::Invalid;
+        }
+    }
+    return type;
+}
 
 } // end namespace strutils
 
