@@ -1,13 +1,18 @@
 #include "parse.h"
 
+#include <stdio.h>
+
 // Static table of recognized lexemes in the language
 static const struct Lexeme {
     char* str;
     int   tok;
 } LEXEMES[] = {
     {"and", TOK_AND},
-    {"or",  TOK_OR},
-    {"!",   TOK_NOT},
+    {"or", TOK_OR},
+    {"!", TOK_NOT},
+    {"&", TOK_BIT_AND},
+    {"|", TOK_BIT_OR},
+    {"^", TOK_BIT_XOR},
     {"(", TOK_LPAR},
     {")", TOK_RPAR},
     {"[", TOK_LBRA},
@@ -24,21 +29,23 @@ static const struct Lexeme {
     {",", TOK_COMMA},
     {";", TOK_SEMI},
     {"=", TOK_ASSIGN},
-    {"<",  TOK_LT},
+    {"<", TOK_LT},
     {"<=", TOK_LTE},
     {"==", TOK_EQ},
     {"!=", TOK_NEQ},
-    {">",  TOK_GT},
+    {">", TOK_GT},
     {">=", TOK_GTE},
     {"nil", TOK_NIL},
-    {"if",    TOK_IF},
+    {"true", TOK_TRUE},
+    {"false", TOK_FALSE},
+    {"if", TOK_IF},
     {"elsif", TOK_ELSIF},
-    {"else",  TOK_ELSE},
-    {"for",     TOK_FOR},
+    {"else", TOK_ELSE},
+    {"for", TOK_FOR},
     {"foreach", TOK_FOREACH},
-    {"while",   TOK_WHILE},
-    {"return",   TOK_RETURN},
-    {"break",    TOK_BREAK},
+    {"while", TOK_WHILE},
+    {"return", TOK_RETURN},
+    {"break", TOK_BREAK},
     {"continue", TOK_CONTINUE},
     {"func", TOK_FUNC},
     {"...", TOK_ELLIPSIS},
@@ -49,6 +56,11 @@ static const struct Lexeme {
     {"*=", TOK_MULEQ},
     {"/=", TOK_DIVEQ},
     {"~=", TOK_CATEQ},
+    {"&=", TOK_BIT_ANDEQ},
+    {"|=", TOK_BIT_OREQ},
+    {"^=", TOK_BIT_XOREQ},
+    {"??", TOK_NULL_CHAIN},
+    {"?.", TOK_NULL_ACCESS},
     {"forindex", TOK_FORINDEX},
 };
 
@@ -124,6 +136,16 @@ static void newToken(struct Parser* p, int pos, int type,
         }
     }
 
+    // Treat boolean constants as numeric literals
+    if(type == TOK_TRUE) {
+        type = TOK_LITERAL;
+        num = 1.0;
+    }
+    if(type == TOK_FALSE) {
+        type = TOK_LITERAL;
+        num = 0.0;
+    }
+
     tok = naParseAlloc(p, sizeof(struct Token));
     tok->type = type;
     tok->line = getLine(p, pos);
@@ -136,13 +158,14 @@ static void newToken(struct Parser* p, int pos, int type,
     tok->lastChild = 0;
     tok->rule = 0;
     
-    // Context sensitivity hack: a "-" following a binary operator of
+    // Context sensitivity hack: a "-" or "~" following a binary operator of
     // equal or higher precedence must be a unary negation.  Needed to
     // get precedence right in the parser for expressiong like "a * -2"
-    if(type == TOK_MINUS && tok->prev) {
+    if((type == TOK_MINUS || type == TOK_CAT) && tok->prev) {
         int pt = tok->prev->type;
-        if(pt==TOK_PLUS||pt==TOK_MINUS||pt==TOK_CAT||pt==TOK_MUL||pt==TOK_DIV)
-            tok->type = type = TOK_NEG;
+        if( pt==TOK_PLUS||pt==TOK_MINUS||pt==TOK_CAT||pt==TOK_MUL||pt==TOK_DIV
+         || pt==TOK_BIT_AND||pt==TOK_BIT_OR||pt==TOK_BIT_XOR )
+            tok->type = type = (type == TOK_MINUS ? TOK_NEG : TOK_BIT_NEG);
     }
 
     if(!p->tree.children) p->tree.children = tok;
@@ -245,12 +268,12 @@ static int lexStringLiteral(struct Parser* p, int index, char q)
     return i+1;
 }
 
-static int lexHexLiteral(struct Parser* p, int index)
+static int lexIntLiteral(struct Parser* p, int index, int base)
 {
     int nib, i = index;
     double d = 0;
-    while(i < p->len && (nib = hex(p->buf[i])) >= 0) {
-        d = d*16 + nib;
+    while(i < p->len && (nib = hex(p->buf[i])) >= 0 && nib < base) {
+        d = d * base + nib;
         i++;
     }
     newToken(p, index, TOK_LITERAL, 0, 0, d);
@@ -266,8 +289,12 @@ static int lexNumLiteral(struct Parser* p, int index)
     unsigned char* buf = (unsigned char*)p->buf;
     double d;
 
-    if(buf[i] == '0' && i+2<len && buf[i+1] == 'x' && ISHEX(buf[i+2]))
-       return lexHexLiteral(p, index+2);
+    if( buf[i] == '0' && i + 2 < len ) {
+        if( buf[i+1] == 'x' && ISHEX(buf[i+2]) )
+            return lexIntLiteral(p, index+2, 16);
+        if( buf[i+1] == 'o' && ISNUM(buf[i+2]) )
+            return lexIntLiteral(p, index+2, 8);
+    }
 
     while(i<len && ISNUM(buf[i])) i++;
     if(i<len && buf[i] == '.') {
@@ -380,6 +407,21 @@ void naLex(struct Parser* p)
                 newToken(p, i, TOK_SYMBOL, p->buf+i, symlen, 0);
                 i += symlen;
             } else {
+                //const char* line_begin = p->buf;
+                int line = 1;
+                int column = 0;
+                for (int j=0; j<i; ++j) {
+                    if (p->buf[j] == '\n') {
+                        line += 1;
+                        column = 0;
+                        //line_begin = p->buf + j + 1;
+                    }
+                    else {
+                        column += 1;
+                    }
+                }
+                fprintf(stderr, "%s:%i: illegal character 0x%x at line=%i column=%i.\n",
+                        __FILE__, __LINE__, c, line, column);
                 error(p, "illegal character", i);
             }
         }
