@@ -47,6 +47,39 @@ static naRef f_append(naContext c, naRef me, int argc, naRef* args)
     return args[0];
 }
 
+static naRef f_vec_remove(naContext c, naRef me, int argc, naRef* args)
+{
+    if (argc < 2 || !naIsVector(args[0])) ARGERR();
+    naRef vec = args[0];
+    naRef value = args[1];
+    if (naIsNil(vec) || naIsNil(value)) ARGERR();
+
+    int len = naVec_size(vec);
+    int i;
+    for (i = 0; i < len; i++) {
+        if (naEqual(naVec_get(vec, i), value)) {
+            naVec_remove(vec, i);
+            --len;
+        }
+    }
+
+    return args[0]; // return the vec to allow chaining
+}
+
+
+static naRef f_vec_removeat(naContext c, naRef me, int argc, naRef* args)
+{
+    if (argc < 2 || !naIsVector(args[0])) ARGERR();
+    naRef vec = args[0];
+    int index = (int)naNumValue(args[1]).num;
+    if (naIsNil(vec)) ARGERR();
+
+    int len = naVec_size(vec);
+    if ((index < 0) || (index >= len)) ARGERR();
+
+    return naVec_remove(vec, index);
+}
+
 static naRef f_pop(naContext c, naRef me, int argc, naRef* args)
 {
     if(argc < 1 || !naIsVector(args[0])) ARGERR();
@@ -99,11 +132,33 @@ static naRef f_int(naContext c, naRef me, int argc, naRef* args)
     return naNil();
 }
 
+static naRef f_isint(naContext c, naRef me, int argc, naRef* args)
+{
+    if(argc > 0) {
+        naRef n = naNumValue(args[0]);
+        if(naIsNil(n)) return naNum(0);
+        if(n.num < 0) n.num = -n.num;
+        if(n.num == floor(n.num)) return naNum(1);
+        else return naNum(0);
+    } else ARGERR();
+    return naNil();
+}
+
 static naRef f_num(naContext c, naRef me, int argc, naRef* args)
 {
     return argc > 0 ? naNumValue(args[0]) : naNil();
 }
 
+static naRef f_isnum(naContext c, naRef me, int argc, naRef* args)
+{
+    if(argc > 0) {
+        naRef n = naNumValue(args[0]);
+        if(naIsNil(n)) return naNum(0);
+        else return naNum(1);
+    } else ARGERR();
+    return naNil();
+}
+
 static naRef f_streq(naContext c, naRef me, int argc, naRef* args)
 {
     return argc > 1 ? naNum(naStrEqual(args[0], args[1])) : naNil();
@@ -126,6 +181,14 @@ static naRef f_cmp(naContext c, naRef me, int argc, naRef* args)
     return naNum(alen == blen ? 0 : (alen < blen ? -1 : 1));
 }
 
+static naRef f_str(naContext c, naRef me, int argc, naRef* args)
+{
+    if (argc < 1)
+        ARGERR();
+    
+    return naStringValue(c, args[0]);
+}
+
 static naRef f_substr(naContext c, naRef me, int argc, naRef* args)
 {
     int start, len, srclen;
@@ -183,11 +246,40 @@ static naRef f_chr(naContext c, naRef me, int argc, naRef* args)
 
 static naRef f_contains(naContext c, naRef me, int argc, naRef* args)
 {
-    naRef hash = argc > 0 ? args[0] : naNil();
+    naRef hashOrVec = argc > 0 ? args[0] : naNil();
     naRef key = argc > 1 ? args[1] : naNil();
-    if(naIsNil(hash) || naIsNil(key)) ARGERR();
-    if(!naIsHash(hash)) return naNil();
-    return naHash_get(hash, key, &key) ? naNum(1) : naNum(0);
+    if (naIsNil(hashOrVec) || naIsNil(key)) ARGERR();
+
+    if (naIsVector(hashOrVec)) {
+        // very similar to vecindex below
+        const int len = naVec_size(hashOrVec);
+        int i;
+        for (i = 0; i < len; i++) {
+            if (naEqual(naVec_get(hashOrVec, i), key))
+                return naNum(1);
+        }
+        return naNum(0); // not found in the vector
+    } else if (naIsHash(hashOrVec)) {
+        return naHash_get(hashOrVec, key, &key) ? naNum(1) : naNum(0);
+    }
+
+    return naNil();
+}
+
+static naRef f_vecindex(naContext c, naRef me, int argc, naRef* args)
+{
+    naRef vec = argc > 0 ? args[0] : naNil();
+    naRef value = argc > 1 ? args[1] : naNil();
+    if(naIsNil(vec) || naIsNil(value)) ARGERR();
+    if(!naIsVector(vec)) return naNil();
+    const int len = naVec_size(vec);
+    int i;
+    for(i=0; i<len; i++) {
+        if (naEqual(naVec_get(vec, i), value))
+            return naNum(i);
+    }
+    
+    return naNil();
 }
 
 static naRef f_typeof(naContext c, naRef me, int argc, naRef* args)
@@ -204,6 +296,48 @@ static naRef f_typeof(naContext c, naRef me, int argc, naRef* args)
     return NEWCSTR(c, t);
 }
 
+static naRef f_isscalar(naContext c, naRef me, int argc, naRef* args)
+{
+    naRef r = argc > 0 ? args[0] : naNil();
+    if(naIsString(r) || naIsNum(r)) return naNum(1);
+    else return naNum(0);
+}
+
+static naRef f_isstr(naContext c, naRef me, int argc, naRef* args)
+{
+    naRef r = argc > 0 ? args[0] : naNil();
+    if(naIsString(r)) return naNum(1);
+    else return naNum(0);
+}
+
+static naRef f_isvec(naContext c, naRef me, int argc, naRef* args)
+{
+    naRef r = argc > 0 ? args[0] : naNil();
+    if(naIsVector(r)) return naNum(1);
+    else return naNum(0);
+}
+
+static naRef f_ishash(naContext c, naRef me, int argc, naRef* args)
+{
+    naRef r = argc > 0 ? args[0] : naNil();
+    if(naIsHash(r)) return naNum(1);
+    else return naNum(0);
+}
+
+static naRef f_isfunc(naContext c, naRef me, int argc, naRef* args)
+{
+    naRef r = argc > 0 ? args[0] : naNil();
+    if(naIsFunc(r)) return naNum(1);
+    else return naNum(0);
+}
+
+static naRef f_isghost(naContext c, naRef me, int argc, naRef* args)
+{
+    naRef r = argc > 0 ? args[0] : naNil();
+    if(naIsGhost(r)) return naNum(1);
+    else return naNum(0);
+}
+
 static naRef f_ghosttype(naContext c, naRef me, int argc, naRef* args)
 {
     naRef g = argc > 0 ? args[0] : naNil();
@@ -211,8 +345,8 @@ static naRef f_ghosttype(naContext c, naRef me, int argc, naRef* args)
     if(naGhost_type(g)->name) {
         return NEWCSTR(c, (char*)naGhost_type(g)->name);
     } else {
-        char buf[32];
-        sprintf(buf, "%p", naGhost_type(g));
+        char buf[128];
+        snprintf(buf, 128, "%p", naGhost_type(g));
         return NEWCSTR(c, buf);
     }
 }
@@ -295,28 +429,32 @@ static naRef f_die(naContext c, naRef me, int argc, naRef* args)
     return naNil(); // never executes
 }
 
-// Wrapper around vsnprintf, iteratively increasing the buffer size
-// until it fits.  Returned buffer should be freed by the caller.
-static char* dosprintf(char* f, ...)
+char* dosprintf(char* f, ...)
 {
     char* buf;
+
     va_list va;
-    int olen, len = 16;
     va_start(va, f);
-    while(1) {
-        buf = naAlloc(len);
-        va_list vaCopy;
-        va_copy(vaCopy, va);
-        olen = vsnprintf(buf, len, f, vaCopy);
-        if(olen >= 0 && olen < len) {
-            va_end(va);
-            va_end(vaCopy);
-            return buf;
-        }
-        va_end(vaCopy);
-        naFree(buf);
-        len *= 2;
+   
+    int len = vsnprintf(0, 0, f, va);
+    va_end(va);
+
+    if (len <= 0) {
+        buf = (char *) naAlloc(2);
+        *buf = 0;
     }
+    else {
+        len++;// allow for terminating null
+        buf = (char *) naAlloc(len);
+
+        va_list va;
+        va_start(va, f);
+
+        len = vsnprintf(buf, len, f, va);
+        va_end(va);
+    }
+
+    return buf;
 }
 
 // Inspects a printf format string f, and finds the next "%..." format
@@ -384,7 +522,7 @@ static naRef f_sprintf(naContext c, naRef me, int argc, naRef* args)
         } else {
             arg = naNumValue(arg);
             if(naIsNil(arg))
-                fout = dosprintf(fstr, "nil");
+                fout = dosprintf("nil");
             else if(t=='d' || t=='i' || t=='c')
                 fout = dosprintf(fstr, (int)naNumValue(arg).num);
             else if(t=='o' || t=='u' || t=='x' || t=='X')
@@ -585,43 +723,123 @@ static naRef f_id(naContext c, naRef me, int argc, naRef* args)
         naGhostType *gt = PTR(args[0]).ghost->gtype;
         t = gt->name ? (char*)gt->name : "ghost";
     }
-    sprintf(buf, "%s:%p", (char*)t, (void*)PTR(args[0]).obj);
+    snprintf(buf, sizeof(buf), "%s:%p", (char*)t, (void*)PTR(args[0]).obj);
     return NEWCSTR(c, buf);
 }
 
+// Python-like range (a vector filled with numbers from start to stop, spaced by step)
+// Synopsis:
+//    range([start, ]stop[, step])
+//        start=0:    Optional. An integer number specifying at which position to start.
+//        stop:        Required. A number specifying at which position to stop (not included)
+//        step=1:        Optional. An integer number greater 0 specifying the incrementation.
+//
+// Examples:
+//        var r = range(5);
+//         debug.dump(r)
+//    prints [0, 1, 2, 3, 4]
+//
+//        var r = range(2, 5);
+//         debug.dump(r)
+//    prints [2, 3, 4]
+//
+//        var r = range(0, 10, 2);
+//         debug.dump(r)
+//    prints [0, 2, 4, 6, 8]
+//
+//        var r = range(0, 0);
+//        debug.dump(r)
+//    prints []
+//
+//        var r = range(0, 1);
+//        debug.dump(r)
+//    prints [0]
+//
+//        var r = range(0, 0.1);
+//        debug.dump(r)
+//    prints [0]
+static naRef f_range(naContext c, naRef me, int argc, naRef* args)
+{
+    int start = 0;
+    int step = 1;
+    int end = 0;
+    naRef out;
+
+    if ((argc < 1) || (argc > 3) || !naIsNum(args[0]))
+        naRuntimeError(c, "Bad/missing argument to range()");
+    
+    if (argc > 1) {
+        if (!naIsNum(args[1])) {
+            naRuntimeError(c, "Invalid argument to range()");
+        }
+
+        start = (int) args[0].num;
+        end = (int) args[1].num;
+
+        if (argc > 2) {
+            if (!naIsNum(args[2])) {
+                naRuntimeError(c, "Invalid argument to range()");
+            }
+
+            step = (int) args[2].num;
+        }
+    } else {
+        // single arg mode
+        end = (int) args[0].num;
+    }
+
+    out = naNewVector(c);
+    for (int i=start; i<end; i+=step) {
+        naVec_append(out, naNum(i));
+    }
+    return out;
+}
+
 static naCFuncItem funcs[] = {
-    { "size", f_size },
-    { "keys", f_keys }, 
-    { "append", f_append }, 
-    { "pop", f_pop }, 
-    { "setsize", f_setsize }, 
-    { "subvec", f_subvec }, 
-    { "delete", f_delete }, 
-    { "int", f_int },
-    { "num", f_num },
-    { "streq", f_streq },
-    { "cmp", f_cmp },
-    { "substr", f_substr },
-    { "left", f_left },
-    { "right", f_right },
-    { "chr", f_chr },
-    { "contains", f_contains },
-    { "typeof", f_typeof },
-    { "ghosttype", f_ghosttype },
-    { "compile", f_compile },
-    { "call", f_call },
-    { "die", f_die },
-    { "sprintf", f_sprintf },
-    { "caller", f_caller },
-    { "closure", f_closure },
-    { "find", f_find },
-    { "split", f_split },
-    { "rand", f_rand },
-    { "bind", f_bind },
-    { "sort", f_sort },
-    { "id", f_id },
-    { 0 }
-};
+    {"size", f_size},
+    {"keys", f_keys},
+    {"append", f_append},
+    {"pop", f_pop},
+    {"setsize", f_setsize},
+    {"subvec", f_subvec},
+    {"vecindex", f_vecindex},
+    {"remove", f_vec_remove},
+    {"removeat", f_vec_removeat},
+    {"delete", f_delete},
+    {"int", f_int},
+    {"num", f_num},
+    {"str", f_str},
+    {"streq", f_streq},
+    {"cmp", f_cmp},
+    {"substr", f_substr},
+    {"left", f_left},
+    {"right", f_right},
+    {"chr", f_chr},
+    {"contains", f_contains},
+    {"typeof", f_typeof},
+    {"ghosttype", f_ghosttype},
+    {"compile", f_compile},
+    {"call", f_call},
+    {"die", f_die},
+    {"sprintf", f_sprintf},
+    {"caller", f_caller},
+    {"closure", f_closure},
+    {"find", f_find},
+    {"split", f_split},
+    {"rand", f_rand},
+    {"bind", f_bind},
+    {"sort", f_sort},
+    {"id", f_id},
+    {"isscalar", f_isscalar},
+    {"isint", f_isint},
+    {"isnum", f_isnum},
+    {"isghost", f_isghost},
+    {"isstr", f_isstr},
+    {"isvec", f_isvec},
+    {"ishash", f_ishash},
+    {"isfunc", f_isfunc},
+    {"range", f_range},
+    {0}};
 
 naRef naInit_std(naContext c)
 {
