@@ -19,10 +19,11 @@
 
 #include <simgear/package/Root.hxx>
 
-#include <boost/foreach.hpp>
+#include <algorithm>
+#include <cassert>
 #include <cstring>
-#include <map>
 #include <deque>
+#include <map>
 #include <set>
 
 #include <simgear/debug/logstream.hxx>
@@ -50,7 +51,7 @@ namespace {
         return strutils::encodeHex(sha1_result(&info), HASH_LENGTH);
     }
 } // of anonymous namespace
-    
+
 namespace pkg {
 
 typedef std::map<std::string, CatalogRef> CatalogDict;
@@ -70,7 +71,7 @@ public:
             m_realUrl = aUrl;
         }
     }
-    
+
     std::string realUrl() const
     {
         return m_realUrl;
@@ -105,7 +106,7 @@ public:
             d->installStatusChanged(install, status);
         }
     }
-    
+
     void fireStartInstall(InstallRef install)
     {
         for (auto d : delegates) {
@@ -132,7 +133,9 @@ public:
 
     void fireRefreshStatus(CatalogRef catalog, Delegate::StatusCode status)
     {
-        for (auto d : delegates) {
+        // take a copy of delegates since firing this can modify the data
+        const auto currentDelegates = delegates;
+        for (auto d : currentDelegates) {
             d->catalogRefreshed(catalog, status);
         }
     }
@@ -151,7 +154,7 @@ public:
         std::string u = dl->realUrl();
         if (status == Delegate::STATUS_SUCCESS) {
             thumbnailCache[u].requestPending = false;
-            
+
             // if this was a network load, rather than a re-load from the disk cache,
             // then persist to disk now.
             if (strutils::starts_with(request->url(), "http")) {
@@ -178,8 +181,7 @@ public:
                 SG_LOG(SG_IO, SG_WARN, "Download failed for: " << u << ", will use old cached data");
                 cachePath.touch(); // touch the file so we don't repeat this danxce
                 // kick a load from the cache
-                
-                queueLoadFromPersistentCache(u, cachePath);
+                loadFromPersistentCache(u, cachePath);
             }
         }
 
@@ -202,6 +204,7 @@ public:
 
         std::string u = pendingThumbnails.front();
         pendingThumbnails.pop_front();
+
         thumbnailDownloadRequest = new Root::ThumbnailDownloader(this, u);
 
         if (http) {
@@ -217,7 +220,7 @@ public:
             d->finishUninstall(pkg);
         }
     }
-    
+
     SGPath pathInCache(const std::string& url) const
     {
         const auto hash = hashForUrl(url);
@@ -226,7 +229,7 @@ public:
         if (pos == std::string::npos) {
             return SGPath();
         }
-        
+
         return path / "ThumbnailCache" / (hash + url.substr(pos));
     }
 
@@ -243,14 +246,14 @@ public:
         assert(it != thumbnailCache.end());
         it->second.pathOnDisk = cachePath;
     }
-    
+
     bool checkPersistentCache(const std::string& url)
     {
         SGPath cachePath = pathInCache(url);
         if (!cachePath.exists()) {
             return false;
         }
-        
+
         // check age, if it's too old, expire and download again
         int age = time(nullptr) - cachePath.modTime();
         const int cacheMaxAge = SECONDS_PER_DAY * 7;
@@ -259,15 +262,15 @@ public:
             // cache refresh might fail
             return false;
         }
-        
-        queueLoadFromPersistentCache(url, cachePath);
+
+        loadFromPersistentCache(url, cachePath);
         return true;
     }
-    
-    void queueLoadFromPersistentCache(const std::string& url, const SGPath& path)
+
+    void loadFromPersistentCache(const std::string& url, const SGPath& path)
     {
         assert(path.exists());
-        
+
         auto it = thumbnailCache.find(url);
         if (it == thumbnailCache.end()) {
             ThumbnailCacheEntry entry;
@@ -276,20 +279,38 @@ public:
         } else {
             assert(it->second.pathOnDisk.isNull() || (it->second.pathOnDisk == path));
         }
-        
-        if (it->second.requestPending) {
-            return; // all done
-        }
-        
-        it->second.requestPending = true;
-        auto dl = new Root::ThumbnailDownloader(this, path.fileUrl(), url);
-        if (http) {
-            http->makeRequest(dl);
-        } else {
-            httpPendingRequests.push_back(dl);
-        }
+
+        sg_ifstream thumbnailStream(path, std::ios::in | std::ios::binary);
+        std::string bytes = thumbnailStream.read_all();
+        fireDataForThumbnail(url, reinterpret_cast<const uint8_t*>(bytes.data()), bytes.size());
     }
-    
+
+    void writeRemovedCatalogsFile() const {
+      SGPath p = path / "RemovedCatalogs";
+      sg_ofstream stream(p, std::ios::out | std::ios::trunc | std::ios::binary);
+      for (const auto &cid : manuallyRemovedCatalogs) {
+        stream << cid << "\n";
+      }
+      stream.close();
+    }
+
+    void loadRemovedCatalogsFile() {
+      manuallyRemovedCatalogs.clear();
+      SGPath p = path / "RemovedCatalogs";
+      if (!p.exists())
+        return;
+
+      sg_ifstream stream(p, std::ios::in);
+      while (!stream.eof()) {
+        std::string line;
+        std::getline(stream, line);
+        const auto trimmed = strutils::strip(line);
+        if (!trimmed.empty()) {
+          manuallyRemovedCatalogs.push_back(trimmed);
+        }
+      } // of lines iteration
+    }
+
     DelegateVec delegates;
 
     SGPath path;
@@ -307,18 +328,21 @@ public:
 
     HTTP::Request_ptr thumbnailDownloadRequest;
     StringDeque pendingThumbnails;
-    
+
     struct ThumbnailCacheEntry
     {
         int retryCount = 0;
         bool requestPending = false;
         SGPath pathOnDisk;
     };
-    
+
     std::map<std::string, ThumbnailCacheEntry> thumbnailCache;
 
     typedef std::map<PackageRef, InstallRef> InstallCache;
     InstallCache m_installs;
+
+    /// persistent list of catalogs the user has manually removed
+    string_list manuallyRemovedCatalogs;
 };
 
 
@@ -328,7 +352,7 @@ void Root::ThumbnailDownloader::onDone()
         m_owner->thumbnailDownloadComplete(this, Delegate::STATUS_SUCCESS, m_buffer);
         return;
     }
-    
+
     if (responseCode() != 200) {
         auto status = (responseCode() == 403) ? Delegate::FAIL_HTTP_FORBIDDEN : Delegate::FAIL_DOWNLOAD;
         SG_LOG(SG_NETWORK, SG_INFO, "thumbnail download failure: " << url() << " with reason " << responseCode());
@@ -357,7 +381,7 @@ unsigned int Root::maxAgeSeconds() const
 void Root::setHTTPClient(HTTP::Client* aHTTP)
 {
     d->http = aHTTP;
-    BOOST_FOREACH(HTTP::Request_ptr req, d->httpPendingRequests) {
+    for (auto req : d->httpPendingRequests) {
         d->http->makeRequest(req);
     }
 
@@ -406,7 +430,9 @@ Root::Root(const SGPath& aPath, const std::string& aVersion) :
     if (!thumbsCacheDir.exists()) {
         thumbsCacheDir.create(0755);
     }
-    
+
+    d->loadRemovedCatalogsFile();
+
     for (SGPath c : dir.children(Dir::TYPE_DIR | Dir::NO_DOT_OR_DOTDOT)) {
         // note this will set the catalog status, which will insert into
         // disabled catalogs automatically if necesary
@@ -414,7 +440,7 @@ Root::Root(const SGPath& aPath, const std::string& aVersion) :
         if (cat && cat->isEnabled()) {
             d->catalogs.insert({cat->id(), cat});
         } else if (cat) {
-            SG_LOG(SG_GENERAL, SG_WARN, "Package-Root init: catalog is disabled: " << cat->id());
+            SG_LOG(SG_GENERAL, SG_DEBUG, "Package-Root init: catalog is disabled: " << cat->id());
         }
     } // of child directories iteration
 }
@@ -450,6 +476,17 @@ CatalogRef Root::getCatalogById(const std::string& aId) const
     return it->second;
 }
 
+CatalogRef Root::getCatalogByUrl(const std::string& aUrl) const
+{
+    auto it = std::find_if(d->catalogs.begin(), d->catalogs.end(),
+                           [aUrl](const CatalogDict::value_type& v)
+                           { return (v.second->url() == aUrl); });
+    if (it == d->catalogs.end())
+        return {};
+
+    return it->second;
+}
+
 PackageRef Root::getPackageById(const std::string& aName) const
 {
     size_t lastDot = aName.rfind('.');
@@ -488,7 +525,7 @@ CatalogList Root::catalogs() const
 
     return r;
 }
-    
+
 CatalogList Root::allCatalogs() const
 {
     CatalogList r = catalogs();
@@ -497,13 +534,13 @@ CatalogList Root::allCatalogs() const
 }
 
 PackageList
-Root::allPackages() const
+Root::allPackages(Type ty) const
 {
     PackageList r;
 
     CatalogDict::const_iterator it = d->catalogs.begin();
     for (; it != d->catalogs.end(); ++it) {
-        const PackageList& r2(it->second->packages());
+        const PackageList& r2(it->second->packages(ty));
         r.insert(r.end(), r2.begin(), r2.end());
     }
 
@@ -542,7 +579,7 @@ void Root::refresh(bool aForce)
 {
     bool didStartAny = false;
 
-    // copy all candidate catalogs to a seperate list, since refreshing
+    // copy all candidate catalogs to a separate list, since refreshing
     // can modify both the main collection and/or the disabled list
     CatalogList toRefresh;
     CatalogDict::iterator it = d->catalogs.begin();
@@ -595,6 +632,12 @@ void Root::scheduleToUpdate(InstallRef aInstall)
         throw sg_exception("missing argument to scheduleToUpdate");
     }
 
+    auto it = std::find(d->updateDeque.begin(), d->updateDeque.end(), aInstall);
+    if (it != d->updateDeque.end()) {
+        // already scheduled to update
+        return;
+    }
+
     PackageList deps = aInstall->package()->dependencies();
     for (Package* dep : deps) {
         // will internally schedule for update if required
@@ -611,6 +654,13 @@ void Root::scheduleToUpdate(InstallRef aInstall)
     }
 }
 
+void Root::scheduleAllUpdates() {
+  auto toBeUpdated = packagesNeedingUpdate(); // make a copy
+  for (const auto &u : toBeUpdated) {
+    scheduleToUpdate(u->existingInstall());
+  }
+}
+
 bool Root::isInstallQueued(InstallRef aInstall) const
 {
     auto it = std::find(d->updateDeque.begin(), d->updateDeque.end(), aInstall);
@@ -643,7 +693,7 @@ void Root::startNext(InstallRef aCurrent)
 void Root::finishInstall(InstallRef aInstall, Delegate::StatusCode aReason)
 {
     if (aReason != Delegate::STATUS_SUCCESS) {
-        SG_LOG(SG_GENERAL, SG_ALERT, "failed to install package:"
+        SG_LOG(SG_GENERAL, SG_WARN, "failed to install package:"
                << aInstall->package()->id() << ":" << aReason);
     }
 
@@ -674,20 +724,17 @@ void Root::catalogRefreshStatus(CatalogRef aCat, Delegate::StatusCode aReason)
     auto catIt = d->catalogs.find(aCat->id());
     d->fireRefreshStatus(aCat, aReason);
 
-    if (aReason == Delegate::STATUS_IN_PROGRESS) {
-        d->refreshing.insert(aCat);
-    } else {
-        d->refreshing.erase(aCat);
-    }
-
-    if ((aReason == Delegate::STATUS_REFRESHED) && (catIt == d->catalogs.end())) {
+    if (aCat->isUserEnabled() &&
+        (aReason == Delegate::STATUS_REFRESHED) && 
+        (catIt == d->catalogs.end())) 
+    {
         assert(!aCat->id().empty());
         d->catalogs.insert(catIt, CatalogDict::value_type(aCat->id(), aCat));
 
-        // catalog might have been previously disabled, let's remove in that case
+    // catalog might have been previously disabled, let's remove in that case
         auto j = std::find(d->disabledCatalogs.begin(),
-                           d->disabledCatalogs.end(),
-                           aCat);
+                        d->disabledCatalogs.end(),
+                        aCat);
         if (j != d->disabledCatalogs.end()) {
             SG_LOG(SG_GENERAL, SG_INFO, "re-enabling disabled catalog:" << aCat->id());
             d->disabledCatalogs.erase(j);
@@ -695,7 +742,7 @@ void Root::catalogRefreshStatus(CatalogRef aCat, Delegate::StatusCode aReason)
     }
 
     if (!aCat->isEnabled()) {
-        // catalog has errors, disable it
+        // catalog has errors or was disabled by user, disable it
         auto j = std::find(d->disabledCatalogs.begin(),
                            d->disabledCatalogs.end(),
                            aCat);
@@ -710,22 +757,33 @@ void Root::catalogRefreshStatus(CatalogRef aCat, Delegate::StatusCode aReason)
         }
     } // of catalog is disabled
 
+    // remove from refreshing /after/ checking for enable / disabled, since for
+    // new catalogs, the reference in d->refreshing might be our /only/
+    // reference to the catalog. Once the refresh is done (either failed or
+    // succeeded) the Catalog will be in either d->catalogs or
+    // d->disabledCatalogs
+    if (aReason == Delegate::STATUS_IN_PROGRESS) {
+      d->refreshing.insert(aCat);
+    } else {
+      d->refreshing.erase(aCat);
+    }
+
     if (d->refreshing.empty()) {
         d->fireRefreshStatus(CatalogRef(), Delegate::STATUS_REFRESHED);
         d->firePackagesChanged();
     }
 }
-    
+
 bool Root::removeCatalog(CatalogRef cat)
 {
     if (!cat)
         return false;
-    
+
     // normal remove path
     if (!cat->id().empty()) {
         return removeCatalogById(cat->id());
     }
-    
+
     if (!cat->removeDirectory()) {
         SG_LOG(SG_GENERAL, SG_WARN, "removeCatalog: failed to remove directory " << cat->installRoot());
     }
@@ -735,10 +793,10 @@ bool Root::removeCatalog(CatalogRef cat)
     if (it != d->disabledCatalogs.end()) {
         d->disabledCatalogs.erase(it);
     }
-    
+
     // notify that a catalog is being removed
     d->firePackagesChanged();
-    
+
     return true;
 }
 
@@ -770,6 +828,9 @@ bool Root::removeCatalogById(const std::string& aId)
             << "failed to remove directory");
     }
 
+    d->manuallyRemovedCatalogs.push_back(aId);
+    d->writeRemovedCatalogsFile();
+
     // notify that a catalog is being removed
     d->firePackagesChanged();
 
@@ -796,7 +857,7 @@ void Root::requestThumbnailData(const std::string& aUrl)
         }
     } else {
         if (!it->second.requestPending && it->second.pathOnDisk.exists()) {
-            d->queueLoadFromPersistentCache(aUrl, it->second.pathOnDisk);
+            d->loadFromPersistentCache(aUrl, it->second.pathOnDisk);
         }
     }
 }
@@ -841,6 +902,46 @@ void Root::unregisterInstall(InstallRef ins)
     d->fireFinishUninstall(ins->package());
 }
 
+string_list Root::explicitlyRemovedCatalogs() const {
+  return d->manuallyRemovedCatalogs;
+}
+
+
+PackageList Root::packagesProviding(const std::string& path, bool onlyInstalled) const
+{
+    std::string modPath = path;
+    auto inferredType = AnyPackageType;
+    if (strutils::starts_with(path, "Aircraft/")) {
+        inferredType = AircraftPackage;
+        modPath = path.substr(9);
+    } else if (strutils::starts_with(path, "AI/Aircraft/")) {
+        inferredType = AIModelPackage;
+        modPath = path.substr(12);
+    }
+
+    std::string subPath;
+    const auto firstSeperatorPos = modPath.find('/');
+    if (firstSeperatorPos != std::string::npos) {
+        subPath = modPath.substr(firstSeperatorPos + 1);
+        modPath.resize(firstSeperatorPos);
+    }
+
+    PackageList r;
+    for (auto cat : d->catalogs) {
+        const auto p = cat.second->packagesProviding(inferredType, modPath, subPath);
+        r.insert(r.end(), p.begin(), p.end());
+    } // catalog iteratrion
+
+    if (onlyInstalled) {
+        auto it = std::remove_if(r.begin(), r.end(), [](const PackageRef& p) {
+            return p->isInstalled();
+        });
+        r.erase(it, r.end());
+    }
+
+    return r;
+}
+
 } // of namespace pkg
 
 } // of namespace simgear
