@@ -24,12 +24,14 @@
 #include <simgear/package/Root.hxx>
 #include <simgear/misc/sg_dir.hxx>
 
-#include <boost/foreach.hpp>
 #include <iostream>
 #include <cstring>
 
 using namespace simgear;
-using namespace std;
+using std::cerr;
+using std::cout;
+using std::endl;
+using std::string;
 
 bool keepRunning = true;
 
@@ -59,7 +61,7 @@ public:
 
     virtual void installProgress(pkg::InstallRef aInstall, unsigned int bytes, unsigned int total)
     {
-        unsigned int percent = (bytes * 100) / total;
+        size_t percent = (static_cast<size_t>(bytes) * 100) / total;
         if (percent == _lastPercent) {
             return;
         }
@@ -78,7 +80,7 @@ public:
     }
 
 private:
-    unsigned int _lastPercent;
+    size_t _lastPercent;
 
 };
 
@@ -98,7 +100,7 @@ void printPackageInfo(pkg::Package* pkg)
 
     if (pkg->properties()->hasChild("author")) {
         cout << "Authors:" << endl;
-        BOOST_FOREACH(SGPropertyNode* author, pkg->properties()->getChildren("author")) {
+        for (auto author : pkg->properties()->getChildren("author")) {
             if (author->hasChild("name")) {
                 cout << "\t" << author->getStringValue("name") << endl;
 
@@ -174,7 +176,7 @@ int main(int argc, char** argv)
         pkg->install()->uninstall();
     } else if (!strcmp(argv[1], "update-all")) {
         pkg::PackageList updates = root->packagesNeedingUpdate();
-        BOOST_FOREACH(pkg::Package* p, updates) {
+        for (auto p : updates) {
             root->scheduleToUpdate(p->install());
         }
     } else if (!strcmp(argv[1], "list-updated")) {
@@ -185,7 +187,7 @@ int main(int argc, char** argv)
         }
 
         cout << updates.size() << " packages have updates" << endl;
-        BOOST_FOREACH(pkg::Package* p, updates) {
+        for (auto p : updates) {
             cout << "\t" << p->id() << " " << p->getLocalisedProp("name") << endl;
         }
     } else if (!strcmp(argv[1], "info")) {
