@@ -32,13 +32,10 @@
 #include <map>
 #include <queue>
 #include <utility>
-#include <boost/tr1/unordered_map.hpp>
+#include <unordered_map>
+#include <mutex>
 
-#include <boost/bind.hpp>
-#include <boost/foreach.hpp>
 #include <boost/functional/hash.hpp>
-#include <boost/tuple/tuple.hpp>
-#include <boost/tuple/tuple_comparison.hpp>
 
 #include <osg/AlphaFunc>
 #include <osg/BlendFunc>
@@ -70,16 +67,19 @@
 #include <osgDB/ReadFile>
 #include <osgDB/Registry>
 
-#include <simgear/scene/util/SGReaderWriterOptions.hxx>
+#include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
+#include <simgear/props/props_io.hxx>
+#include <simgear/props/vectorPropTemplates.hxx>
 #include <simgear/scene/tgdb/userdata.hxx>
 #include <simgear/scene/util/OsgMath.hxx>
+#include <simgear/scene/util/SGProgram.hxx>
+#include <simgear/scene/util/SGReaderWriterOptions.hxx>
 #include <simgear/scene/util/SGSceneFeatures.hxx>
 #include <simgear/scene/util/StateAttributeFactory.hxx>
-#include <simgear/structure/OSGUtils.hxx>
+#include <simgear/scene/util/load_shader.hxx>
+#include <simgear/scene/util/OsgUtils.hxx>
 #include <simgear/structure/SGExpression.hxx>
-#include <simgear/props/vectorPropTemplates.hxx>
-#include <simgear/threads/SGThread.hxx>
-#include <simgear/threads/SGGuard.hxx>
 
 namespace simgear
 {
@@ -89,40 +89,21 @@ using namespace osgUtil;
 
 using namespace effect;
 
-class UniformFactoryImpl {
-public:
-    ref_ptr<Uniform> getUniform( Effect * effect, 
-                                 const string & name, 
-                                 Uniform::Type uniformType, 
-                                 SGConstPropertyNode_ptr valProp, 
-                                 const SGReaderWriterOptions* options );
-    void updateListeners( SGPropertyNode* propRoot );
-    void addListener(DeferredPropertyListener* listener);
-private:
-    // Default names for vector property components
-    static const char* vec3Names[];
-    static const char* vec4Names[];
-
-    SGMutex _mutex;
-
-    typedef boost::tuple<std::string, Uniform::Type, std::string, std::string> UniformCacheKey;
-    typedef boost::tuple<ref_ptr<Uniform>, SGPropertyChangeListener*> UniformCacheValue;
-    std::map<UniformCacheKey,ref_ptr<Uniform> > uniformCache;
-
-    typedef std::queue<DeferredPropertyListener*> DeferredListenerList;
-    DeferredListenerList deferredListenerList;
-};
-
 const char* UniformFactoryImpl::vec3Names[] = {"x", "y", "z"};
 const char* UniformFactoryImpl::vec4Names[] = {"x", "y", "z", "w"};
 
-ref_ptr<Uniform> UniformFactoryImpl::getUniform( Effect * effect, 
-                                 const string & name, 
-                                 Uniform::Type uniformType, 
+void UniformFactoryImpl::reset()
+{
+  uniformCache.clear();
+}
+
+ref_ptr<Uniform> UniformFactoryImpl::getUniform( Effect * effect,
+                                 const string & name,
+                                 Uniform::Type uniformType,
                                  SGConstPropertyNode_ptr valProp,
                                  const SGReaderWriterOptions* options )
 {
-	SGGuard<SGMutex> scopeLock(_mutex);
+	std::lock_guard<std::mutex> scopeLock(_mutex);
 	std::string val = "0";
 
 	if (valProp->nChildren() == 0) {
@@ -141,11 +122,11 @@ ref_ptr<Uniform> UniformFactoryImpl::getUniform( Effect * effect,
 				val = getGlobalProperty(prop, options);
 			}
 		} else {
-			SG_LOG(SG_GL,SG_DEBUG,"Invalid parameter " << valProp->getName() << " for uniform " << name << " in Effect ");
+			SG_LOG(SG_GL,SG_DEBUG,"Invalid parameter " << valProp->getNameString() << " for uniform " << name << " in Effect ");
 		}
 	}
 
-	UniformCacheKey key = boost::make_tuple(name, uniformType, val, effect->getName());
+	UniformCacheKey key = std::make_tuple(name, uniformType, val, effect->getName());
 	ref_ptr<Uniform> uniform = uniformCache[key];
 
     if (uniform.valid()) {
@@ -154,6 +135,7 @@ ref_ptr<Uniform> UniformFactoryImpl::getUniform( Effect * effect,
     }
 
     SG_LOG(SG_GL,SG_DEBUG,"new uniform " << name << " value " << uniformCache.size());
+    // REVIEW: Memory Leak - 13,800 bytes in 69 blocks are indirectly lost
     uniformCache[key] = uniform = new Uniform;
 
     uniform->setName(name);
@@ -186,6 +168,9 @@ ref_ptr<Uniform> UniformFactoryImpl::getUniform( Effect * effect,
     case Uniform::SAMPLER_1D_SHADOW:
     case Uniform::SAMPLER_2D_SHADOW:
     case Uniform::SAMPLER_CUBE:
+    case Uniform::IMAGE_1D:
+    case Uniform::IMAGE_2D:
+    case Uniform::IMAGE_3D:
     	initFromParameters(effect, valProp, uniform.get(),
                            static_cast<bool (Uniform::*)(int)>(&Uniform::set),
                            options);
@@ -209,7 +194,7 @@ void UniformFactoryImpl::addListener(DeferredPropertyListener* listener)
 
 void UniformFactoryImpl::updateListeners( SGPropertyNode* propRoot )
 {
-	SGGuard<SGMutex> scopeLock(_mutex);
+	std::lock_guard<std::mutex> scopeLock(_mutex);
 
 	if (deferredListenerList.empty()) return;
 
@@ -223,7 +208,99 @@ void UniformFactoryImpl::updateListeners( SGPropertyNode* propRoot )
 	}
 }
 
-typedef Singleton<UniformFactoryImpl> UniformFactory;
+bool EffectSchemeSingleton::is_valid_scheme(const std::string& name,
+                                            const SGReaderWriterOptions* options)
+{
+    if (!_schemes_xml_read) {
+        read_schemes_xml(options);
+    }
+    if (name.empty()) {
+        // Empty Effect scheme means the default scheme, which is a valid scheme
+        return true;
+    }
+    auto it = std::find_if(_schemes.begin(), _schemes.end(),
+                           [&name](const EffectScheme& scheme) {
+                               return name == scheme.name;
+                           });
+    if (it != _schemes.end()) {
+        return true;
+    }
+    return false;
+}
+
+void EffectSchemeSingleton::maybe_merge_fallbacks(Effect* effect,
+                                                  const SGReaderWriterOptions* options)
+{
+    if (!_schemes_xml_read) {
+        read_schemes_xml(options);
+    }
+    for (const auto& scheme : _schemes) {
+        if (!scheme.fallback) {
+            // The scheme does not have a fallback effect, skip
+            continue;
+        }
+        const std::string& scheme_name = scheme.name;
+        std::vector<SGPropertyNode_ptr> techniques = effect->root->getChildren("technique");
+        auto it = std::find_if(techniques.begin(), techniques.end(),
+                               [&scheme_name](const SGPropertyNode_ptr& tniq) {
+                                   return tniq->getStringValue("scheme") == scheme_name;
+                               });
+        // Only merge the fallback effect if we haven't found a technique
+        // implementing the scheme.
+        if (it == techniques.end()) {
+            SGPropertyNode_ptr new_root = new SGPropertyNode;
+            mergePropertyTrees(new_root, effect->root, scheme.fallback->root);
+            effect->root = new_root;
+            effect->parametersProp = new_root->getChild("parameters");
+            // Copy the generator only if it doesn't exist yet
+            if (effect->generator.empty()) {
+                effect->generator = scheme.fallback->generator;
+            }
+        }
+    }
+}
+
+void EffectSchemeSingleton::read_schemes_xml(const SGReaderWriterOptions* options)
+{
+    SGPropertyNode_ptr scheme_list = new SGPropertyNode;
+    const std::string schemes_file{"Effects/schemes.xml"};
+    std::string abs_file_name = SGModelLib::findDataFile(schemes_file, options);
+    if (abs_file_name.empty()) {
+        SG_LOG(SG_INPUT, SG_ALERT, "Could not find Effect schemes file \"" << schemes_file << "\"");
+        return;
+    }
+    try {
+        readProperties(abs_file_name, scheme_list, 0, true);
+    } catch (sg_io_exception& e) {
+        SG_LOG(SG_INPUT, SG_ALERT, "Error reading Effect schemes file \""
+               << schemes_file << "\": " << e.getFormattedMessage());
+        return;
+    }
+
+    PropertyList p_schemes = scheme_list->getChildren("scheme");
+    for (const auto& p_scheme : p_schemes) {
+        EffectScheme scheme;
+        scheme.name = p_scheme->getStringValue("name");
+        if (scheme.name.empty()) {
+            SG_LOG(SG_INPUT, SG_ALERT, "Scheme with index " << p_scheme->getIndex()
+                   << " does not have a name. Skipping...");
+            continue;
+        }
+        std::string fallback_name = p_scheme->getStringValue("fallback");
+        if (!fallback_name.empty()) {
+            // Read the fallback Effect
+            scheme.fallback = makeEffect(fallback_name, false, options);
+            if (!scheme.fallback.valid()) {
+                SG_LOG(SG_INPUT, SG_ALERT, "Scheme fallback was provided (" << fallback_name
+                       << ") for scheme \"" << scheme.name << "\", but it could not be built. Skipping...");
+                continue;
+            }
+        }
+        scheme.description = p_scheme->getStringValue("description");
+        _schemes.push_back(scheme);
+    }
+    _schemes_xml_read = true;
+}
 
 
 Effect::Effect()
@@ -232,8 +309,9 @@ Effect::Effect()
 }
 
 Effect::Effect(const Effect& rhs, const CopyOp& copyop)
-    : osg::Object(rhs,copyop), root(rhs.root), parametersProp(rhs.parametersProp), _cache(0),
-      _isRealized(rhs._isRealized)
+    : osg::Object(rhs, copyop), root(rhs.root), parametersProp(rhs.parametersProp), _cache(0),
+      _isRealized(rhs._isRealized),
+      _effectFilePath(rhs._effectFilePath)
 {
     typedef vector<ref_ptr<Technique> > TechniqueList;
     for (TechniqueList::const_iterator itr = rhs.techniques.begin(),
@@ -247,11 +325,19 @@ Effect::Effect(const Effect& rhs, const CopyOp& copyop)
     _name += " clone";
 }
 
-// Assume that the last technique is always valid.
+// Try to use the state set of the last technique without a scheme.
 StateSet* Effect::getDefaultStateSet()
 {
-    Technique* tniq = techniques.back().get();
-    if (!tniq)
+    if (techniques.empty())
+        return 0;
+    auto it = std::find_if(techniques.rbegin(), techniques.rend(),
+                           [](const osg::ref_ptr<Technique> &t) {
+                               return t && t->getScheme().empty();
+                           });
+    if (it == techniques.rend())
+        return 0;
+    Technique* tniq = (*it);
+    if (tniq->passes.empty())
         return 0;
     Pass* pass = tniq->passes.front().get();
     return pass;
@@ -266,11 +352,11 @@ int Effect::getGenerator(Effect::Generator what) const
 
 // There should always be a valid technique in an effect.
 
-Technique* Effect::chooseTechnique(RenderInfo* info)
+Technique* Effect::chooseTechnique(RenderInfo* info, const std::string &scheme)
 {
-    BOOST_FOREACH(ref_ptr<Technique>& technique, techniques)
-    {
-        if (technique->valid(info) == Technique::VALID)
+    for (auto& technique : techniques) {
+        if ((technique->valid(info) == Technique::VALID) &&
+            (technique->getScheme() == scheme))
             return technique.get();
     }
     return 0;
@@ -278,16 +364,14 @@ Technique* Effect::chooseTechnique(RenderInfo* info)
 
 void Effect::resizeGLObjectBuffers(unsigned int maxSize)
 {
-    BOOST_FOREACH(const ref_ptr<Technique>& technique, techniques)
-    {
+    for (const auto& technique : techniques) {
         technique->resizeGLObjectBuffers(maxSize);
     }
 }
 
 void Effect::releaseGLObjects(osg::State* state) const
 {
-    BOOST_FOREACH(const ref_ptr<Technique>& technique, techniques)
-    {
+    for (const auto& technique : techniques) {
         technique->releaseGLObjects(state);
     }
 }
@@ -300,6 +384,9 @@ Effect::~Effect()
 void buildPass(Effect* effect, Technique* tniq, const SGPropertyNode* prop,
                const SGReaderWriterOptions* options)
 {
+    // REVIEW: Memory Leak - 35,712 bytes in 72 blocks are still reachable
+    simgear::ErrorReportContext ec("effect-pass", prop->getPath());
+
     Pass* pass = new Pass;
     tniq->passes.push_back(pass);
     for (int i = 0; i < prop->nChildren(); ++i) {
@@ -310,7 +397,7 @@ void buildPass(Effect* effect, Technique* tniq, const SGPropertyNode* prop,
             builder->buildAttribute(effect, pass, attrProp, options);
         else
             SG_LOG(SG_INPUT, SG_ALERT,
-                   "skipping unknown pass attribute " << attrProp->getName());
+                   "skipping unknown pass attribute " << attrProp->getNameString());
     }
 }
 
@@ -323,7 +410,7 @@ osg::Vec4f getColor(const SGPropertyNode* prop)
             return osg::Vec4f(toOsg(prop->getValue<SGVec3d>()), 1.0f);
         } else {
             SG_LOG(SG_INPUT, SG_ALERT,
-                   "invalid color property " << prop->getName() << " "
+                   "invalid color property " << prop->getNameString() << " "
                    << prop->getStringValue());
             return osg::Vec4f(0.0f, 0.0f, 0.0f, 1.0f);
         }
@@ -502,6 +589,8 @@ void MaterialBuilder::buildAttribute(Effect* effect, Pass* pass,
 {
     if (!isAttributeActive(effect, prop))
         return;
+
+    // REVIEW: Memory Leak - 20,160 bytes in 72 blocks are still reachable
     Material* mat = new Material;
     const SGPropertyNode* color = 0;
     if ((color = getEffectPropertyChild(effect, prop, "ambient")))
@@ -832,13 +921,14 @@ size_t hash_value(const ProgramKey& key)
 
 // XXX Should these be protected by a mutex? Probably
 
-typedef tr1::unordered_map<ProgramKey, ref_ptr<Program>,
+typedef std::unordered_map<ProgramKey, ref_ptr<Program>,
                            boost::hash<ProgramKey>, ProgramKey::EqualTo>
 ProgramMap;
 ProgramMap programMap;
 ProgramMap resolvedProgramMap;  // map with resolved shader file names
+inline static std::mutex _programMap_mutex; // Protects the programMap and resolvedProgramMap for multi-threaded access
 
-typedef tr1::unordered_map<ShaderKey, ref_ptr<Shader>, boost::hash<ShaderKey> >
+typedef std::unordered_map<ShaderKey, ref_ptr<Shader>, boost::hash<ShaderKey> >
 ShaderMap;
 ShaderMap shaderMap;
 
@@ -849,7 +939,12 @@ void reload_shaders()
         Shader *shader = sitr->second.get();
         string fileName = SGModelLib::findDataFile(sitr->first.first);
         if (!fileName.empty()) {
-            shader->loadShaderSourceFromFile(fileName);
+            simgear::loadShaderFromUTF8File(shader, fileName);
+        } else {
+            SG_LOG(SG_INPUT, SG_ALERT, "Could not locate shader: " << fileName);
+            simgear::reportFailure(simgear::LoadFailure::NotFound,
+                                   simgear::ErrorCode::LoadEffectsShaders,
+                                   "Reload: couldn't find shader:" + sitr->first.first);
         }
     }
 }
@@ -887,21 +982,27 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
                                           const SGReaderWriterOptions*
                                           options)
 {
-    using namespace boost;
     if (!isAttributeActive(effect, prop))
         return;
     PropertyList pVertShaders = prop->getChildren("vertex-shader");
     PropertyList pGeomShaders = prop->getChildren("geometry-shader");
     PropertyList pFragShaders = prop->getChildren("fragment-shader");
+    PropertyList pCompShaders = prop->getChildren("compute-shader");
     PropertyList pAttributes = prop->getChildren("attribute");
     ProgramKey prgKey;
     std::back_insert_iterator<vector<ShaderKey> > inserter(prgKey.shaders);
-    transform(pVertShaders.begin(), pVertShaders.end(), inserter,
-              boost::bind(makeShaderKey, _1, Shader::VERTEX));
-    transform(pGeomShaders.begin(), pGeomShaders.end(), inserter,
-              boost::bind(makeShaderKey, _1, Shader::GEOMETRY));
-    transform(pFragShaders.begin(), pFragShaders.end(), inserter,
-              boost::bind(makeShaderKey, _1, Shader::FRAGMENT));
+    std::transform(pVertShaders.begin(), pVertShaders.end(), inserter,
+                   [] (SGPropertyNode_ptr& ptr) {
+                       return makeShaderKey(ptr, Shader::VERTEX); });
+    std::transform(pGeomShaders.begin(), pGeomShaders.end(), inserter,
+                   [] (SGPropertyNode_ptr& ptr) {
+                       return makeShaderKey(ptr, Shader::GEOMETRY); });
+    std::transform(pFragShaders.begin(), pFragShaders.end(), inserter,
+                   [] (SGPropertyNode_ptr& ptr) {
+                       return makeShaderKey(ptr, Shader::FRAGMENT); });
+    std::transform(pCompShaders.begin(), pCompShaders.end(), inserter,
+                   [] (SGPropertyNode_ptr& ptr) {
+                       return makeShaderKey(ptr, Shader::COMPUTE); });
     for (PropertyList::iterator itr = pAttributes.begin(),
              e = pAttributes.end();
          itr != e;
@@ -919,6 +1020,8 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
     if (options)
         prgKey.paths = options->getDatabasePathList();
     Program* program = 0;
+
+    std::lock_guard<std::mutex> lock(_programMap_mutex); // Lock the programMap and resolvedProgramMap for this scope
     ProgramMap::iterator pitr = programMap.find(prgKey);
     if (pitr != programMap.end()) {
         program = pitr->second.get();
@@ -932,14 +1035,20 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
     // resolvedProgramMap for a program using those shaders.
     ProgramKey resolvedKey;
     resolvedKey.attributes = prgKey.attributes;
-    BOOST_FOREACH(const ShaderKey& shaderKey, prgKey.shaders)
-    {
-        const string& shaderName = shaderKey.first;
+    for (const auto& shaderKey : prgKey.shaders) {
+        // FIXME orig: const string& shaderName = shaderKey.first;
+        string shaderName = shaderKey.first;
         Shader::Type stype = (Shader::Type)shaderKey.second;
         string fileName = SGModelLib::findDataFile(shaderName, options);
         if (fileName.empty())
+        {
+            SG_LOG(SG_INPUT, SG_ALERT, "Could not locate shader" << shaderName);
+            simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::LoadEffectsShaders,
+                                   "Couldn't locate shader:" + shaderName, sg_location{shaderName});
+
             throw BuilderException(string("couldn't find shader ") +
-                                   shaderName);
+                shaderName);
+        }
         resolvedKey.shaders.push_back(ShaderKey(fileName, stype));
     }
     ProgramMap::iterator resitr = resolvedProgramMap.find(resolvedKey);
@@ -949,9 +1058,12 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
         pass->setAttributeAndModes(program);
         return;
     }
-    program = new Program;
-    BOOST_FOREACH(const ShaderKey& skey, resolvedKey.shaders)
-    {
+
+    auto sgprogram = new SGProgram;
+    program = sgprogram;
+    sgprogram->setEffectFilePath(effect->filePath());
+
+    for (const auto& skey : resolvedKey.shaders) {
         const string& fileName = skey.first;
         Shader::Type stype = (Shader::Type)skey.second;
         ShaderMap::iterator sitr = shaderMap.find(skey);
@@ -959,14 +1071,37 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
             program->addShader(sitr->second.get());
         } else {
             ref_ptr<Shader> shader = new Shader(stype);
-			shader->setName(fileName);
-            if (shader->loadShaderSourceFromFile(fileName)) {
-                program->addShader(shader.get());
+            shader->setName(fileName);
+            if (simgear::loadShaderFromUTF8File(shader, fileName)) {
+                if (!program->addShader(shader.get())) {
+                    simgear::reportFailure(simgear::LoadFailure::BadData,
+                                           simgear::ErrorCode::LoadEffectsShaders,
+                                           "Program::addShader failed",
+                                           SGPath::fromUtf8(fileName));
+                }
+
                 shaderMap.insert(ShaderMap::value_type(skey, shader));
+            } else {
+                simgear::reportFailure(simgear::LoadFailure::BadData,
+                                       simgear::ErrorCode::LoadEffectsShaders,
+                                       "Failed to read shader source code",
+                                       SGPath::fromUtf8(fileName));
             }
         }
     }
-    BOOST_FOREACH(const ProgramKey::AttribKey& key, prgKey.attributes) {
+
+    // disabling this check because some effects have disabled technqiues
+    // with no shaders defined, intentionally.
+#if 0
+    if (sgprogram->getNumShaders() == 0) {
+        simgear::reportFailure(simgear::LoadFailure::BadData,
+                               simgear::ErrorCode::LoadEffectsShaders,
+                               "No shader source code defined for effect",
+                               effect->filePath());
+    }
+#endif
+
+    for (const auto& key : prgKey.attributes) {
         program->addBindAttribLocation(key.first, key.second);
     }
     const SGPropertyNode* pGeometryVerticesOut
@@ -989,6 +1124,13 @@ void ShaderProgramBuilder::buildAttribute(Effect* effect, Pass* pass,
                  type);
         program->setParameter(GL_GEOMETRY_OUTPUT_TYPE_EXT, type);
     }
+    PropertyList pUniformBlockBindings
+        = prop->getChildren("uniform-block-binding");
+    for (const auto &pUniformBlockBinding : pUniformBlockBindings) {
+        program->addBindUniformBlock(
+            pUniformBlockBinding->getStringValue("name"),
+            pUniformBlockBinding->getIntValue("index"));
+    }
     programMap.insert(ProgramMap::value_type(prgKey, program));
     resolvedProgramMap.insert(ProgramMap::value_type(resolvedKey, program));
     pass->setAttributeAndModes(program);
@@ -1008,7 +1150,10 @@ EffectNameValue<Uniform::Type> uniformTypesInit[] =
     {"sampler-2d", Uniform::SAMPLER_2D},
     {"sampler-2d-shadow", Uniform::SAMPLER_2D_SHADOW},
     {"sampler-3d", Uniform::SAMPLER_3D},
-    {"sampler-cube", Uniform::SAMPLER_CUBE}
+    {"sampler-cube", Uniform::SAMPLER_CUBE},
+    {"image-1d", Uniform::IMAGE_1D},
+    {"image-2d", Uniform::IMAGE_2D},
+    {"image-3d", Uniform::IMAGE_3D},
 };
 EffectPropertyMap<Uniform::Type> uniformTypes(uniformTypesInit);
 
@@ -1037,7 +1182,6 @@ struct UniformBuilder :public PassAttributeBuilder
             return;
         SGConstPropertyNode_ptr nameProp = prop->getChild("name");
         SGConstPropertyNode_ptr typeProp = prop->getChild("type");
-        SGConstPropertyNode_ptr positionedProp = prop->getChild("positioned");
         SGConstPropertyNode_ptr valProp = prop->getChild("value");
         string name;
         Uniform::Type uniformType = Uniform::FLOAT;
@@ -1094,12 +1238,10 @@ struct UniformBuilder :public PassAttributeBuilder
                 uniform = colorMode[val];
             }
         }
+
+        // REVIEW: Memory Leak - 144,400 bytes in 38 blocks are indirectly lost
+        // Leak occurs within OSG, likely caused by passing a raw pointer
         pass->addUniform(uniform.get());
-        if (positionedProp && positionedProp->getBoolValue() && uniformType == Uniform::FLOAT_VEC4) {
-            osg::Vec4 offset;
-            uniform->get(offset);
-            pass->addPositionedUniform( name, offset );
-        }
     }
 };
 
@@ -1123,6 +1265,24 @@ struct NameBuilder : public PassAttributeBuilder
 
 InstallAttributeBuilder<NameBuilder> installName("name");
 
+struct DefineBuilder : public PassAttributeBuilder {
+    void buildAttribute(Effect* effect, Pass* pass, const SGPropertyNode* prop,
+                        const SGReaderWriterOptions* options) {
+        const SGPropertyNode* pName = getEffectPropertyChild(effect, prop, "name");
+        if (!pName) {
+            return;
+        }
+        const SGPropertyNode *pValue = getEffectPropertyChild(effect, prop, "value");
+        if (pValue) {
+            pass->setDefine(pName->getStringValue(), pValue->getStringValue());
+        } else {
+            pass->setDefine(pName->getStringValue());
+        }
+    }
+};
+
+InstallAttributeBuilder<DefineBuilder> installDefine("define");
+
 EffectNameValue<PolygonMode::Mode> polygonModeModesInit[] =
 {
     {"fill", PolygonMode::FILL},
@@ -1319,8 +1479,17 @@ InstallAttributeBuilder<DepthBuilder> installDepth("depth");
 void buildTechnique(Effect* effect, const SGPropertyNode* prop,
                     const SGReaderWriterOptions* options)
 {
+    simgear::ErrorReportContext ec("effect-technique", prop->getPath());
+
+    // REVIEW: Memory Leak - 13,248 bytes in 72 blocks are indirectly lost
     Technique* tniq = new Technique;
     effect->techniques.push_back(tniq);
+    std::string scheme = prop->getStringValue("scheme");
+    tniq->setScheme(scheme);
+    if (!EffectSchemeSingleton::instance()->is_valid_scheme(scheme, options)) {
+        SG_LOG(SG_INPUT, SG_ALERT, "technique scheme \"" << scheme << "\" is undefined");
+        tniq->setAlwaysValid(false);
+    }
     const SGPropertyNode* predProp = prop->getChild("predicate");
     if (!predProp) {
         tniq->setAlwaysValid(true);
@@ -1430,6 +1599,11 @@ bool Effect::realizeTechniques(const SGReaderWriterOptions* options)
 {
     if (_isRealized)
         return true;
+
+    simgear::ErrorReportContext ec{"effect", getName()};
+
+    EffectSchemeSingleton::instance()->maybe_merge_fallbacks(this, options);
+
     PropertyList tniqList = root->getChildren("technique");
     for (PropertyList::iterator itr = tniqList.begin(), e = tniqList.end();
          itr != e;
@@ -1484,7 +1658,7 @@ bool Effect_writeLocalData(const Object& obj, osgDB::Output& fw)
     const Effect& effect = static_cast<const Effect&>(obj);
 
     fw.indent() << "techniques " << effect.techniques.size() << "\n";
-    BOOST_FOREACH(const ref_ptr<Technique>& technique, effect.techniques) {
+    for (const auto& technique : effect.techniques) {
         fw.writeObject(*technique);
     }
     return true;
@@ -1570,3 +1744,15 @@ expression::ExpParserRegistrar propvalueRegistrar("float-property",
                                                  propertyExpressionParser<float>);
 
 }
+
+using namespace simgear;
+
+void Effect::setFilePath(const SGPath& path)
+{
+    _effectFilePath = path;
+}
+
+SGPath Effect::filePath() const
+{
+    return _effectFilePath;
+}
