@@ -19,7 +19,8 @@
 #endif
 
 #include <osg/StateSet>
-#include <osg/Texture2D>
+#include <osg/Camera>
+#include <osgViewer/Renderer>
 
 #include "EffectCullVisitor.hxx"
 
@@ -27,20 +28,21 @@
 #include "Effect.hxx"
 #include "Technique.hxx"
 
-#include <simgear/scene/util/RenderConstants.hxx>
-
 namespace simgear
 {
 
 using osgUtil::CullVisitor;
 
-EffectCullVisitor::EffectCullVisitor(bool collectLights) :
-    _collectLights(collectLights)
+EffectCullVisitor::EffectCullVisitor(bool collectLights, const std::string &effScheme) :
+    _collectLights(collectLights),
+    _effScheme(effScheme)
 {
 }
 
 EffectCullVisitor::EffectCullVisitor(const EffectCullVisitor& rhs) :
-    CullVisitor(rhs)
+    osg::Object(), CullVisitor(rhs),
+    _collectLights(rhs._collectLights),
+    _effScheme(rhs._effScheme)
 {
 }
 
@@ -49,6 +51,18 @@ CullVisitor* EffectCullVisitor::clone() const
     return new EffectCullVisitor(*this);
 }
 
+void EffectCullVisitor::apply(osg::Node &node)
+{
+    CullVisitor::apply(node);
+    if (_collectLights) {
+        // TODO: Properly cull lights outside the viewport
+        // (override computeBounds() in SGLight)
+        SGLight *light = dynamic_cast<SGLight *>(&node);
+        if (light)
+            _lightList.push_back(light);
+    }
+}
+
 void EffectCullVisitor::apply(osg::Geode& node)
 {
     if (isCulled(node))
@@ -58,15 +72,12 @@ void EffectCullVisitor::apply(osg::Geode& node)
         CullVisitor::apply(node);
         return;
     }
-    if (_collectLights && ( eg->getNodeMask() & MODELLIGHT_BIT ) ) {
-        _lightList.push_back( eg );
-    }
     Effect* effect = eg->getEffect();
     Technique* technique = 0;
     if (!effect) {
         CullVisitor::apply(node);
         return;
-    } else if (!(technique = effect->chooseTechnique(&getRenderInfo()))) {
+    } else if (!(technique = effect->chooseTechnique(&getRenderInfo(), _effScheme))) {
         return;
     }
     // push the node's state.
@@ -92,19 +103,33 @@ void EffectCullVisitor::reset()
     osgUtil::CullVisitor::reset();
 }
 
-void EffectCullVisitor::clearBufferList()
+void
+installEffectCullVisitor(osg::Camera *camera,
+                         bool collect_lights,
+                         const std::string &effect_scheme)
 {
-    _bufferList.clear();
-}
+    auto renderer = static_cast<osgViewer::Renderer *>(camera->getRenderer());
+    if (!renderer) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "Could not install the Effect cull visitor."
+               "Camera does not have a renderer assigned");
+        return;
+    }
+    for (int i = 0; i < 2; ++i) {
+        osgUtil::SceneView *sceneView = renderer->getSceneView(i);
+        osg::ref_ptr<osgUtil::CullVisitor::Identifier> identifier
+            = sceneView->getCullVisitor()->getIdentifier();
+        sceneView->setCullVisitor(new EffectCullVisitor(collect_lights, effect_scheme));
+        sceneView->getCullVisitor()->setIdentifier(identifier.get());
 
-void EffectCullVisitor::addBuffer(std::string b, osg::Texture2D* tex)
-{
-    _bufferList.insert(std::make_pair(b,tex));
-}
+        // Also set the left and right cull visitors for stereo rendering
+        identifier = sceneView->getCullVisitorLeft()->getIdentifier();
+        sceneView->setCullVisitorLeft(sceneView->getCullVisitor()->clone());
+        sceneView->getCullVisitorLeft()->setIdentifier(identifier.get());
 
-osg::Texture2D* EffectCullVisitor::getBuffer(std::string b)
-{
-    return _bufferList[b];
+        identifier = sceneView->getCullVisitorRight()->getIdentifier();
+        sceneView->setCullVisitorRight(sceneView->getCullVisitor()->clone());
+        sceneView->getCullVisitorRight()->setIdentifier(identifier.get());
+    }
 }
 
 }
