@@ -27,9 +27,6 @@
 #include <osg/Image>
 #include <osg/Vec4>
 
-#include <boost/lexical_cast.hpp>
-#include <boost/tuple/tuple_comparison.hpp>
-
 namespace simgear { namespace effect {
 
 EffectNameValue<MipMapFunction> mipmapFunctionsInit[] =
@@ -85,13 +82,13 @@ void _writeColor(GLenum pixelFormat, T* data, float scale, osg::Vec4 value)
     switch(pixelFormat)
     {
         case(GL_DEPTH_COMPONENT):   //intentionally fall through and execute the code for GL_LUMINANCE
-        case(GL_LUMINANCE):         { *data++ = value.r()*scale; break; }
-        case(GL_ALPHA):             { *data++ = value.a()*scale; break; }
-        case(GL_LUMINANCE_ALPHA):   { *data++ = value.r()*scale; *data++ = value.a()*scale; break; }
-        case(GL_RGB):               { *data++ = value.r()*scale; *data++ = value.g()*scale; *data++ = value.b()*scale; break; }
-        case(GL_RGBA):              { *data++ = value.r()*scale; *data++ = value.g()*scale; *data++ = value.b()*scale; *data++ = value.a()*scale; break; }
-        case(GL_BGR):               { *data++ = value.b()*scale; *data++ = value.g()*scale; *data++ = value.r()*scale; break; }
-        case(GL_BGRA):              { *data++ = value.b()*scale; *data++ = value.g()*scale; *data++ = value.r()*scale; *data++ = value.a()*scale; break; }
+        case(GL_LUMINANCE):         { *data = value.r()*scale; break; }
+        case(GL_ALPHA):             { *data = value.a()*scale; break; }
+        case(GL_LUMINANCE_ALPHA):   { *data++ = value.r()*scale; *data = value.a()*scale; break; }
+        case(GL_RGB):               { *data++ = value.r()*scale; *data++ = value.g()*scale; *data = value.b()*scale; break; }
+        case(GL_RGBA):              { *data++ = value.r()*scale; *data++ = value.g()*scale; *data++ = value.b()*scale; *data = value.a()*scale; break; }
+        case(GL_BGR):               { *data++ = value.b()*scale; *data++ = value.g()*scale; *data = value.r()*scale; break; }
+        case(GL_BGRA):              { *data++ = value.b()*scale; *data++ = value.g()*scale; *data++ = value.r()*scale; *data = value.a()*scale; break; }
     }
 }
 
@@ -153,7 +150,11 @@ osg::Vec4::value_type computeAverage( int c, osg::Vec4 colors[2][2][2], bool col
             nb += 1;
         }
     }
-    return r / nb;
+
+    if (nb != 0)
+        return r / nb;
+    
+    return nb;
 }
 
 osg::Vec4::value_type computeSum( int c, osg::Vec4 colors[2][2][2], bool colorValid[2][2][2] )
@@ -218,13 +219,13 @@ osg::Vec4 computeColor( osg::Vec4 colors[2][2][2], bool colorValid[2][2][2], Mip
 {
     osg::Vec4 result;
     unsigned int nbComponents = osg::Image::computeNumComponents( pixelFormat );
-    result[0] = computeComponent( 0, colors, colorValid, attrs.get<0>() );
+    result[0] = computeComponent( 0, colors, colorValid, std::get<0>(attrs) );
     if ( nbComponents >= 2 )
-        result[1] = computeComponent( 1, colors, colorValid, attrs.get<1>() );
+        result[1] = computeComponent( 1, colors, colorValid, std::get<1>(attrs) );
     if ( nbComponents >= 3 )
-        result[2] = computeComponent( 2, colors, colorValid, attrs.get<2>() );
+        result[2] = computeComponent( 2, colors, colorValid, std::get<2>(attrs) );
     if ( nbComponents == 4 )
-        result[3] = computeComponent( 3, colors, colorValid, attrs.get<3>() );
+        result[3] = computeComponent( 3, colors, colorValid, std::get<3>(attrs) );
     return result;
 }
 
@@ -268,17 +269,23 @@ osg::Image* computeMipmap( osg::Image* image, MipMapTuple attrs )
 {
     bool computeMipmap = false;
     unsigned int nbComponents = osg::Image::computeNumComponents( image->getPixelFormat() );
-    if ( attrs.get<0>() != AUTOMATIC &&
-        ( attrs.get<1>() != AUTOMATIC || nbComponents < 2 ) &&
-        ( attrs.get<2>() != AUTOMATIC || nbComponents < 3 ) &&
-        ( attrs.get<3>() != AUTOMATIC || nbComponents < 4 ) )
+    int s = image->s();
+    int t = image->t();
+    if ( (s & (s - 1)) || (t & (t - 1)) ) // power of two test
+    {
+        SG_LOG(SG_IO, SG_DEV_ALERT, "mipmapping: texture size not a power-of-two: " + image->getFileName());
+    }
+    else if ( std::get<0>(attrs) != AUTOMATIC &&
+        ( std::get<1>(attrs) != AUTOMATIC || nbComponents < 2 ) &&
+        ( std::get<2>(attrs) != AUTOMATIC || nbComponents < 3 ) &&
+        ( std::get<3>(attrs) != AUTOMATIC || nbComponents < 4 ) )
     {
         computeMipmap = true;
     }
-    else if ( attrs.get<0>() != AUTOMATIC ||
-        ( attrs.get<1>() != AUTOMATIC && nbComponents >= 2 ) ||
-        ( attrs.get<2>() != AUTOMATIC && nbComponents >= 3 ) ||
-        ( attrs.get<3>() != AUTOMATIC && nbComponents == 4 ) )
+    else if ( std::get<0>(attrs) != AUTOMATIC ||
+             ( std::get<1>(attrs) != AUTOMATIC && nbComponents >= 2 ) ||
+             ( std::get<2>(attrs) != AUTOMATIC && nbComponents >= 3 ) ||
+             ( std::get<3>(attrs) != AUTOMATIC && nbComponents == 4 ) )
     {
         throw BuilderException("invalid mipmap control function combination");
     }
@@ -286,9 +293,7 @@ osg::Image* computeMipmap( osg::Image* image, MipMapTuple attrs )
     if ( computeMipmap )
     {
         osg::ref_ptr<osg::Image> mipmaps = new osg::Image();
-        int s = image->s(),
-            t = image->t(),
-            r = image->r();
+        int r = image->r();
         int nb = osg::Image::computeNumberOfMipmapLevels(s, t, r);
         osg::Image::MipmapDataType mipmapOffsets;
         unsigned int offset = 0;
@@ -392,6 +397,8 @@ osg::Image* computeMipmap( osg::Image* image, MipMapTuple attrs )
             image->getInternalTextureFormat(), image->getPixelFormat(),
             image->getDataType(), data, osg::Image::USE_NEW_DELETE, image->getPacking() );
         mipmaps->setMipmapLevels( mipmapOffsets );
+        mipmaps->setName(image->getName());
+        mipmaps->setFileName(image->getFileName());
 
         return mipmaps.release();
     }
