@@ -29,6 +29,7 @@
 #include <simgear/scene/model/model.hxx>
 #include <simgear/scene/model/ConditionNode.hxx>
 #include <simgear/scene/util/OsgMath.hxx>
+#include <simgear/scene/util/SGSceneUserData.hxx>
 
 using namespace std;
 using namespace simgear;
@@ -38,71 +39,125 @@ namespace {
  * Get a color from properties.
  */
 struct ColorSpec {
-  float red, green, blue;
-  float factor;
-  float offset;
-  SGPropertyNode_ptr red_prop;
-  SGPropertyNode_ptr green_prop;
-  SGPropertyNode_ptr blue_prop;
-  SGPropertyNode_ptr factor_prop;
-  SGPropertyNode_ptr offset_prop;
+  SGSharedPtr<SGExpressiond> redex, greenex, blueex;
+  SGSharedPtr<SGExpressiond> factorex;
+  SGSharedPtr<SGExpressiond> offsetex;
   SGVec4f v;
   
   ColorSpec(const SGPropertyNode* configNode, SGPropertyNode* modelRoot)
   {
-    red = -1.0;
-    green = -1.0;
-    blue = -1.0;
+    redex = new SGConstExpression<double>(-1.0);
+    greenex = new SGConstExpression<double>(-1.0);
+    blueex = new SGConstExpression<double>(-1.0);
+    factorex = new SGConstExpression<double>(1.0);
+    offsetex = new SGConstExpression<double>(0.0);
     if (!configNode)
       return;
-    
-    red = configNode->getFloatValue("red", -1.0);
-    green = configNode->getFloatValue("green", -1.0);
-    blue = configNode->getFloatValue("blue", -1.0);
-    factor = configNode->getFloatValue("factor", 1.0);
-    offset = configNode->getFloatValue("offset", 0.0);
-    
+
     if (!modelRoot)
       return;
     const SGPropertyNode *node;
+    const SGPropertyNode *expNode;
+
+    node = configNode->getNode("red");
+    if (node) {
+      expNode = node->getNode("expression");
+      if (expNode) {
+        redex = SGReadDoubleExpression(modelRoot, expNode->getChild(0));
+        if (redex) {
+          redex = redex->simplify();
+        }
+      } else {
+        redex = new SGConstExpression<double>(node->getFloatValue(-1.0));
+      }
+    }
+    node = configNode->getNode("green");
+    if (node) {
+      expNode = node->getNode("expression");
+      if (expNode) {
+        greenex = SGReadDoubleExpression(modelRoot, expNode->getChild(0));
+        if (greenex) {
+          greenex->simplify();
+        }
+      } else {
+        greenex = new SGConstExpression<double>(node->getFloatValue(-1.0));
+      }
+    }
+    node = configNode->getNode("blue");
+    if (node) {
+      expNode = node->getNode("expression");
+      if (expNode) {
+        blueex = SGReadDoubleExpression(modelRoot, expNode->getChild(0));
+        if (blueex) {
+          blueex = blueex->simplify();
+        }
+      } else {
+        blueex = new SGConstExpression<double>(node->getFloatValue(-1.0));
+      }
+    }
+    node = configNode->getNode("factor");
+    if (node) {
+      expNode = node->getNode("expression");
+      if (expNode) {
+        factorex = SGReadDoubleExpression(modelRoot, expNode->getChild(0));
+        if (factorex) {
+          factorex = factorex->simplify();
+        }
+      } else {
+        factorex = new SGConstExpression<double>(node->getFloatValue(1.0));
+      }
+    }
+    node = configNode->getNode("offset");
+    if (node) {
+      expNode = node->getNode("expression");
+      if (expNode) {
+        offsetex = SGReadDoubleExpression(modelRoot, expNode->getChild(0));
+        if (offsetex) {
+          offsetex = offsetex->simplify();
+        }
+      } else {
+        offsetex = new SGConstExpression<double>(node->getFloatValue(0.0));
+      }
+    }
+
     node = configNode->getChild("red-prop");
     if (node)
-      red_prop = modelRoot->getNode(node->getStringValue(), true);
+      redex = new SGPropertyExpression<double>(modelRoot->getNode(node->getStringValue(), true));
     node = configNode->getChild("green-prop");
     if (node)
-      green_prop = modelRoot->getNode(node->getStringValue(), true);
+      greenex = new SGPropertyExpression<double>(modelRoot->getNode(node->getStringValue(), true));
     node = configNode->getChild("blue-prop");
     if (node)
-      blue_prop = modelRoot->getNode(node->getStringValue(), true);
+      blueex = new SGPropertyExpression<double>(modelRoot->getNode(node->getStringValue(), true));
     node = configNode->getChild("factor-prop");
     if (node)
-      factor_prop = modelRoot->getNode(node->getStringValue(), true);
+      factorex = new SGPropertyExpression<double>(modelRoot->getNode(node->getStringValue(), true));
     node = configNode->getChild("offset-prop");
     if (node)
-      offset_prop = modelRoot->getNode(node->getStringValue(), true);
+      offsetex = new SGPropertyExpression<double>(modelRoot->getNode(node->getStringValue(), true));
   }
   
   bool dirty() {
-    return red >= 0 || green >= 0 || blue >= 0;
+    return redex->getValue() >= 0 || greenex->getValue() >= 0 || blueex->getValue() >= 0;
   }
   bool live() {
-    return red_prop || green_prop || blue_prop
-      || factor_prop || offset_prop;
+    return !(redex->isConst() && greenex->isConst() && blueex->isConst() && factorex->isConst() && offsetex->isConst());
   }
   SGVec4f &rgba() {
-    if (red_prop)
-      red = red_prop->getFloatValue();
-    if (green_prop)
-      green = green_prop->getFloatValue();
-    if (blue_prop)
-      blue = blue_prop->getFloatValue();
-    if (factor_prop)
-      factor = factor_prop->getFloatValue();
-    if (offset_prop)
-      offset = offset_prop->getFloatValue();
-    v[0] = SGMiscf::clip(red*factor + offset, 0, 1);
-    v[1] = SGMiscf::clip(green*factor + offset, 0, 1);
-    v[2] = SGMiscf::clip(blue*factor + offset, 0, 1);
+    float red = -1.0, green = -1.0, blue = -1.0, factor = 1.0, offset = 0.0;
+    if (redex)
+      red = redex->getValue();
+    if (greenex)
+      green = greenex->getValue();
+    if (blueex)
+      blue = blueex->getValue();
+    if (factorex)
+      factor = factorex->getValue();
+    if (offsetex)
+      offset = offsetex->getValue();
+    v[0] = SGMiscf::clip(red * factor + offset, 0, 1);
+    v[1] = SGMiscf::clip(green * factor + offset, 0, 1);
+    v[2] = SGMiscf::clip(blue * factor + offset, 0, 1);
     v[3] = 1;
     return v;
   }
@@ -111,14 +166,6 @@ struct ColorSpec {
   {
     return toOsg(rgba());
   }
-  
-  SGVec4f &initialRgba() {
-    v[0] = SGMiscf::clip(red*factor + offset, 0, 1);
-    v[1] = SGMiscf::clip(green*factor + offset, 0, 1);
-    v[2] = SGMiscf::clip(blue*factor + offset, 0, 1);
-    v[3] = 1;
-    return v;
-  }
 };
 
 /**
@@ -357,7 +404,7 @@ public:
       _textureProp = modelRoot->getNode(node->getStringValue(), true);
   }
 
-  virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
+  void operator()(osg::Node* node, osg::NodeVisitor* nv) override
   {
     osg::StateSet* stateSet = node->getStateSet();
     if (stateSet) {
@@ -381,9 +428,9 @@ public:
                                        osg::StateAttribute::ON);
             }
           } else {
-              SG_LOG(SG_IO, SG_WARN, "texture animation: requested texture : " << textureName << " not found. Searched paths:" );
+              SG_LOG(SG_IO, SG_DEV_WARN, "texture animation: requested texture : " << textureName << " not found. Searched paths:" );
               for( osgDB::FilePathList::iterator it = _texturePathList.begin(); it != _texturePathList.end(); ++it ) {
-                SG_LOG(SG_IO, SG_WARN, " - " << *it );
+                SG_LOG(SG_IO, SG_DEV_WARN, " - " << *it );
               }
           }
         }
@@ -419,17 +466,25 @@ private:
 SGMaterialAnimation::SGMaterialAnimation(simgear::SGTransientModelData &modelData) :
     SGAnimation(modelData), texturePathList(modelData.getOptions()->getDatabasePathList())
 {
+    // add model directory to texture path - this allows short paths relative to 
+    // model root.
+    SGPath modelDir(_modelData.getPath());
+    texturePathList.insert(texturePathList.begin(), modelDir.dir());
+
   if (modelData.getConfigNode()->hasChild("global"))
-    SG_LOG(SG_IO, SG_ALERT, modelData.getPath() <<
+    SG_LOG(SG_IO, SG_DEV_ALERT, modelData.getPath() <<
            ": Use of <global> in material animation is "
            "no longer supported.");
 }
 
+SGMaterialAnimation::~SGMaterialAnimation() = default;
+
 osg::Group*
 SGMaterialAnimation::createAnimationGroup(osg::Group& parent)
 {
   osg::Group* group = new osg::Group;
   group->setName("material animation group");
+  SGSceneUserData::getOrCreateSceneUserData(group)->setLocation(getConfig()->getLocation());
 
   SGPropertyNode* inputRoot = getModelRoot();
   const SGPropertyNode* node = getConfig()->getChild("property-base");
