@@ -16,11 +16,10 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
-#ifdef HAVE_CONFIG_H
-#  include <simgear_config.h>
-#endif
+#include <simgear_config.h>
 
 #include <algorithm>
+#include <optional>
 //yuck
 #include <cstring>
 #include <cassert>
@@ -39,6 +38,7 @@
 #include <simgear/props/condition.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
+
 #include <simgear/scene/util/SGNodeMasks.hxx>
 #include <simgear/scene/util/SGReaderWriterOptions.hxx>
 #include <simgear/structure/exception.hxx>
@@ -91,7 +91,6 @@ SGReaderWriterXML::readNode(const std::string& name,
           return ReadResult::FILE_NOT_FOUND;
         }
 
-        static std::tuple<int, osg::Node *> retval;
         int num_anims;
         std::tie(num_anims, result) = sgLoad3DModel_internal(p, options);
     } catch (const sg_exception &t) {
@@ -165,14 +164,14 @@ void makeEffectAnimations(PropertyList& animation_nodes,
         if (!typeProp)
             continue;
 
-        const char* typeString = typeProp->getStringValue();
-        if (!strcmp(typeString, "material")) {
+        std::string typeString = typeProp->getStringValue();
+        if (typeString == "material") {
             effectProp
                 = SGMaterialAnimation::makeEffectProperties(animProp);
-        } else if (!strcmp(typeString, "shader")) {
+        } else if (typeString == "shader") {
 
             SGPropertyNode* shaderProp = animProp->getChild("shader");
-            if (!shaderProp || strcmp(shaderProp->getStringValue(), "chrome"))
+            if (!shaderProp || shaderProp->getStringValue() != "chrome")
                 continue;
             *itr = 0;           // effect replaces animation
             SGPropertyNode* textureProp = animProp->getChild("texture");
@@ -183,7 +182,7 @@ void makeEffectAnimations(PropertyList& animation_nodes,
                 ->setValue("Effects/chrome");
             SGPropertyNode* paramsProp = makeChild(effectProp.get(), "parameters");
             makeChild(paramsProp, "chrome-texture")
-                ->setValue(textureProp->getStringValue());
+                ->setValue(textureProp->getStringValue().c_str());
         }
         if (effectProp.valid()) {
             PropertyList objectNameNodes = animProp->getChildren("object-name");
@@ -258,23 +257,303 @@ namespace {
     }
 }
 
+struct DumpSGPropertyNode {
+    
+    SGPropertyNode*     _node;
+    const std::string&  _indent;
+    
+    DumpSGPropertyNode(SGPropertyNode* node, const std::string& indent="")
+    :
+    _node(node),
+    _indent(indent)
+    {}
+    
+    friend std::ostream& operator << (std::ostream& out, const DumpSGPropertyNode& dump)
+    {
+        if (!dump._node)    return out;
+        out << dump._indent << dump._node->getDisplayName() << "=" << dump._node->getStringValue() << "\n";
+        for (int i=0; i<dump._node->nChildren(); ++i) {
+            SGPropertyNode* child = dump._node->getChild(i);
+            out << DumpSGPropertyNode(child, dump._indent + "    ");
+        }
+        return out;
+    }
+};
+
+struct DumpOsgNode {
+    
+    osg::Node*          _node;
+    const std::string&  _indent;
+    
+    DumpOsgNode(osg::Node* node, const std::string& indent="")
+    :
+    _node(node),
+    _indent(indent)
+    {}
+    
+    friend std::ostream& operator << (std::ostream& out, const DumpOsgNode& dump)
+    {
+        if (!dump._node)    return out;
+        out << dump._indent << dump._node->getName() << "\n";
+        osg::Group* group = dynamic_cast<osg::Group*>(dump._node);
+        if (group) {
+            for (unsigned i=0; i<group->getNumChildren(); ++i) {
+                out << DumpOsgNode(group->getChild(i), dump._indent + "    ");
+            }
+        }
+        return out;
+    }
+};
+
+/* Finds all names recursively in an osg::Node. */
+struct OSGNodeGetNames
+{
+    std::vector<std::string>    names;
+    
+    OSGNodeGetNames(osg::Node* node=NULL)
+    {
+        add(node);
+    }
+    
+    void add(osg::Node* node)
+    {
+        if (!node)  return;
+        const std::string&  name = node->getName();
+        if (name != "") names.push_back(name);
+        
+        osg::Group* group = dynamic_cast<osg::Group*>(node);
+        if (group) {
+            for (unsigned i=0; i<group->getNumChildren(); ++i) {
+                add(group->getChild(i));
+            }
+        }
+    }
+};
+
+
+/* For each existing animation in <props>, we add a tooltip showing information
+on the properties that the animation depends on.
+
+At runtime, the tooltips only show if sim/animation-tooltips is true.
+
+The dummy animations will show up as yellow (like clickable items) if the user
+presses Ctrl-C, even if tooltips aren't showing because sim/animation-tooltips
+is false. */
+void addTooltipAnimations(const SGPath& path, SGPropertyNode_ptr props, osg::ref_ptr<osg::Node> model, int autoTooltipsMasterMax)
+{
+    OSGNodeGetNames model_names;
+    if (0) {
+        /* Include all names in the .asi file, in an attempt to make
+        tooltips activate for more than just the needle in instrument dials
+        for example. This doesn't seem to help. */
+        model_names.add(model);
+        if (!model_names.names.empty()) {
+            /* First name will be filename of .ac file. */
+            model_names.names.erase(model_names.names.begin());
+        }
+    }
+
+    /* For each animation add an extra animation with type=pick containing
+    set-tooltip. We use the object-name as the tooltip-id, and we use the
+    animation's objectname and property name/value(s) in the tooltip label. */
+    PropertyList animations = props->getChildren("animation");
+    SG_LOG(SG_INPUT, SG_BULK, "animations.size()=" << animations.size()
+            << " path=" << path
+            << " props=" << (props ? props->getPath() : "")
+            );
+
+    /* We want to add to any existing tooltip for a particular object-name
+    (e.g. joysticks might have separate animations for elevator and aileron
+    control properties), so we keep track of the ones we've created in this
+    map. */
+    std::map<std::string, SGPropertyNode*>  object_name_to_animation_node;
+    
+    /* Keep track of the total number of animations, so we can respect
+    autoTooltipsMasterMax. */
+    static int  num_new_animations = 0;
+
+    for (unsigned i = 0; i < animations.size(); ++i) {
+
+        SGPropertyNode* animation = animations[i];
+
+        if (animation->getStringValue("type") == "pick") {
+            /* There appear to be many of these, and we end up consuming
+            GB's of memory if we install a tooltip for each one, so ignore.
+            */
+            continue;
+        }
+
+        PropertyList    properties = animation->getChildren("property");
+        if (properties.empty()) {
+            /* We can't really show anything useful in tooltip. */
+            continue;
+        }
+
+        PropertyList objectnames(animation->getChildren("object-name"));
+        if (objectnames.empty()) {
+            continue;
+        }
+
+        /* Make a unique tooltip-id. */
+        std::string tooltip_id = "auto-tooltip-";
+        {
+            if (autoTooltipsMasterMax > 0 && num_new_animations > autoTooltipsMasterMax) {
+                continue;
+            }
+            num_new_animations += 1;
+            std::ostringstream  s;
+            s << num_new_animations;
+            tooltip_id += s.str();
+        }
+
+        /* If we reach here, we create a new dummy animation with type="pick"
+        that will implement a tooltip with information about <animation>. */
+
+        SGPropertyNode* new_animation = nullptr;
+        std::string     objectname_for_label;
+
+        /* Use <animation>'s object-names so that our tooltip appears whenever
+        the user hovers over <animation>'s objects. */
+        for (unsigned i=0; i<objectnames.size(); ++i) {
+            std::string objectname = objectnames[i]->getStringValue();
+            SGPropertyNode*&    n = object_name_to_animation_node[objectname];
+            if (n) {
+                /* We're adding to an existing animation so might as well
+                correct num_new_animations. */
+                num_new_animations -= 1;
+            }
+            else {
+                n = props->addChild("animation");
+                n->addChild("object-name")->setStringValue(objectname);
+                n->addChild("type")->setValue("pick");
+            }
+            new_animation = n;
+            objectname_for_label = objectname;
+        }
+
+        /* We could be adding to previously-created animation or setting up a
+        new animation. So we use getChild() with create=true to populate. */
+        SGPropertyNode* hovered_binding = new_animation
+                ->getChild("hovered", 0 /*index*/, true /*create*/)
+                ->getChild("binding", 0 /*index*/, true /*create*/)
+                ;
+        hovered_binding
+                ->getChild("command", 0 /*index*/, true /*create*/)
+                ->setValue("set-tooltip")
+                ;
+        hovered_binding
+                ->getChild("condition", 0 /*index*/, true /*create*/)
+                ->getChild("property", 0 /*index*/, true /*create*/)
+                ->setValue("sim/animation-tooltips")
+                ;
+        if (!hovered_binding->getChild("tooltip-id")) {
+            hovered_binding
+                    ->getChild("tooltip-id", 0 /*index*/, true /*create*/)
+                    ->setStringValue(tooltip_id)
+                    ;
+        }
+
+        std::string label = hovered_binding->getStringValue("label");
+        if (label == "") {
+            label += objectname_for_label + ":";
+        }
+
+        /* Build up printf-style label string showing the property values
+        that <animation> depend on. */
+        for (size_t i=0; i<properties.size(); ++i) {
+            label += " ";
+            SGPropertyNode* p = properties[i];
+            /* Using an alias here (rather then just copying the path) ensures
+            things work if <p> is itself an alias. */
+            hovered_binding->addChild("property")->alias(p, false);
+            if (p->isAlias()) {
+                /* We only get things like "/rpm[0]" here, rather than the full
+                path of the property to which p points. */
+                label += p->getAliasTarget()->getPath(true /*simplify*/);
+            }
+            else {
+                label += p->getStringValue();
+            }
+            label += "=%s";
+        }
+        hovered_binding->setStringValue("label", label);
+
+        SG_LOG(SG_INPUT, SG_BULK,
+                "have added/updated auto-tooltip."
+                << " num_new_animations=" << num_new_animations
+                << " new_animation=" << new_animation << ":\n"
+                << DumpSGPropertyNode(new_animation, "    ")
+                );
+    }
+
+    /* This is very verbose, so disabled even at level SG_BULK. */
+    if (0) {
+        SG_LOG(SG_INPUT, SG_BULK, "props for path=" << path << ":\n"
+                << DumpSGPropertyNode(props, "    ")
+                );
+    }
+    
+    SG_LOG(SG_INPUT, SG_DEBUG, "auto-tooltips: num_new_animations=" << num_new_animations);
+}
+
+static std::optional<SGPath> isModelWithRemovedXMLWrapper(const SGPath& model)
+{
+    if (model.extension() != "xml") {
+        return {};
+    }
+
+    const auto grandparentDirPath = model.dirPath().dirPath();
+    if (grandparentDirPath.file() != "Models") {
+        return {};
+    }
+
+    const auto names = std::vector<std::string>{
+        "marker",
+        "ndb",
+    };
+
+    const auto it = std::find(names.begin(), names.end(), model.file_base());
+    if (it != names.end()) {
+        return SGPath{model.dirPath() / (*it + ".ac")};
+    }
+
+    return {};
+}
+
 static std::tuple<int, osg::Node *>
 sgLoad3DModel_internal(const SGPath& path,
                        const osgDB::Options* dbOptions,
                        SGPropertyNode *overlay)
 {
+    SGPath modelpath(path);
+    SGPath texturepath(path);
+
     if (!path.exists()) {
-        simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLModelLoad,
-                               "Failed to load model XML: not found", path);
-        SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load file: \"" << path << "\"");
-        return std::make_tuple(0, (osg::Node*)NULL);
+        // tolerate .xml paths where we removed the wrapper XML
+        // because this is a resolved path, only works when the bare model
+        // is in the same location.
+
+        // NOTE: there is separate logic to handle this situation, in the ModelRegistry,
+        // for the DelayedModelLoadCallback case. (See fileNameIsFGDataModelXML)]
+        // Would be nice to share the implementation but inside an OSG RegistryCallback,
+        // it's tricky to work with SGPaths as our code here does.
+        auto b = isModelWithRemovedXMLWrapper(path);
+        if (b) {
+            modelpath = b.value_or(SGPath{});
+            texturepath = b.value_or(SGPath{});
+            SG_LOG(SG_IO, SG_DEV_WARN, "Requested model which previously used an XML wrapper:" << path 
+                << " mapped to " << modelpath);
+        } else {
+            simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLModelLoad,
+                                   "Failed to load model XML: not found", path);
+            return std::make_tuple(0, (osg::Node*) nullptr);
+        }
     }
 
     osg::ref_ptr<SGReaderWriterOptions> options;
     options = SGReaderWriterOptions::copyOrCreate(dbOptions);
 
-    SGPath modelpath(path);
-    SGPath texturepath(path);
+
     SGPath modelDir(modelpath.dir());
     int animationcount = 0;
 
@@ -285,6 +564,10 @@ sgLoad3DModel_internal(const SGPath& path,
     osg::ref_ptr<SGModelData> data = options->getModelData();
     options->setModelData(0);
 
+    // remember the current value of the vertex order setting 
+    // because an included <model> may change this.
+    bool currentVertexOrderXYZ = options->getVertexOrderXYZ();
+
     osg::ref_ptr<osg::Node> model;
     osg::ref_ptr<osg::Group> group;
     SGPropertyNode_ptr props = new SGPropertyNode;
@@ -293,7 +576,7 @@ sgLoad3DModel_internal(const SGPath& path,
     // Check for an XML wrapper
     if (modelpath.extension() == "xml") {
        try {
-            readProperties(modelpath, props);
+           readProperties(modelpath, props);
         } catch (const sg_exception &t) {
             simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::XMLModelLoad,
                                    "Failed to load model XML:" + t.getFormattedMessage(), t.getLocation());
@@ -301,13 +584,25 @@ sgLoad3DModel_internal(const SGPath& path,
                    << t.getFormattedMessage());
             throw;
         }
+
         if (overlay)
             copyProperties(overlay, props);
 
+        if (options->getAutoTooltipsMaster()) {
+            addTooltipAnimations(path, props, model, options->getAutoTooltipsMasterMax());
+        }
+        
         if (previewMode && props->hasChild("nopreview")) {
             return std::make_tuple(0, (osg::Node *) NULL);
         }
 
+        if (props->hasChild("defaults")) {
+            SGPropertyNode_ptr defaultsNode = props->getNode("defaults");
+            if (defaultsNode->hasChild("axis-animation-vertex-order-xyz"))
+                options->setVertexOrderXYZ(true);
+            if (defaultsNode->hasChild("axis-animation-vertex-order-x"))
+                options->setVertexOrderXYZ(false);
+        }
         if (props->hasValue("/path")) {
             string modelPathStr = props->getStringValue("/path");
             modelpath = SGModelLib::findDataFile(modelPathStr, NULL, modelDir);
@@ -349,12 +644,11 @@ sgLoad3DModel_internal(const SGPath& path,
             texturepath = texturepath.dir();
 
         options->setDatabasePath(texturepath.utf8Str());
+        options->setPluginStringData("filePath", modelpath.utf8Str());
+
         osgDB::ReaderWriter::ReadResult modelResult;
-#if OSG_VERSION_LESS_THAN(3,4,1)
-        modelResult = osgDB::readNodeFile(modelpath.utf8Str(), options.get());
-#else
         modelResult = osgDB::readRefNodeFile(modelpath.utf8Str(), options.get());
-#endif
+
         if (!modelResult.validNode()) {
             simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::XMLModelLoad,
                                    "Failed to load 3D model:" + modelResult.message(), modelpath);
@@ -370,6 +664,7 @@ sgLoad3DModel_internal(const SGPath& path,
         // clone all structural nodes here we need that extra
         // reference to the original object
         SGDatabaseReference* databaseReference;
+        // REVIEW: Memory Leak - 4,352 bytes in 272 blocks are definitely lost
         databaseReference = new SGDatabaseReference(modelResult.getNode());
         model->addObserver(databaseReference);
 
@@ -430,7 +725,7 @@ sgLoad3DModel_internal(const SGPath& path,
         if (submodelPath.isNull()) {
           SG_LOG(SG_IO, SG_DEV_ALERT, "Failed to load file: \"" << subPathStr << "\"");
           simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLModelLoad,
-                                 "Couldn't find file for submodel:" + subPathStr,
+                                 "Couldn't find file for submodel '" + sub_props->getStringValue("name") + "': " + subPathStr,
                                  SGPath::fromUtf8(subPathStr));
           continue;
         }
@@ -544,7 +839,7 @@ sgLoad3DModel_internal(const SGPath& path,
     for (unsigned i = 0; i < light_nodes.size(); ++i) {
         group->addChild(SGLight::appendLight(light_nodes[i],
                                              prop_root,
-                                             options.get()));
+                                             false /* legacy mode */));
     }
 
     PropertyList effect_nodes = props->getChildren("effect");
@@ -560,7 +855,7 @@ sgLoad3DModel_internal(const SGPath& path,
     makeEffectAnimations(animation_nodes, effect_nodes);
     {
         ref_ptr<Node> modelWithEffects = instantiateEffects(group.get(), effect_nodes, options.get(),
-                                                            path.dirPath());
+                                                            path);
         group = static_cast<Group*>(modelWithEffects.get());
     }
 
@@ -583,17 +878,28 @@ sgLoad3DModel_internal(const SGPath& path,
 
             /// OSGFIXME: duh, why not only model?????
             SGAnimation::animate(modelData);
+
+            // note from James: we used to re-throw these errors, and they
+            // were caught one level up as an SG_DEV_ALERT log message and
+            // loading an empty osg::Node for the entire model.
+            // Chosing instead to trap them there, since a single failed animation
+            // isn't necessarily a reason to abandon the model load.
         } catch (sg_exception& e) {
-            simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLModelLoad,
-                                   "Couldn't load animation " + animation_nodes[i]->getNameString()
-                                    + ":" + e.getFormattedMessage(),
+            simgear::reportFailure(simgear::LoadFailure::Misconfigured, simgear::ErrorCode::XMLModelLoad,
+                                   "Couldn't load animation " + animation_nodes[i]->getNameString() + ":" + e.getFormattedMessage(),
+                                   modelpath);
+        } catch (std::exception& e) { // needed because props throws std::runtime_error sometimes
+            simgear::reportFailure(simgear::LoadFailure::Misconfigured, simgear::ErrorCode::XMLModelLoad,
+                                   "Couldn't load animation " + animation_nodes[i]->getNameString() + ":" + e.what(),
                                    modelpath);
-            throw;
         }
     }
 
     animationcount += animation_nodes.size();
 
+    // restore the vertex order in case a submodel changed it.
+    options->setVertexOrderXYZ(currentVertexOrderXYZ);
+
     if (!needTransform && group->getNumChildren() < 2) {
         model = group->getChild(0);
         group->removeChild(model.get());
