@@ -39,9 +39,11 @@
 #include <simgear/bvh/BVHGroup.hxx>
 #include <simgear/bvh/BVHLineGeometry.hxx>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/math/interpolater.hxx>
 #include <simgear/props/condition.hxx>
 #include <simgear/props/props.hxx>
+
 #include <simgear/scene/material/EffectGeode.hxx>
 #include <simgear/scene/material/EffectCullVisitor.hxx>
 #include <simgear/scene/util/DeletionManager.hxx>
@@ -60,6 +62,7 @@
 
 #include "SGTranslateTransform.hxx"
 #include "SGMaterialAnimation.hxx"
+#include "SGPBRAnimation.hxx"
 #include "SGRotateTransform.hxx"
 #include "SGScaleTransform.hxx"
 #include "SGInteractionAnimation.hxx"
@@ -95,7 +98,7 @@ class LineCollector : public osg::NodeVisitor {
     };
 
 public:
-    LineCollector() : osg::NodeVisitor(osg::NodeVisitor::NODE_VISITOR, osg::NodeVisitor::TRAVERSE_ALL_CHILDREN) { }
+    LineCollector(bool orderXYZ, bool swapAxis) : osg::NodeVisitor(osg::NodeVisitor::NODE_VISITOR, osg::NodeVisitor::TRAVERSE_ALL_CHILDREN), _orderXYZ(orderXYZ), _swapAxis(swapAxis){ }
 
     virtual void apply(osg::Geode& geode)
     {
@@ -123,20 +126,53 @@ public:
     {
         return _lineSegments;
     }
-
+    // instead of using the lowest X to instead find the lowest of all (x,y,z) and use this
+    // see https://sourceforge.net/p/flightgear/codetickets/2706/
+    bool compareVec3(const osg::Vec3& v1, const osg::Vec3& v2) {
+        // compare to the nearest 0.01mm
+        if (abs(v2[0]-v1[0]) > 0.00001)
+            return v2[0] < v1[0];
+        else if (abs(v2[1] - v1[1]) > 0.00001)
+            return v2[1] < v1[1];
+        else 
+            return v2[2] < v1[2];
+    }
     void addLine(const osg::Vec3& v1, const osg::Vec3& v2)
     {
-        // Trick to get the ends in the right order.
-        // Use the x axis in the original coordinate system. Choose the
-        // most negative x-axis as the one pointing forward
         SGVec3f tv1(toSG(_matrix.preMult(v1)));
         SGVec3f tv2(toSG(_matrix.preMult(v2)));
-        if (tv1[0] > tv2[0])
-            _lineSegments.push_back(SGLineSegmentf(tv1, tv2));
-        else
-            _lineSegments.push_back(SGLineSegmentf(tv2, tv1));
-    }
 
+        if (_orderXYZ)
+        {
+            // Get the ends in the right order based on their 
+            // lowest coordinates in x,y,z
+            // This gives us a definitive vertex order in all cases
+            // whereas previously when X was equal the order would
+            // effectively be determined by the order of the vertices in the
+            // model file.
+            if (compareVec3(v1, v2))
+                addLineSegment(tv1, tv2);
+            else
+                addLineSegment(tv2, tv1);
+        }
+        else
+        {
+            // 2020.3 and prior: sort only by X
+            if (tv1[0] > tv2[0])
+                addLineSegment(tv1, tv2);
+            else
+                addLineSegment(tv2, tv1);
+        }
+    }
+    
+    // add a line segment handling axis swapping.
+    void addLineSegment(SGVec3f v1, SGVec3f v2)
+    {
+        if (_swapAxis)
+            _lineSegments.push_back(SGLineSegmentf(v2, v1));
+        else
+            _lineSegments.push_back(SGLineSegmentf(v1, v2));
+    }
     void addBVHElements(osg::Node& node, simgear::BVHLineGeometry::Type type)
     {
         if (_lineSegments.empty())
@@ -168,6 +204,8 @@ public:
 private:
     osg::Matrix _matrix;
     std::vector<SGLineSegmentf> _lineSegments;
+    bool _orderXYZ; /// 2020.3 and prior sorting vertex ordering rules (Only compare X component of vector)
+    bool _swapAxis; // invert the vector direction.
 };
 
 /**
@@ -300,10 +338,9 @@ read_value(const SGPropertyNode* configNode, SGPropertyNode* modelRoot,
     if (minClip > SGMiscd::min(SGLimitsd::min(), -SGLimitsd::max()) ||
         maxClip < SGLimitsd::max())
       value = new SGClipExpression<double>(value, minClip, maxClip);
-    
-    return value;
   }
-  return 0;
+
+  return value;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -426,12 +463,14 @@ struct DoDrawArraysVisitor : public osg::NodeVisitor {
 
 SGAnimation::SGAnimation(simgear::SGTransientModelData &modelData) :
   osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN),
+  _modelData(modelData),
   _found(false),
   _configNode(modelData.getConfigNode()),
   _modelRoot(modelData.getModelRoot())
 {
   _name = modelData.getConfigNode()->getStringValue("name", "");
   _enableHOT = modelData.getConfigNode()->getBoolValue("enable-hot", true);
+
   std::vector<SGPropertyNode_ptr> objectNames =
     modelData.getConfigNode()->getChildren("object-name");
   for (unsigned i = 0; i < objectNames.size(); ++i)
@@ -454,8 +493,11 @@ SGAnimation::~SGAnimation()
       }
       if (!info.empty())
       {
-          SG_LOG(SG_IO, SG_ALERT, "Could not find at least one of the following"
-                  " objects for animation: " << info);
+          reportFailure(LoadFailure::Misconfigured, ErrorCode::XMLModelLoad,
+                        "Could not find at least one of the following object for animation:" + info,
+                        SGPath::fromUtf8(_modelData.getPath()));
+          SG_LOG(SG_IO, SG_DEV_ALERT, "Could not find at least one of the following"
+                  " objects for animation: " << info << " in file: " << _modelData.getPath());
       }
   }
 }
@@ -485,6 +527,9 @@ SGAnimation::animate(simgear::SGTransientModelData &modelData)
   } else if (type == "material") {
     SGMaterialAnimation anim(modelData);
     anim.apply(modelData);
+  } else if (type == "pbr") {
+    SGPBRAnimation anim(modelData);
+    anim.apply(modelData);
   } else if (type == "noshadow") {
     SGShadowAnimation anim(modelData);
     anim.apply(modelData);
@@ -497,6 +542,9 @@ SGAnimation::animate(simgear::SGTransientModelData &modelData)
   } else if (type == "slider") {
     SGSliderAnimation anim(modelData);
     anim.apply(modelData.getNode());
+  } else if (type == "touch") {
+      SGTouchAnimation anim(modelData);
+      anim.apply(modelData.getNode());
   } else if (type == "range") {
     SGRangeAnimation anim(modelData);
     anim.apply(modelData);
@@ -668,7 +716,7 @@ public:
         _group(0)
     {
         if (name.empty())
-            SG_LOG(SG_IO, SG_WARN, "FindGroupVisitor: empty name provided");
+            SG_LOG(SG_IO, SG_DEV_WARN, "FindGroupVisitor: empty name provided");
     }
 
     osg::Group* getGroup() const
@@ -690,7 +738,7 @@ public:
             SG_LOG
             (
                 SG_IO,
-                SG_WARN,
+                SG_DEV_WARN,
                 "FindGroupVisitor: name not unique '" << _name << "'"
             );
     }
@@ -706,21 +754,24 @@ protected:
  * This function will take action when axis has an object-name tag and the corresponding object
  * can be found within the hierarchy.
  */
-bool SGAnimation::setCenterAndAxisFromObject(osg::Node *rootNode, SGVec3d& center, SGVec3d &axis, simgear::SGTransientModelData &modelData) const
+const SGLineSegment<double>* SGAnimation::setCenterAndAxisFromObject(osg::Node* rootNode,
+                                                                     SGVec3d& center, SGVec3d& axis, simgear::SGTransientModelData& modelData,
+                                                                     const std::string& axisName) const
 {
     std::string axis_object_name = std::string();
     bool can_warn = true;
 
-    if (_configNode->hasValue("axis/object-name"))
-    {
-        axis_object_name = _configNode->getStringValue("axis/object-name");
-    }
-    else if (!_configNode->getNode("axis")) {
-        axis_object_name = _configNode->getStringValue("object-name") + std::string("-axis");
+    const SGPropertyNode* axisNode = _configNode->getNode(axisName);
+
+    if (!axisNode) {
+        axis_object_name = _configNode->getStringValue("object-name") + std::string("-" + axisName);
         // for compatibility we will not warn if no axis object can be found when there was nothing 
         // specified - as the axis could just be the default at the origin
         // so if there is a [objectname]-axis use it, otherwise fallback to the previous behaviour
-        can_warn = false; 
+        can_warn = false;
+    }
+    else if (axisNode->hasValue("object-name")) {
+        axis_object_name = axisNode->getStringValue("object-name");
     }
 
     if (!axis_object_name.empty())
@@ -738,16 +789,44 @@ bool SGAnimation::setCenterAndAxisFromObject(osg::Node *rootNode, SGVec3d& cente
             FindGroupVisitor axis_object_name_finder(axis_object_name);
             rootNode->accept(axis_object_name_finder);
             osg::Group *object_group = axis_object_name_finder.getGroup();
-
             if (object_group)
             {
+                /*
+                 * work out which vertex sorting rule to use.
+                 * - specified in the axis node
+                 * - or when not specified use the <defaults> value in the options.
+                 */
+                bool orderXYZ = false; 
+
+                // check to see if this node specifies the vertex sorting rule
+                if (axisNode != nullptr && axisNode->hasChild("order-by-xyz"))
+                    orderXYZ = true;
+                else if (axisNode != nullptr && axisNode->hasChild("order-by-x"))
+                    orderXYZ = false;
+                else {
+                    // no local definition so use the <defaults> specified vertex order.
+                    const SGReaderWriterOptions* wOpts = dynamic_cast<const SGReaderWriterOptions*>(modelData.getOptions());
+                    if (wOpts)
+                        orderXYZ = wOpts->getVertexOrderXYZ();
+                }
+
+                /*
+                 * Check to see if we need to swap the axis direction (i.e. the 
+                 * collected line segment vertices).
+                 * This helps to avoid having to negate the rotations to get the
+                 * required direction.
+                 */
+                bool swapAxis = false;
+                if (axisNode != nullptr && axisNode->hasChild("swap-axis-direction")) {
+                    swapAxis = true;
+                }
                 /*
                  * we have found the object group (for the axis). This should be two vertices
                  * Now process this (with the line collector) to get the vertices.
                  * Once we have that we can then calculate the center and the affected axes.
                  */
                 object_group->setNodeMask(0xffffffff);
-                LineCollector lineCollector;
+                LineCollector lineCollector(orderXYZ, swapAxis);
                 object_group->accept(lineCollector);
                 std::vector<SGLineSegmentf> segs = lineCollector.getLineSegments();
 
@@ -768,49 +847,56 @@ bool SGAnimation::setCenterAndAxisFromObject(osg::Node *rootNode, SGVec3d& cente
                      */
                     object_group->setNodeMask(0);
                 }
-                else
-                    SG_LOG(SG_INPUT, SG_ALERT, "Could find a valid line segment for animation:  " << axis_object_name);
+                else {
+                    reportFailure(LoadFailure::Misconfigured, ErrorCode::XMLModelLoad,
+                                  "Could not find valid line segment for axis animation:" + axis_object_name,
+                                  SGPath::fromUtf8(_modelData.getPath()));
+                    SG_LOG(SG_IO, SG_DEV_ALERT, "Could not find a valid line segment for animation:  " << axis_object_name << " in file: " << _modelData.getPath());
+                }
+            }
+            else if (can_warn) {
+                reportFailure(LoadFailure::Misconfigured, ErrorCode::XMLModelLoad,
+                              "Could not find object for axis animation:" + axis_object_name,
+                              SGPath::fromUtf8(_modelData.getPath()));
+                SG_LOG(SG_IO, SG_DEV_ALERT, "Could not find at least one of the following objects for axis animation: " << axis_object_name << " in file: " << _modelData.getPath());
             }
-            else if (can_warn)
-                SG_LOG(SG_INPUT, SG_ALERT, "Could not find at least one of the following objects for axis animation: " << axis_object_name);
         }
         if (axisSegment)
         {
             center = 0.5*(axisSegment->getStart() + axisSegment->getEnd());
             axis = axisSegment->getEnd() - axisSegment->getStart();
-            return true;
+            return axisSegment;
         }
     }
-    return false;
+    return nullptr;
 }
 //------------------------------------------------------------------------------
 // factored out to share with SGKnobAnimation
-void SGAnimation::readRotationCenterAndAxis(osg::Node *_rootNode, SGVec3d& center,
-                                             SGVec3d& axis, simgear::SGTransientModelData &modelData) const
+void SGAnimation::readRotationCenterAndAxis(osg::Node* _rootNode, SGVec3d& center,
+                                            SGVec3d& axis, simgear::SGTransientModelData& modelData,
+                                            const std::string& centerName,
+                                            const std::string& axisName) const
 {
   center = SGVec3d::zeros();
-  if (setCenterAndAxisFromObject(_rootNode, center, axis, modelData))
-  {
+  if (setCenterAndAxisFromObject(_rootNode, center, axis, modelData, axisName)) {
       if (8 * SGLimitsd::min() < norm(axis))
           axis = normalize(axis);
       return;
   }
 
-  if( _configNode->hasValue("axis/x1-m") )
-  {
-    SGVec3d v1 = readVec3("axis", "1-m"), // axis/[xyz]1-m
-            v2 = readVec3("axis", "2-m"); // axis/[xyz]2-m
-    center = 0.5*(v1+v2);
-    axis = v2 - v1;
-  }
-  else
-  {
-    axis = readVec3("axis");
+  if (_configNode->hasValue(axisName + "/x1-m")) {
+      SGVec3d v1 = readVec3(axisName, "1-m"), // axis/[xyz]1-m
+          v2 = readVec3(axisName, "2-m");     // axis/[xyz]2-m
+      center = 0.5 * (v1 + v2);
+      axis = v2 - v1;
+  } else {
+      axis = readVec3(axisName);
   }
+
   if( 8 * SGLimitsd::min() < norm(axis) )
     axis = normalize(axis);
 
-  center = readVec3("center", "-m", center);
+  center = readVec3(centerName, "-m", center);
 }
 
 //------------------------------------------------------------------------------
@@ -943,19 +1029,33 @@ public:
 SGTranslateAnimation::SGTranslateAnimation(simgear::SGTransientModelData &modelData) :
   SGAnimation(modelData)
 {
-  _condition = getCondition();
-  SGSharedPtr<SGExpressiond> value;
-  value = read_value(modelData.getConfigNode(), modelData.getModelRoot(), "-m",
-                     -SGLimitsd::max(), SGLimitsd::max());
-  _animationValue = value->simplify();
-  if (_animationValue)
-    _initialValue = _animationValue->getValue();
-  else
-    _initialValue = 0;
+	_condition = getCondition();
+	SGSharedPtr<SGExpressiond> value;
 
-      SGVec3d _center;
-  if (modelData.getNode() && !setCenterAndAxisFromObject(modelData.getNode(), _center, _axis, modelData))
-    _axis = readTranslateAxis(modelData.getConfigNode());
+	value = read_value(modelData.getConfigNode(), modelData.getModelRoot(), "-m",
+		-SGLimitsd::max(), SGLimitsd::max());
+	if (!value) {
+		throw sg_format_exception("Invalid translate expression", "Value is not readable");
+	}
+	_animationValue = value->simplify();
+	if (_animationValue)
+		_initialValue = _animationValue->getValue();
+	else
+		_initialValue = 0;
+
+	SGVec3d _center;
+
+	if (modelData.getNode())
+	{
+		auto segment = setCenterAndAxisFromObject(modelData.getNode(), _center, _axis, modelData);
+		if (segment) {
+			_center = segment->getStart();
+			_axis = segment->getEnd() - segment->getStart();
+		}
+		else {
+			_axis = readTranslateAxis(modelData.getConfigNode());
+		}
+	}
 }
 
 osg::Group*
@@ -966,6 +1066,7 @@ SGTranslateAnimation::createAnimationGroup(osg::Group& parent)
   if (_animationValue && !_animationValue->isConst()) {
     UpdateCallback* uc = new UpdateCallback(_condition, _animationValue);
     transform->setUpdateCallback(uc);
+    transform->_animationValue = _animationValue;
   }
   transform->setAxis(_axis);
   transform->setValue(_initialValue);
@@ -1129,16 +1230,8 @@ void SpinAnimCallback::operator()(osg::Node* node, osg::NodeVisitor* nv)
         double intPart;
         double rot = modf(rotation, &intPart);
         double angle = rot * 2.0 * osg::PI;
-        const SGVec3d& sgcenter = transform->getCenter();
-        const SGVec3d& sgaxis = transform->getAxis();
-        Matrixd mat = Matrixd::translate(-sgcenter[0], -sgcenter[1], -sgcenter[2])
-            * Matrixd::rotate(angle, sgaxis[0], sgaxis[1], sgaxis[2])
-            * Matrixd::translate(sgcenter[0], sgcenter[1], sgcenter[2])
-            * *cv->getModelViewMatrix();
-        ref_ptr<RefMatrix> refmat = new RefMatrix(mat);
-        cv->pushModelViewMatrix(refmat.get(), transform->getReferenceFrame());
+        transform->setAngleRad(angle);
         traverse(transform, nv);
-        cv->popModelViewMatrix();
     } else {
         traverse(transform, nv);
     }
@@ -1178,6 +1271,9 @@ SGRotateAnimation::SGRotateAnimation(simgear::SGTransientModelData &modelData) :
   SGSharedPtr<SGExpressiond> value;
   value = read_value(modelData.getConfigNode(), modelData.getModelRoot(), "-deg",
                      -SGLimitsd::max(), SGLimitsd::max());
+    if (!value) {
+        throw sg_format_exception("Invalid rotate expression", "INvalid value");
+    }
   _animationValue = value->simplify();
   if (_animationValue)
     _initialValue = _animationValue->getValue();
@@ -1818,6 +1914,7 @@ SGRangeAnimation::createAnimationGroup(osg::Group& parent)
 {
   osg::Group* group = new osg::Group;
   group->setName("range animation group");
+  SGSceneUserData::getOrCreateSceneUserData(group)->setLocation(getConfig()->getLocation());
 
   osg::LOD* lod = new osg::LOD;
   lod->setName("range animation node");
@@ -2024,6 +2121,9 @@ public:
 SGBlendAnimation::SGBlendAnimation(simgear::SGTransientModelData &modelData) :
     SGAnimation(modelData), _animationValue(read_value(modelData.getConfigNode(), modelData.getModelRoot(), "", 0, 1))
 {
+    if (!_animationValue) {
+        throw sg_format_exception("Invalid blend expression", "Invalid value");
+    }
 }
 
 osg::Group*
@@ -2317,8 +2417,8 @@ protected:
 class SGTexTransformAnimation::UpdateCallback :
   public osg::StateAttribute::Callback {
 public:
-  UpdateCallback(const SGCondition* condition) :
-    _condition(condition)
+  UpdateCallback(const SGCondition* condition, osg::Uniform* uniform) :
+    _condition(condition), _uniform(uniform)
   {
       setName("SGTexTransformAnimation::UpdateCallback");
   }
@@ -2335,6 +2435,11 @@ public:
     TransformList::const_iterator i;
     for (i = _transforms.begin(); i != _transforms.end(); ++i)
       i->transform->transform(texMat->getMatrix());
+
+    // Update the core profile uniform as well
+    if (_uniform) {
+      _uniform->set(texMat->getMatrix());
+    }
   }
   void appendTransform(Transform* transform, SGExpressiond* value)
   {
@@ -2352,6 +2457,7 @@ private:
   TransformList _transforms;
   SGSharedPtr<const SGCondition> _condition;
   osg::Matrix _matrix;
+  osg::ref_ptr<osg::Uniform> _uniform;
 };
 
 SGTexTransformAnimation::SGTexTransformAnimation(simgear::SGTransientModelData &modelData) :
@@ -2365,9 +2471,24 @@ SGTexTransformAnimation::createAnimationGroup(osg::Group& parent)
   osg::Group* group = new osg::Group;
   group->setName("texture transform group");
   osg::StateSet* stateSet = group->getOrCreateStateSet();
-  stateSet->setDataVariance(osg::Object::STATIC/*osg::Object::DYNAMIC*/);  
+  stateSet->setDataVariance(osg::Object::STATIC);
+
+  // Core profile alternative to osg::TexMat. There is no fixed-function
+  // texture matrix available, so use an uniform instead.
+  //
+  // NOTE: Uniforms are not positional like StateAttributes. The top-level
+  // osg::StateSet (usually the one from an osg::Camera) must set this uniform
+  // to the identity matrix. Otherwise, the texture matrix will "leak" into
+  // other nodes that do not belong to this animation. This issue is caused by
+  // the matrix not being set to identity after the objects in the animation
+  // group are done being rendered.
+  osg::Uniform* texmat_uniform = stateSet->getOrCreateUniform(
+    "fg_TextureMatrix", osg::Uniform::FLOAT_MAT4);
+  // Initialize to identity
+  texmat_uniform->set(osg::Matrix());
+
   osg::TexMat* texMat = new osg::TexMat;
-  UpdateCallback* updateCallback = new UpdateCallback(getCondition());
+  UpdateCallback* updateCallback = new UpdateCallback(getCondition(), texmat_uniform);
   // interpret the configs ...
   std::string type = getType();
 
@@ -2388,12 +2509,13 @@ SGTexTransformAnimation::createAnimationGroup(osg::Group& parent)
         appendTexRotate(*transformConfigs[i], updateCallback);
       else if (subtype == "textrapezoid")
         appendTexTrapezoid(*transformConfigs[i], updateCallback);
-      else
-        SG_LOG(SG_INPUT, SG_ALERT,
-               "Ignoring unknown texture transform subtype");
+      else {
+        SG_LOG(SG_IO, SG_DEV_ALERT,
+               "Ignoring unknown texture transform subtype in file: " << _modelData.getPath());
+      }
     }
   } else {
-    SG_LOG(SG_INPUT, SG_ALERT, "Ignoring unknown texture transform type");
+    SG_LOG(SG_IO, SG_DEV_ALERT, "Ignoring unknown texture transform type in file: " << _modelData.getPath());
   }
 
   texMat->setUpdateCallback(updateCallback);
@@ -2485,3 +2607,16 @@ SGTexTransformAnimation::appendTexTrapezoid( const SGPropertyNode& cfg,
   trapezoid->setValue(cfg.getDoubleValue("starting-position", 0));
   updateCallback->appendTransform(trapezoid, readValue(cfg));
 }
+
+SGSharedPtr<SGExpressiond const> TransformExpression(osg::Transform* transform)
+{
+    SGSharedPtr<SGExpressiond const>    ret;
+    if (auto rot_anim_transform = dynamic_cast<SGRotAnimTransform*>(transform)) {
+        ret = rot_anim_transform->_animationValue;
+    }
+    else if (auto translate_transform = dynamic_cast<SGTranslateTransform*>(transform)) {
+        ret = translate_transform->_animationValue;
+    }
+    return ret;
+}
+
