@@ -42,17 +42,9 @@ SGLoadTexture2D(bool staticTexture, const std::string& path,
 {
   osg::ref_ptr<osg::Image> image;
   if (options)
-#if OSG_VERSION_LESS_THAN(3,4,0)
-      image = osgDB::readImageFile(path, options);
-#else
       image = osgDB::readRefImageFile(path, options);
-#endif
   else
-#if OSG_VERSION_LESS_THAN(3,4,0)
-      image = osgDB::readImageFile(path);
-#else
       image = osgDB::readRefImageFile(path);
-#endif
 
   osg::ref_ptr<osg::Texture2D> texture = new osg::Texture2D;
   texture->setImage(image);
@@ -153,11 +145,7 @@ Texture2D* TextureUpdateVisitor::textureReplace(int unit, const StateAttribute*
     if (fullLiveryFile.empty() || fullLiveryFile == *fullFilePath)
         return 0;
 
-#if OSG_VERSION_LESS_THAN(3,4,0)
-    Image* newImage = readImageFile(fullLiveryFile);
-#else
     osg::ref_ptr<Image> newImage = readRefImageFile(fullLiveryFile);
-#endif
     if (!newImage)
         return 0;
 
@@ -167,11 +155,7 @@ Texture2D* TextureUpdateVisitor::textureReplace(int unit, const StateAttribute*
         return 0;
 
     newTexture->setImage(newImage);
-#if OSG_VERSION_LESS_THAN(3,4,0)
-    if (newImage->valid())
-#else
     if (newImage.valid())
-#endif
     {
         newTexture->setMaxAnisotropy(SGSceneFeatures::instance()->getTextureFilter());
     }
@@ -234,7 +218,7 @@ public:
     typedef std::map<string, SGPropertyNode_ptr> EffectMap;
     using SplicingVisitor::apply;
     MakeEffectVisitor(const SGReaderWriterOptions* options = 0)
-        : _options(options)
+        : _options(options), _modelPath(SGPath{})
     {
     }
     virtual void apply(osg::Group& node);
@@ -246,10 +230,17 @@ public:
         _currentEffectParent = effect;
     }
     SGPropertyNode* getDefaultEffect() { return _currentEffectParent; }
+
+    void setModelPath(const SGPath& p)
+    {
+        _modelPath = p;
+    }
+
 protected:
     EffectMap _effectMap;
     SGPropertyNode_ptr _currentEffectParent;
     osg::ref_ptr<const SGReaderWriterOptions> _options;
+    SGPath _modelPath;
 };
 
 void MakeEffectVisitor::apply(osg::Group& node)
@@ -287,7 +278,7 @@ void MakeEffectVisitor::apply(osg::Geode& geode)
     makeParametersFromStateSet(ssRoot, ss);
     SGPropertyNode_ptr effectRoot = new SGPropertyNode;
     effect::mergePropertyTrees(effectRoot, ssRoot, _currentEffectParent);
-    Effect* effect = makeEffect(effectRoot, true, _options.get());
+    Effect* effect = makeEffect(effectRoot, true, _options.get(), _modelPath);
     EffectGeode* eg = dynamic_cast<EffectGeode*>(&geode);
     if (eg) {
         eg->setEffect(effect);
@@ -314,17 +305,19 @@ void MakeEffectVisitor::apply(osg::Geode& geode)
 
 namespace
 {
-class DefaultEffect : public simgear::Singleton<DefaultEffect>
+
+class DefaultEffect
 {
 public:
-    DefaultEffect()
+    DefaultEffect(const std::string& effectPath = "Effects/model-default")
     {
         _effect = new SGPropertyNode;
         makeChild(_effect.ptr(), "inherits-from")
-            ->setStringValue("Effects/model-default");
+            ->setStringValue(effectPath);
     }
     virtual ~DefaultEffect() {}
     SGPropertyNode* getEffect() { return _effect.ptr(); }
+
 protected:
     SGPropertyNode_ptr _effect;
 };
@@ -332,7 +325,8 @@ protected:
 
 ref_ptr<Node> instantiateEffects(osg::Node* modelGroup,
                                  PropertyList& effectProps,
-                                 const SGReaderWriterOptions* options)
+                                 const SGReaderWriterOptions* options,
+                                 const SGPath& modelPath)
 {
     SGPropertyNode_ptr defaultEffectPropRoot;
     MakeEffectVisitor visitor(options);
@@ -354,16 +348,19 @@ ref_ptr<Node> instantiateEffects(osg::Node* modelGroup,
         configNode->removeChild("default");
         configNode->removeChildren("object-name");
     }
-    if (!defaultEffectPropRoot)
-        defaultEffectPropRoot = DefaultEffect::instance()->getEffect();
+    if (!defaultEffectPropRoot) {
+        defaultEffectPropRoot = DefaultEffect(options->getDefaultEffect()).getEffect();
+    }
     visitor.setDefaultEffect(defaultEffectPropRoot.ptr());
+    visitor.setModelPath(modelPath);
     modelGroup->accept(visitor);
     osg::NodeList& result = visitor.getResults();
     return ref_ptr<Node>(result[0].get());
 }
 
 ref_ptr<Node> instantiateMaterialEffects(osg::Node* modelGroup,
-                                        const SGReaderWriterOptions* options)
+                                         const SGReaderWriterOptions* options,
+                                         const SGPath& modelPath)
 {
 
     SGPropertyNode_ptr effect;
@@ -371,26 +368,25 @@ ref_ptr<Node> instantiateMaterialEffects(osg::Node* modelGroup,
 
     if (options->getMaterialLib()) {
       const SGGeod loc = SGGeod(options->getLocation());
-      SGMaterialCache* matcache = options->getMaterialLib()->generateMatCache(loc);
+      osg::ref_ptr<SGMaterialCache> matcache = options->getMaterialLib()->generateMatCache(loc, options);
       SGMaterial* mat = matcache->find(options->getMaterialName());
-      delete matcache;
 
       if (mat) {
         effect = new SGPropertyNode();
         makeChild(effect, "inherits-from")->setStringValue(mat->get_effect_name());
       } else {
-        effect = DefaultEffect::instance()->getEffect();
+        effect = DefaultEffect().getEffect();
         SG_LOG( SG_TERRAIN, SG_ALERT, "Unable to get effect for " << options->getMaterialName());
         simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::LoadEffectsShaders,
                                "Unable to get effect for material:" + options->getMaterialName());
       }
     } else {
-      effect = DefaultEffect::instance()->getEffect();
+      effect = DefaultEffect().getEffect();
     }
 
     effect->addChild("default")->setBoolValue(true);
     effectProps.push_back(effect);
-    return instantiateEffects(modelGroup, effectProps, options);
+    return instantiateEffects(modelGroup, effectProps, options, modelPath);
 }
 
 }
