@@ -21,6 +21,7 @@
 
 #include <boost/algorithm/string.hpp>
 
+#include <osg/Version>
 #include <osg/PagedLOD>
 #include <osg/ProxyNode>
 #include <osgDB/ReadFile>
@@ -28,12 +29,13 @@
 #include <osgDB/Registry>
 
 #include <simgear/constants.h>
+#include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/misc/ResourceManager.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
-#include <simgear/scene/model/model.hxx>
 #include <simgear/scene/model/ModelRegistry.hxx>
+#include <simgear/scene/model/model.hxx>
 #include <simgear/scene/util/SGReaderWriterOptions.hxx>
-#include <simgear/misc/ResourceManager.hxx>
 
 #include "SGReaderWriterXML.hxx"
 
@@ -68,7 +70,7 @@ void SGModelLib::setPanelFunc(panel_func pf)
   static_panelFunc = pf;
 }
 
-std::string SGModelLib::findDataFile(const std::string& file, 
+std::string SGModelLib::findDataFile(const std::string& file,
   const osgDB::Options* opts,
   SGPath currentPath)
 {
@@ -76,13 +78,27 @@ std::string SGModelLib::findDataFile(const std::string& file,
     return file;
   SGPath p = ResourceManager::instance()->findPath(file, currentPath);
   if (p.exists()) {
-    return p.local8BitStr();
+    return p.utf8Str();
   }
 
   // finally hand on to standard OSG behaviour
   return osgDB::findDataFile(file, opts);
 }
 
+std::string SGModelLib::findDataFile(const SGPath& file,
+  const osgDB::Options* opts,
+  SGPath currentPath)
+{
+  SGPath p = ResourceManager::instance()->findPath(file.utf8Str(), currentPath);
+  if (p.exists()) {
+    return p.utf8Str();
+  }
+
+  // finally hand on to standard OSG behaviour
+  return osgDB::findDataFile(file.utf8Str(), opts);
+}
+
+
 SGModelLib::SGModelLib()
 {
 }
@@ -97,8 +113,10 @@ osg::Node* loadFile(const string& path, SGReaderWriterOptions* options)
 {
     using namespace osg;
     using namespace osgDB;
-    if (boost::iends_with(path, ".ac"))
+    if (boost::iends_with(path, ".ac") || boost::iends_with(path, ".obj")) {
         options->setInstantiateEffects(true);
+    }
+
     ref_ptr<Node> model = readRefNodeFile(path, options);
     if (!model)
         return 0;
@@ -111,18 +129,23 @@ osg::Node*
 SGModelLib::loadModel(const string &path,
                        SGPropertyNode *prop_root,
                        SGModelData *data,
-                       bool load2DPanels)
+                       bool load2DPanels,
+                       bool autoTooltipsMaster,
+                       int autoTooltipsMasterMax)
 {
     osg::ref_ptr<SGReaderWriterOptions> opt;
     opt = SGReaderWriterOptions::copyOrCreate(osgDB::Registry::instance()->getOptions());
     opt->getDatabasePathList().push_front( osgDB::getFilePath(path) );
     opt->setPropertyNode(prop_root ? prop_root: static_propRoot.get());
     opt->setModelData(data);
-    
+
     if (load2DPanels) {
        opt->setLoadPanel(static_panelFunc);
     }
-    
+
+    opt->setAutoTooltipsMaster(autoTooltipsMaster);
+    opt->setAutoTooltipsMasterMax(autoTooltipsMasterMax);
+
     osg::Node *n = loadFile(path, opt.get());
     if (n && n->getName().empty())
         n->setName("Direct loaded model \"" + path + "\"");
@@ -138,15 +161,17 @@ SGModelLib::loadDeferredModel(const string &path, SGPropertyNode *prop_root,
     proxyNode->setLoadingExternalReferenceMode(osg::ProxyNode::DEFER_LOADING_TO_DATABASE_PAGER);
     proxyNode->setFileName(0, path);
 
-
     osg::ref_ptr<SGReaderWriterOptions> opt;
     opt = SGReaderWriterOptions::copyOrCreate(osgDB::Registry::instance()->getOptions());
     opt->getDatabasePathList().push_front( osgDB::getFilePath(path) );
     opt->setPropertyNode(prop_root ? prop_root: static_propRoot.get());
     opt->setModelData(data);
     opt->setLoadPanel(static_panelFunc);
-    if (SGPath(path).lower_extension() == "ac")
+    std::string lext = SGPath(path).lower_extension();
+    if ((lext == "ac") || (lext == "obj")) {
         opt->setInstantiateEffects(true);
+    }
+
     if (!prop_root || prop_root->getBoolValue("/sim/rendering/cache", true))
         opt->setObjectCacheHint(osgDB::Options::CACHE_ALL);
     else
@@ -156,29 +181,69 @@ SGModelLib::loadDeferredModel(const string &path, SGPropertyNode *prop_root,
     return proxyNode;
 }
 
+
+/*
+ * Load a set of models at different LOD range_nearest
+ *
+ */
 osg::PagedLOD*
-SGModelLib::loadPagedModel(const string &path, SGPropertyNode *prop_root,
-                           SGModelData *data)
+SGModelLib::loadPagedModel(SGPropertyNode *prop_root, SGModelData *data, SGModelLOD model_lods)
 {
+    unsigned int simple_models = 0;
     osg::PagedLOD *plod = new osg::PagedLOD;
-    plod->setName("Paged LOD for \"" + path + "\"");
-    plod->setFileName(0, path);
-    plod->setRange(0, 0.0, 50.0*SG_NM_TO_METER);
-    plod->setMinimumExpiryTime( 0, prop_root->getDoubleValue("/sim/rendering/plod-minimum-expiry-time-secs", 180.0 ) );
 
     osg::ref_ptr<SGReaderWriterOptions> opt;
     opt = SGReaderWriterOptions::copyOrCreate(osgDB::Registry::instance()->getOptions());
     opt->setPropertyNode(prop_root ? prop_root: static_propRoot.get());
     opt->setModelData(data);
     opt->setLoadPanel(static_panelFunc);
-    if (SGPath(path).lower_extension() == "ac")
-        opt->setInstantiateEffects(true);
     if (!prop_root || prop_root->getBoolValue("/sim/rendering/cache", true))
         opt->setObjectCacheHint(osgDB::Options::CACHE_ALL);
     else
         opt->setObjectCacheHint(osgDB::Options::CACHE_NONE);
+
+    for(unsigned int i = 0; i < model_lods.getNumLODs(); i++) {
+      SGModelLOD::ModelLOD lod = model_lods.getModelLOD(i);
+      plod->setName("Paged LOD for \"" + lod.path + "\"");
+      plod->setFileName(i, lod.path);
+      plod->setRange(i, lod.min_range, lod.max_range);
+      plod->setMinimumExpiryTime(i, prop_root->getDoubleValue("/sim/rendering/plod-minimum-expiry-time-secs", 180.0 ) );
+
+      std::string lext = SGPath(lod.path).lower_extension();
+      if ((lext == "ac") || (lext == "obj")) {
+        simple_models++;
+      }
+    }
+
+    // If all we have are simple models, then we can instantiate effects in
+    // the loader.
+    if (simple_models == model_lods.getNumLODs()) opt->setInstantiateEffects(true);
+
     plod->setDatabaseOptions(opt.get());
+
     return plod;
 }
 
+osg::PagedLOD*
+SGModelLib::loadPagedModel(const string &path, SGPropertyNode *prop_root,
+                           SGModelData *data)
+{
+    SGModelLOD model_lods;
+    model_lods.insert(path, 0.0, 50.0*SG_NM_TO_METER);
+    return SGModelLib::loadPagedModel(prop_root, data, model_lods);
+}
+
+osg::PagedLOD*
+SGModelLib::loadPagedModel(std::vector<string> paths, SGPropertyNode *prop_root,
+                           SGModelData *data)
+{
+    SGModelLOD model_lods;
+    for(unsigned int i = 0; i < paths.size(); i++) {
+      // We don't have any range data, so simply set them all up to full range.
+      // Some other code will update the LoD ranges later.  (AIBase::updateLOD)
+      model_lods.insert(paths[i], 0.0, 50.0*SG_NM_TO_METER);
+    }
+    return SGModelLib::loadPagedModel(prop_root, data, model_lods);
+}
+
 // end of modellib.cxx
