@@ -103,13 +103,7 @@ SGVasiDrawable::drawImplementation(osg::RenderInfo& renderInfo) const
     draw(eyePoint, _lights[i]);
 }
 
-osg::BoundingBox
-#if OSG_VERSION_LESS_THAN(3,3,2)
-SGVasiDrawable::computeBound()
-#else
-SGVasiDrawable::computeBoundingBox()
-#endif
-const
+osg::BoundingBox SGVasiDrawable::computeBoundingBox() const
 {
   osg::BoundingBox bb;
   for (unsigned i = 0; i < _lights.size(); ++i)
