@@ -42,11 +42,11 @@
 #include <signal.h>             // signal()
 #include <string.h>
 
-#include <iostream>
-#include <fstream>
-#include <string>
-#include <map>
 #include <algorithm>
+#include <fstream>
+#include <iostream>
+#include <map>
+#include <string>
 
 #include <simgear/version.h>
 
@@ -66,10 +66,10 @@
 #include <simgear/io/HTTPRepository.hxx>
 #include <simgear/io/DNSClient.hxx>
 #include <simgear/structure/exception.hxx>
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
 
 using namespace simgear;
-using namespace std;
+using std::string;
 
 namespace UpdateInterval
 {
@@ -79,7 +79,7 @@ namespace UpdateInterval
     static const double FailedAttempt     = 10*60;
 }
 
-typedef map<string,time_t> TileAgeCache;
+typedef std::map<std::string, time_t> TileAgeCache;
 
 ///////////////////////////////////////////////////////////////////////////////
 // helper functions ///////////////////////////////////////////////////////////
@@ -146,6 +146,37 @@ public:
     Status _status;
 };
 
+std::ostream& operator << (std::ostream& out, const SyncItem::Type& t)
+{
+    if (t == SyncItem::Stop)            return out << "Stop";
+    if (t == SyncItem::Tile)            return out << "Tile";
+    if (t == SyncItem::AirportData)     return out << "AirportData";
+    if (t == SyncItem::SharedModels)    return out << "SharedModels";
+    if (t == SyncItem::AIData)          return out << "AIData";
+    if (t == SyncItem::OSMTile)         return out << "OSMTile";
+    return out << ((int) t);
+}
+
+std::ostream& operator << (std::ostream& out, const SyncItem::Status& s)
+{
+    if (s == SyncItem::Invalid)     return out << "Invalid";
+    if (s == SyncItem::Waiting)     return out << "Waiting";
+    if (s == SyncItem::Cached)      return out << "Cached";
+    if (s == SyncItem::Updated)     return out << "Updated";
+    if (s == SyncItem::NotFound)    return out << "NotFound";
+    if (s == SyncItem::Failed)      return out << "Failed";
+    return out << ((int) s);
+}
+
+std::ostream& operator << (std::ostream& out, const SyncItem& s)
+{
+    return out << "SyncItem:{_dir="
+            << s._dir << " _type="
+            << s._type << " _status="
+            << s._status
+            << "}";
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 
 /**
@@ -178,7 +209,7 @@ static const unsigned int NUM_SYNC_SLOTS = 4;
 
 /**
  * @brief translate a sync item type into one of the available slots.
- * This provides the scheduling / balancing / prioritising between slots.
+ * This provides the scheduling / balancing / prioritizing between slots.
  */
 static unsigned int syncSlotForType(SyncItem::Type ty)
 {
@@ -287,6 +318,11 @@ public:
       _isAutomaticServer = (server == "automatic");
    }
 
+   void setDNSServer(const std::string& nameserver)
+   {
+       _dnsNameserver = nameserver;
+   }
+
    void setDNSDN( const std::string & dn )
    {
      _dnsdn = simgear::strutils::strip(dn);
@@ -353,7 +389,7 @@ public:
 
     void drainWaitingTiles();
 
-    // commond helpers between both internal and external models
+    // common helpers between both internal and external models
 
     SyncItem::Status isPathCached(const SyncItem& next) const;
     void updated(SyncItem item, bool isNewDirectory);
@@ -384,6 +420,7 @@ public:
     string _sceneryVersion;
     string _protocol;
     string _dnsdn;
+    string _dnsNameserver;
 
     TerrasyncThreadState _state;
     mutable std::mutex _stateLock;
@@ -507,7 +544,7 @@ std::string SGTerraSync::WorkerThread::dnsSelectServerForService(const std::stri
     naptrRequest->qflags = "U";
     DNS::Request_ptr r(naptrRequest);
 
-    DNS::Client dnsClient;
+    DNS::Client dnsClient{_dnsNameserver};
     dnsClient.makeRequest(r);
     SG_LOG(SG_TERRASYNC,SG_DEBUG,"DNS NAPTR query for '" << _dnsdn << "' '" << naptrRequest->qservice << "'" );
     while (!r->isComplete() && !r->isTimeout()) {
@@ -521,7 +558,7 @@ std::string SGTerraSync::WorkerThread::dnsSelectServerForService(const std::stri
 
     // walk through responses, they are ordered by 1. order and 2. preference
     // For now, only take entries with lowest order
-    // TODO: try all available servers in the order given by preferenc and order
+    // TODO: try all available servers in the order given by preference and order
     int order = naptrRequest->entries[0]->order;
 
     // get all servers with this order and the same (for now only lowest preference)
@@ -551,8 +588,17 @@ std::string SGTerraSync::WorkerThread::dnsSelectServerForService(const std::stri
     // now pick a random entry from the available servers
     auto idx = static_cast<int>(sg_random() * availableServers.size());
     const auto server = availableServers.at(idx)->regexp;
-    SG_LOG(SG_TERRASYNC, SG_INFO, "picking entry # " << idx << ", server is " << server.substr(6, server.length() - 7););
-    return server.substr(6, server.length() - 7);
+    std::string ret = server.substr(6, server.length() - 7);
+    SG_LOG(SG_TERRASYNC, SG_INFO, "service=" << service << " returning entry # " << idx << ": " << ret);
+    {
+        std::string env = "SIMGEAR_TERRASYNC_SERVER_" + service;
+        const char* ret = getenv(env.c_str());
+        if (ret) {
+            SG_LOG(SG_TERRASYNC, SG_INFO, "service=" << service << " overriding to return " << env << " = " << ret);
+            return ret;
+        }
+    }
+    return ret;
 }
 
 void SGTerraSync::WorkerThread::run()
@@ -702,7 +748,7 @@ bool SGTerraSync::WorkerThread::beginSyncTile(SyncSlot& slot)
 {
     // avoid 404 requests by doing a sync which excludes all paths
     // except our tile path. In the case of a missing 1x1 tile, we will
-    // stop becuase all directories are filtered out, which is what we want
+    // stop because all directories are filtered out, which is what we want
 
     auto comps = strutils::split(slot.currentItem._dir, "/");
     if (comps.size() != 3) {
@@ -852,7 +898,9 @@ SyncItem::Status SGTerraSync::WorkerThread::isPathCached(const SyncItem& next) c
         ii = _notFoundItems.find( next._dir );
         // Invalid means 'not cached', otherwise we want to return to
         // higher levels the cache status
-        return (ii == _notFoundItems.end()) ? SyncItem::Invalid : SyncItem::NotFound;
+        SyncItem::Status ret = (ii == _notFoundItems.end()) ? SyncItem::Invalid : SyncItem::NotFound;
+        SG_LOG(SG_TERRASYNC, SG_DEBUG, "next=" << next << " returning ret=" << ret);
+        return ret;
     }
 
     // check if the path still physically exists. This is needed to
@@ -860,11 +908,14 @@ SyncItem::Status SGTerraSync::WorkerThread::isPathCached(const SyncItem& next) c
     SGPath p(_local_dir);
     p.append(next._dir);
     if (!p.exists()) {
+        SG_LOG(SG_TERRASYNC, SG_DEBUG, "next=" << next << " returning SyncItem::Invalid");
         return SyncItem::Invalid;
     }
 
     time_t now = time(0);
-    return (ii->second > now) ? SyncItem::Cached : SyncItem::Invalid;
+    SyncItem::Status ret = (ii->second > now) ? SyncItem::Cached : SyncItem::Invalid;
+    SG_LOG(SG_TERRASYNC, SG_DEBUG, "next=" << next << " returning ret=" << ret);
+    return ret;
 }
 
 void SGTerraSync::WorkerThread::fail(SyncItem failedItem)
@@ -873,7 +924,7 @@ void SGTerraSync::WorkerThread::fail(SyncItem failedItem)
     time_t now = time(0);
 
     if (_osmCityServer.empty() && (failedItem._type == SyncItem::OSMTile)) {
-        // don't count these as errors, otherwise normla sync will keep
+        // don't count these as errors, otherwise normal sync will keep
         // being abandoned
     } else {
         _state._consecutive_errors++;
@@ -890,8 +941,8 @@ void SGTerraSync::WorkerThread::fail(SyncItem failedItem)
 
 void SGTerraSync::WorkerThread::notFound(SyncItem item)
 {
-    // treat not found as authorative, so use the same cache expiry
-    // as succesful download. Important for MP models and similar so
+    // treat not found as authoritative, so use the same cache expiry
+    // as successful download. Important for MP models and similar so
     // we don't spam the server with lookups for models that don't
     // exist
 
@@ -932,15 +983,20 @@ void SGTerraSync::WorkerThread::drainWaitingTiles()
     while (!waitingTiles.empty()) {
         SyncItem next = waitingTiles.pop_front();
         SyncItem::Status cacheStatus = isPathCached(next);
+        SG_LOG(SG_TERRASYNC, SG_INFO, "next._type=" << next._type
+                << " next._dir=" << next._dir
+                << " cacheStatus=" << cacheStatus
+                );
         if (cacheStatus != SyncItem::Invalid) {
             incrementCacheHits();
-            SG_LOG(SG_TERRASYNC, SG_BULK, "\nTerraSync Cache hit for: '" << next._dir << "'");
+            SG_LOG(SG_TERRASYNC, SG_BULK, "TerraSync Cache hit for: '" << next._dir << "'");
             next._status = cacheStatus;
             _freshTiles.push_back(next);
             continue;
         }
 
         const auto slot = syncSlotForType(next._type);
+        SG_LOG(SG_TERRASYNC, SG_INFO, "adding to _syncSlots slot=" << slot);
         _syncSlots[slot].queue.push_back(next);
     }
 }
@@ -994,9 +1050,15 @@ void SGTerraSync::WorkerThread::initCompletedTilesPersistentCache() {
 
   for (int i = 0; i < cacheRoot->nChildren(); ++i) {
     SGPropertyNode *entry = cacheRoot->getChild(i);
-    bool isNotFound = (strcmp(entry->getName(), "not-found") == 0);
+    bool isNotFound = (entry->getNameString() == "not-found");
     string tileName = entry->getStringValue("path");
     time_t stamp = entry->getIntValue("stamp");
+    SG_LOG(SG_TERRASYNC, SG_DEBUG, "tileName=" << tileName
+            << " isNotFound=" << isNotFound
+            << " stamp=" << stamp
+            << " now=" << now
+            << " stamp<now=" << (stamp < now)
+            );
     if (stamp < now) {
       continue;
     }
@@ -1012,11 +1074,13 @@ void SGTerraSync::WorkerThread::initCompletedTilesPersistentCache() {
 void SGTerraSync::WorkerThread::writeCompletedTilesPersistentCache() const {
   // cache is disabled
   if (_persistentCachePath.isNull()) {
+    SG_LOG(SG_TERRASYNC, SG_DEBUG, "_persistentCachePath.isNull()");
     return;
   }
 
   sg_ofstream f(_persistentCachePath, std::ios::trunc);
   if (!f.is_open()) {
+    SG_LOG(SG_TERRASYNC, SG_DEBUG, "!f.is_open() _persistentCachePath=" << _persistentCachePath);
     return;
   }
 
@@ -1067,12 +1131,10 @@ void SGTerraSync::setRoot(SGPropertyNode_ptr root)
 {
     if (!root) {
         _terraRoot.clear();
-        _renderingRoot.clear();
         return;
     }
 
     _terraRoot = root->getNode("/sim/terrasync",true);
-    _renderingRoot = root->getNode("/sim/rendering", true);
 }
 
 void SGTerraSync::init()
@@ -1115,6 +1177,16 @@ void SGTerraSync::reinit()
         _workerThread->setSceneryVersion( _terraRoot->getStringValue("scenery-version","ws20") );
         _workerThread->setOSMCityVersion(_terraRoot->getStringValue("osm2city-version", "o2c"));
         _workerThread->setProtocol( _terraRoot->getStringValue("protocol","") );
+
+        if (_terraRoot->hasChild("dns-server")) {
+            auto ns = _terraRoot->getStringValue("dns-server");
+            if (ns == "google") {
+                ns = "8.8.8.8";
+            }
+            
+            SG_LOG(SG_TERRASYNC,SG_INFO,"DNS server override:" << ns);
+            _workerThread->setDNSServer(ns);
+        }
 #if 1
         // leave it hardcoded for now, not sure about the security implications for now
         _workerThread->setDNSDN( "terrasync.flightgear.org");
@@ -1192,7 +1264,6 @@ void SGTerraSync::unbind()
     _stalledNode.clear();
     _activeNode.clear();
     _cacheHits.clear();
-    _renderingRoot.clear();
     _busyNode.clear();
     _updateCountNode.clear();
     _errorCountNode.clear();
@@ -1270,27 +1341,9 @@ void SGTerraSync::syncAirportsModels()
   _workerThread->request(a);
 }
 
-string_list SGTerraSync::getSceneryPathSuffixes() const
-{
-    string_list scenerySuffixes;
-
-    for (auto node : _renderingRoot->getChildren("scenery-path-suffix")) {
-        if (node->getBoolValue("enabled", true) && node->hasValue("name")) {
-            scenerySuffixes.push_back(node->getStringValue("name"));
-        }
-    }
-
-    if (scenerySuffixes.empty()) {
-        // if preferences didn't load, use some default
-        scenerySuffixes = {"Objects", "Terrain"}; // default values
-    }
-
-    return scenerySuffixes;
-}
-
 bool isOSMSuffix(const std::string& suffix)
 {
-    return (suffix == "Buildings") || (suffix == "Roads") || (suffix == "Pylons") || (suffix == "Details");
+    return (suffix == "Buildings") || (suffix == "Roads") || (suffix == "Pylons") || (suffix == "Details") || (suffix == "Trees");
 }
 
 void SGTerraSync::syncAreaByPath(const std::string& aPath)
@@ -1299,7 +1352,11 @@ void SGTerraSync::syncAreaByPath(const std::string& aPath)
         return;
     }
 
-    for (const auto& suffix : getSceneryPathSuffixes()) {
+    if (_sceneryPathSuffixes.empty()) {
+        throw sg_exception("SGTerraSync::syncAreaByPath called before scenery suffixes were defined.");
+    }
+
+    for (const auto& suffix : _sceneryPathSuffixes) {
         const auto dir = suffix + "/" + aPath;
         if (_workerThread->isDirActive(dir)) {
             continue;
@@ -1316,15 +1373,24 @@ bool SGTerraSync::scheduleTile(const SGBucket& bucket)
     return true;
 }
 
+void SGTerraSync::setSceneryPathSuffixes(const string_list& suffixes)
+{
+    _sceneryPathSuffixes = suffixes;
+}
+
 bool SGTerraSync::isTileDirPending(const std::string& sceneryDir) const
+{
+    return innerIsTileDirPending(sceneryDir, false);
+}
+
+bool SGTerraSync::innerIsTileDirPending(const std::string& sceneryDir, bool isOSM) const
 {
     if (!_workerThread->isRunning()) {
         return false;
     }
 
-    for (const auto& suffix : getSceneryPathSuffixes()) {
-        // don't wait on OSM dirs, even if enabled
-        if (isOSMSuffix(suffix)) {
+    for (const auto& suffix : _sceneryPathSuffixes) {
+        if (isOSM != isOSMSuffix(suffix)) {
             continue;
         }
 
@@ -1337,6 +1403,11 @@ bool SGTerraSync::isTileDirPending(const std::string& sceneryDir) const
     return false;
 }
 
+bool SGTerraSync::isTileDirPendingOSM(const std::string& sceneryDir) const
+{
+    return innerIsTileDirPending(sceneryDir, true);
+}
+
 void SGTerraSync::scheduleDataDir(const std::string& dataDir)
 {
     if (!_workerThread->isRunning()) {
@@ -1376,10 +1447,10 @@ void SGTerraSync::writeWarningFile(const SGPath& sceneryDir)
     os << "Any changes you make here or in sub-folders will be overwritten when TerraSync\n";
     os << "downloads updates.\n";
     os << "\n";
-    os << "To use custom scenery or data with FlightGear, put it in a differnet location\n";
+    os << "To use custom scenery or data with FlightGear, put it in a different location\n";
     os << "on your computer, then add the location using either the launcher 'Add-ons' page, or by\n";
     os << "passing '--fg-scenery=<location>' on the command line.";
-    os << endl;
+    os << std::endl;
 }
 
 
