@@ -2,7 +2,7 @@
 //
 // Written for the new SoundSystem by Erik Hofman, October 2009
 //
-// Copyright (C) 2009 Erik Hofman <erik@ehofman.com>
+// Copyright (C) 2009-2019 Erik Hofman <erik@ehofman.com>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -31,44 +31,31 @@
 #include "soundmgr.hxx"
 #include "sample_group.hxx"
 
-SGSampleGroup::SGSampleGroup () :
-    _smgr(NULL),
-    _refname(""),
-    _active(false),
-    _changed(false),
-    _pause(false),
-    _volume(1.0),
-    _tied_to_listener(false),
-    _velocity(SGVec3d::zeros()),
-    _orientation(SGQuatd::zeros())
+SGSampleGroup::SGSampleGroup ()
 {
-    _samples.clear();
 }
 
 SGSampleGroup::SGSampleGroup ( SGSoundMgr *smgr,
                                const std::string &refname ):
     _smgr(smgr),
-    _refname(refname),
-    _active(false),
-    _changed(false),
-    _pause(false),
-    _volume(1.0),
-    _tied_to_listener(false),
-    _velocity(SGVec3d::zeros()),
-    _orientation(SGQuatd::zeros())
+    _refname(refname)
 {
     _smgr->add(this, refname);
-    _samples.clear();
 }
 
 SGSampleGroup::~SGSampleGroup ()
 {
     _active = false;
     stop();
-    _smgr = 0;
+
+    // destroying (or really, shutting down) FGFX removes all
+    // its SGSamples (created by XMLSound instances). So we potentially
+    // have a lot of samples we need to stop+destroy here.
+    for (auto current : _removed_samples) {
+        _smgr->sample_destroy(current.get());
+    }
 }
 
-#include <stdio.h>
 void SGSampleGroup::cleanup_removed_samples()
 {
     // Delete any buffers that might still be in use.
@@ -110,7 +97,8 @@ void SGSampleGroup::check_playing_sample(SGSoundSample *sample)
         sample->no_valid_source();
         _smgr->release_source( sample->get_source() );
         _smgr->release_buffer( sample );
-        remove( sample->get_sample_name() );
+        // Delayed removal because this is called while iterating over _samples
+        remove( sample->get_sample_name(), true );
     } else if ( sample->has_changed() ) {
         if ( !sample->is_playing() ) {
             // a request to stop playing the sound has been filed.
@@ -137,8 +125,8 @@ void SGSampleGroup::update( double dt ) {
         _changed = false;
     }
 
-    for (auto current =_samples.begin(); current !=_samples.end(); ++current) {
-        SGSoundSample *sample = current->second;
+    for (auto current : _samples) {
+        SGSoundSample *sample = current.second;
 
         if ( !sample->is_valid_source() && sample->is_playing() && !sample->test_out_of_range()) {
             start_playing_sample(sample);
@@ -148,6 +136,11 @@ void SGSampleGroup::update( double dt ) {
         }
         testForMgrError("update");
     }
+
+    for (const auto& refname : _refsToRemoveFromSamplesMap) {
+        _samples.erase(refname);
+    }
+    _refsToRemoveFromSamplesMap.clear();
 }
 
 // add a sound effect, return true if successful
@@ -166,8 +159,14 @@ bool SGSampleGroup::add( SGSharedPtr<SGSoundSample> sound,
 
 
 // remove a sound effect, return true if successful
-bool SGSampleGroup::remove( const std::string &refname ) {
-
+//
+// TODO: the 'delayedRemoval' boolean parameter has been specifically added
+// for check_playing_sample() which is only called while update() is iterating
+// over _samples. It may be better style to remove this parameter and add a
+// separate member function for use in check_playing_sample(), whose behavior
+// would be the same as when passing delayedRemoval=true here.
+bool SGSampleGroup::remove( const std::string &refname, bool delayedRemoval )
+{
     auto sample_it = _samples.find( refname );
     if ( sample_it == _samples.end() ) {
         // sample was not found
@@ -177,7 +176,12 @@ bool SGSampleGroup::remove( const std::string &refname ) {
     if ( sample_it->second->is_valid_buffer() )
         _removed_samples.push_back( sample_it->second );
 
-    _samples.erase( sample_it );
+    // Do not erase within the loop in update()
+    if (delayedRemoval) {
+        _refsToRemoveFromSamplesMap.push_back(refname);
+    } else {
+        _samples.erase(refname);
+    }
 
     return true;
 }
@@ -196,12 +200,12 @@ bool SGSampleGroup::exists( const std::string &refname ) {
 
 
 // return a pointer to the SGSoundSample if the specified sound exists
-// in the sound manager system, otherwise return NULL
+// in the sound manager system, otherwise return nullptr
 SGSoundSample *SGSampleGroup::find( const std::string &refname ) {
     auto sample_it = _samples.find( refname );
     if ( sample_it == _samples.end() ) {
         // sample was not found
-        return NULL;
+        return nullptr;
     }
 
     return sample_it->second;
@@ -212,9 +216,8 @@ void
 SGSampleGroup::stop ()
 {
     _pause = true;
-    for (auto current =_samples.begin(); current !=_samples.end(); ++current) {
-        SGSoundSample *sample = current->second;
-        _smgr->sample_destroy( sample );
+    for (auto current : _samples) {
+        _smgr->sample_destroy( current.second );
     }
 }
 
@@ -224,12 +227,11 @@ SGSampleGroup::suspend ()
 {
     if (_active && _pause == false) {
         _pause = true;
-        for (auto current =_samples.begin(); current !=_samples.end(); ++current) {
 #ifdef ENABLE_SOUND
-            SGSoundSample *sample = current->second;
-            _smgr->sample_suspend( sample );
-#endif
+        for (auto current : _samples) {
+            _smgr->sample_suspend( current.second );
         }
+#endif
         testForMgrError("suspend");
     }
 }
@@ -240,9 +242,8 @@ SGSampleGroup::resume ()
 {
     if (_active && _pause == true) {
 #ifdef ENABLE_SOUND
-        for (auto current =_samples.begin(); current !=_samples.end(); ++current) {
-            SGSoundSample *sample = current->second;
-            _smgr->sample_resume( sample );
+        for (auto current : _samples) {
+            _smgr->sample_resume( current.second );
         }
         testForMgrError("resume");
 #endif
@@ -257,7 +258,7 @@ bool SGSampleGroup::play( const std::string &refname,
 {
     SGSoundSample *sample = find( refname );
 
-    if ( sample == NULL ) {
+    if ( sample == nullptr ) {
         return false;
     }
 
@@ -270,7 +271,7 @@ bool SGSampleGroup::play( const std::string &refname,
 bool SGSampleGroup::is_playing( const std::string& refname ) {
     SGSoundSample *sample = find( refname );
 
-    if ( sample == NULL ) {
+    if ( sample == nullptr ) {
         return false;
     }
 
@@ -281,7 +282,7 @@ bool SGSampleGroup::is_playing( const std::string& refname ) {
 bool SGSampleGroup::stop( const std::string& refname ) {
     SGSoundSample *sample  = find( refname );
 
-    if ( sample == NULL ) {
+    if ( sample == nullptr ) {
         return false;
     }
 
@@ -306,24 +307,57 @@ void SGSampleGroup::update_pos_and_orientation() {
     SGQuatd hlOr = SGQuatd::fromLonLat(_base_pos);
     SGQuatd ec2body = hlOr*_orientation;
 
-    SGVec3f velocity = SGVec3f::zeros();
+    SGVec3d velocity = SGVec3d::zeros();
     if ( _velocity[0] || _velocity[1] || _velocity[2] ) {
-       velocity = toVec3f( hlOr.backTransform(_velocity*SG_FEET_TO_METER) );
+       velocity = hlOr.backTransform(_velocity*SG_FEET_TO_METER);
     }
 
-    for (auto current =_samples.begin(); current !=_samples.end(); ++current ) {
-        SGSoundSample *sample = current->second;
+    float speed = 0.0f;
+    double mAngle = SGD_PI_2;
+    if (!_tied_to_listener) {
+        const float Rvapor = 461.52; // Water vapor: individual gas constant
+        const float Rair = 287.5;    // Air: individual gas constant
+        const float y = 1.402;       // Air: Ratio of specific heat
+
+        float T = 273.16 + _degC;    // Kelvin
+        float hr = 0.01 * _humidity;
+        float R = Rair + 0.04f * hr * Rvapor;
+        float sound_speed = sqrtf( y * R * T ); // m/s
+
+        speed = length(velocity);
+        _mach = speed / sound_speed;
+        mAngle = asin(1.0/_mach);
+    }
+
+    for (auto current : _samples ) {
+        SGSoundSample *sample = current.second;
         sample->set_master_volume( _volume );
         sample->set_orientation( _orientation );
         sample->set_rotation( ec2body );
         sample->set_position(base_position);
-        sample->set_velocity( velocity );
+        sample->set_velocity( toVec3f(velocity) );
+        sample->set_atmosphere( _degC, _humidity, _pressure );
 
-        // Test if a sample is farther away than max distance, if so
-        // stop the sound playback and free it's source.
         if (!_tied_to_listener) {
             sample->update_pos_and_orientation();
+
+            // Sample position relative to the listener, including the
+            // sample offset relative to the base position.
+            // Same coordinate system as OpenGL; y=up, x=right, z=back
             SGVec3d position = sample->get_position() - smgr_position;
+            if (_mach > 1.0) {
+                _mOffset_m = position[BACK];
+
+                // Skip the slant calculation for angles greater than 89 deg
+                // to avoid instability
+                if (mAngle < 1.553343) {
+                    _mOffset_m -= position[UP] / tan(mAngle);
+                }
+                _mInCone = (_mOffset_m > 0.01) ? true : false;
+            }
+
+            // Test if a sample is farther away than max distance, if so
+            // stop the sound playback and free it's source.
             float max2 = sample->get_max_dist() * sample->get_max_dist();
             float dist2 = position[0]*position[0]
                           + position[1]*position[1] + position[2]*position[2];
@@ -368,7 +402,7 @@ void SGSampleGroup::update_sample_config( SGSoundSample *sample )
 
 bool SGSampleGroup::testForError(void *p, std::string s)
 {
-   if (p == NULL) {
+   if (p == nullptr) {
       SG_LOG( SG_SOUND, SG_ALERT, "Error (sample group): " << s);
       return true;
    }
