@@ -8,7 +8,7 @@
 // Modified for the new SoundSystem by Erik Hofman, October 2009
 //
 // Copyright (C) 2001  Curtis L. Olson - http://www.flightgear.org/~curt
-// Copyright (C) 2009 Erik Hofman <erik@ehofman.com>
+// Copyright (C) 2009-2019 Erik Hofman <erik@ehofman.com>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -48,11 +48,11 @@
 
 
 // We keep track of the emitters ourselves.
-typedef std::map < unsigned int, aax::Emitter > source_map;
+using source_map = std::map< unsigned int, aax::Emitter >;
 
 // The AeonWave class keeps track of the buffers, so use a reference instead.
-typedef std::map < unsigned int, aax::Buffer& > buffer_map;
-typedef std::map < std::string, SGSharedPtr<SGSampleGroup> > sample_group_map;
+using buffer_map = std::map< unsigned int, aax::Buffer& >;
+using sample_group_map = std::map< std::string, SGSharedPtr<SGSampleGroup> >;
 
 #ifndef NDEBUG
 # define TRY(a)	if ((a) == 0) printf("%i: %s\n", __LINE__, d->_aax.strerror())
@@ -64,21 +64,10 @@ typedef std::map < std::string, SGSharedPtr<SGSampleGroup> > sample_group_map;
 class SGSoundMgr::SoundManagerPrivate
 {
 public:
-    SoundManagerPrivate() :
-        _absolute_pos(SGVec3d::zeros()),
-        _base_pos(SGVec3d::zeros()),
-        _orientation(SGQuatd::zeros()),
-        _buffer_id(0),
-        _source_id(0)
-    {
-    }
+    SoundManagerPrivate() = default;
 
     ~SoundManagerPrivate()
     {
-        for (auto it = _devices.begin(); it != _devices.end(); ++it) {
-            free((void*)*it);
-        }
-        _devices.clear();
         _sample_groups.clear();
     }
 
@@ -88,7 +77,7 @@ public:
     
     void update_pos_and_orientation()
     {
-        SGVec3d sgv_at = _orientation.backTransform(-SGVec3d::e3());
+        SGVec3d sgv_at = _orientation.backTransform(SGVec3d::e3());
         SGVec3d sgv_up = _orientation.backTransform(SGVec3d::e2());
         SGVec3d pos = SGVec3d::zeros();
 
@@ -100,11 +89,13 @@ public:
     aax::AeonWave _aax;
     aax::Matrix64 _mtx;
 
-    SGVec3d _absolute_pos;
-    SGVec3d _base_pos;
-    SGQuatd _orientation;
+    SGVec3d _absolute_pos = SGVec3d::zeros();
+    SGVec3d _base_pos = SGVec3d::zeros();
+    SGQuatd _orientation = SGQuatd::zeros();
 
-    unsigned int _buffer_id;
+    float _sound_velocity = SPEED_OF_SOUND;
+
+    unsigned int _buffer_id = 0;
     buffer_map _buffers;
     aax::Buffer nullBuffer;
     aax::Buffer& get_buffer(unsigned int id) {
@@ -114,7 +105,7 @@ public:
         return nullBuffer;
     }
 
-    unsigned int _source_id;
+    unsigned int _source_id = 0;
     source_map _sources;
     aax::Emitter nullEmitter;
     aax::Emitter& get_emitter(unsigned int id) {
@@ -125,8 +116,6 @@ public:
     }
 
     sample_group_map _sample_groups;
-
-    std::vector<const char*> _devices;
 };
 
 
@@ -135,15 +124,7 @@ public:
 //
 
 // constructor
-SGSoundMgr::SGSoundMgr() :
-    _active(false),
-    _changed(true),
-    _volume(0.0),
-    _velocity(SGVec3d::zeros()),
-    _bad_doppler(false),
-    _renderer("unknown"),
-    _vendor("unknown")
-{
+SGSoundMgr::SGSoundMgr() {
     d.reset(new SoundManagerPrivate);
     d->_base_pos = SGVec3d::fromGeod(_geod_pos);
 }
@@ -197,12 +178,12 @@ void SGSoundMgr::init()
         TRY( d->_aax.set(dsp) );
 
         dsp = aax::dsp(d->_aax, AAX_DISTANCE_FILTER);
-        TRY( dsp.set(AAX_AL_INVERSE_DISTANCE_CLAMPED) );
+        TRY( dsp.set(AAX_ISO9613_DISTANCE) );
         TRY( d->_aax.set(dsp) );
 
         dsp = aax::dsp(d->_aax, AAX_VELOCITY_EFFECT);
         TRY( dsp.set(AAX_DOPPLER_FACTOR, 1.0f) );
-        TRY( dsp.set(AAX_SOUND_VELOCITY, 340.3f) );
+        TRY( dsp.set(AAX_SOUND_VELOCITY, _sound_velocity) );
         TRY( d->_aax.set(dsp) );
         testForError("scenery setup");
 
@@ -235,10 +216,8 @@ void SGSoundMgr::activate()
     if ( is_working() ) {
         _active = true;
                 
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->activate();
+        for ( auto current : d->_sample_groups ) {
+            current.second->activate();
         }
     }
 #endif
@@ -249,10 +228,8 @@ void SGSoundMgr::stop()
 {
 #ifdef ENABLE_SOUND
     // first stop all sample groups
-    for ( auto current = d->_sample_groups.begin();
-               current != d->_sample_groups.end();  ++ current ) {
-        SGSampleGroup *sgrp = current->second;
-        sgrp->stop();
+    for ( auto current : d->_sample_groups ) {
+        current.second->stop();
     }
 
     d->_buffer_id = 0;
@@ -263,7 +240,7 @@ void SGSoundMgr::stop()
 
     if (is_working()) {
         _active = false;
-        TRY( d->_aax.set(AAX_STOPPED) );
+        TRY( d->_aax.set(AAX_PROCESSED) );
 
         _renderer = "unknown";
         _vendor = "unknown";
@@ -275,10 +252,8 @@ void SGSoundMgr::suspend()
 {
 #ifdef ENABLE_SOUND
     if (is_working()) {
-        for (auto current = d->_sample_groups.begin();
-                  current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->stop();
+        for (auto current : d->_sample_groups ) {
+            current.second->stop();
         }
         _active = false;
     }
@@ -289,10 +264,8 @@ void SGSoundMgr::resume()
 {
 #ifdef ENABLE_SOUND
     if (is_working()) {
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->resume();
+        for ( auto current : d->_sample_groups ) {
+            current.second->resume();
         }
         _active = true;
     }
@@ -308,10 +281,8 @@ void SGSoundMgr::update( double dt )
             d->update_pos_and_orientation();
         }
 
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->update(dt);
+        for ( auto current : d->_sample_groups ) {
+            current.second->update(dt);
         }
 
         if (_changed) {
@@ -319,12 +290,6 @@ void SGSoundMgr::update( double dt )
             TRY( dsp.set(AAX_GAIN, _volume) );
             TRY( d->_aax.set(dsp) );
 
-#if 0
-            // TODO: altitude dependent
-            dsp = d->_aax.get(AAX_VELOCITY_EFFECT);
-            TRY( dsp.set(AAX_SOUND_VELOCITY, 340.3f) );
-            TRY( d->_aax.set(dsp) );
-#endif
             aax::Matrix64 mtx = d->_mtx;
             mtx.inverse();
             TRY( d->_aax.sensor_matrix(mtx) );
@@ -337,6 +302,14 @@ void SGSoundMgr::update( double dt )
             aax::Vector vel( toVec3f(velocity).data() );
             TRY( d->_aax.sensor_velocity(vel) );
 
+            if (fabsf(_sound_velocity - d->_sound_velocity) > 10.0f) {
+                d->_sound_velocity = _sound_velocity;
+
+                dsp = aax::dsp(d->_aax, AAX_VELOCITY_EFFECT);
+                TRY( dsp.set(AAX_SOUND_VELOCITY, _sound_velocity) );
+                TRY( d->_aax.set(dsp) );
+            }
+
             testForError("update");
             _changed = false;
         }
@@ -383,21 +356,21 @@ bool SGSoundMgr::exists( const std::string &refname ) {
 
 
 // return a pointer to the SGSampleGroup if the specified sound exists
-// in the sound manager system, otherwise return NULL
+// in the sound manager system, otherwise return nullptr
 SGSampleGroup *SGSoundMgr::find( const std::string &refname, bool create ) {
     auto sample_grp_it = d->_sample_groups.find( refname );
     if ( sample_grp_it == d->_sample_groups.end() ) {
         // sample group was not found.
         if (create) {
-            SGSampleGroup* sgrp = new SGSampleGroup(this, refname);
+            SGSampleGroup *sgrp = new SGSampleGroup(this, refname);
             add( sgrp, refname );
             return sgrp;
         }
         else 
-            return NULL;
+            return nullptr;
     }
 
-    return sample_grp_it->second;
+    return sample_grp_it->second.get();
 }
 
 
@@ -423,11 +396,8 @@ void SGSoundMgr::release_source( unsigned int source )
     if ( source_it != d->_sources.end() )
     {
         aax::Emitter& emitter = source_it->second;
-        enum aaxState state = emitter.state();
-        if (state == AAX_PLAYING || state == AAX_SUSPENDED) {
-           TRY( emitter.set(AAX_STOPPED) );
-           TRY( d->_aax.remove(emitter) );
-        }
+        TRY( emitter.set(AAX_PROCESSED) );
+        TRY( d->_aax.remove(emitter) );
         TRY( emitter.remove_buffer() );
         d->_sources.erase(source_it);
     }
@@ -444,7 +414,7 @@ unsigned int  SGSoundMgr::request_buffer(SGSoundSample *sample)
         std::string sample_name = sample->get_sample_name();
 
         aax::Buffer& buf = d->_aax.buffer(sample_name);
-        if (!buf) {
+        if (sample->is_file() && !buf) {
             SG_LOG(SG_SOUND, SG_ALERT,
                    "Unable to create buffer: " << sample_name);
             sample->set_buffer( SGSoundMgr::FAILED_BUFFER );
@@ -574,7 +544,7 @@ void SGSoundMgr::sample_play( SGSoundSample *sample )
 
     aax::dsp dsp = emitter.get(AAX_DISTANCE_FILTER);
     TRY( dsp.set(AAX_ROLLOFF_FACTOR, 0.3f) );
-    TRY( dsp.set(AAX_AL_INVERSE_DISTANCE_CLAMPED) );
+    TRY( dsp.set(AAX_ISO9613_DISTANCE) );
     TRY( emitter.set(dsp) );
 
     TRY( emitter.set(AAX_LOOPING, sample->is_looping()) );
@@ -595,8 +565,7 @@ void SGSoundMgr::sample_stop( SGSoundSample *sample )
         if ( sample->is_looping() && !stopped) {
 #ifdef ENABLE_SOUND
             aax::Emitter& emitter = d->get_emitter(source);
-            TRY( emitter.set(AAX_STOPPED) );
-            TRY( d->_aax.remove(emitter) );
+            TRY( emitter.set(AAX_PROCESSED) );
 #endif          
             stopped = is_sample_stopped(sample);
         }
@@ -615,8 +584,7 @@ void SGSoundMgr::sample_destroy( SGSoundSample *sample )
         unsigned int source = sample->get_source();
         if ( sample->is_playing() ) {
             aax::Emitter& emitter = d->get_emitter(source);
-            TRY( emitter.set(AAX_STOPPED) );
-            TRY( d->_aax.remove(emitter) );
+            TRY( emitter.set(AAX_PROCESSED) );
         }
         release_source( source );
 #endif
@@ -635,7 +603,7 @@ bool SGSoundMgr::is_sample_stopped(SGSoundSample *sample)
     assert(sample->is_valid_source());
     aax::Emitter& emitter = d->get_emitter(sample->get_source());
     int result = emitter.state();
-    return (result == AAX_STOPPED);
+    return (result == AAX_PROCESSED);
 #else
     return true;
 #endif
@@ -646,39 +614,49 @@ void SGSoundMgr::update_sample_config( SGSoundSample *sample, SGVec3d& position,
     aax::Emitter& emitter = d->get_emitter(sample->get_source());
     aax::dsp dsp;
 
-    aax::Vector64 pos = position.data();
-    aax::Vector ori = orientation.data();
-    aax::Vector vel = velocity.data();
+    if (emitter != d->nullEmitter)
+    {
+        aax::Vector64 pos = position.data();
+        aax::Vector ori = orientation.data();
+        aax::Vector vel = velocity.data();
 
-    aax::Matrix64 mtx(pos, ori);
-    TRY( emitter.matrix(mtx) );
-    TRY( emitter.velocity(vel) );
+        aax::Matrix64 mtx(pos, ori);
+        TRY( emitter.matrix(mtx) );
+        TRY( emitter.velocity(vel) );
 
-    dsp = emitter.get(AAX_VOLUME_FILTER);
-    TRY( dsp.set(AAX_GAIN, sample->get_volume()) );
-    TRY( emitter.set(dsp) );
-
-    dsp = emitter.get(AAX_PITCH_EFFECT);
-    TRY( dsp.set(AAX_PITCH, sample->get_pitch()) );
-    TRY( emitter.set(dsp) );
-
-    if ( sample->has_static_data_changed() ) {
-        dsp = emitter.get(AAX_ANGULAR_FILTER);
-        TRY( dsp.set(AAX_INNER_ANGLE, sample->get_innerangle()) );
-        TRY( dsp.set(AAX_OUTER_ANGLE, sample->get_outerangle()) );
-        TRY( dsp.set(AAX_OUTER_GAIN, sample->get_outergain()) );
+        dsp = emitter.get(AAX_VOLUME_FILTER);
+        TRY( dsp.set(AAX_GAIN, sample->get_volume()) );
         TRY( emitter.set(dsp) );
 
-        dsp = emitter.get(AAX_DISTANCE_FILTER);
-        TRY( dsp.set(AAX_REF_DISTANCE, sample->get_reference_dist()) );
-        TRY( dsp.set(AAX_MAX_DISTANCE, sample->get_max_dist()) );
+        dsp = emitter.get(AAX_PITCH_EFFECT);
+        TRY( dsp.set(AAX_PITCH, sample->get_pitch()) );
         TRY( emitter.set(dsp) );
+
+        if ( sample->has_static_data_changed() ) {
+            dsp = emitter.get(AAX_DIRECTIONAL_FILTER);
+            TRY( dsp.set(AAX_INNER_ANGLE, sample->get_innerangle(), AAX_DEGREES) );
+            TRY( dsp.set(AAX_OUTER_ANGLE, sample->get_outerangle(), AAX_DEGREES) );
+            TRY( dsp.set(AAX_OUTER_GAIN, sample->get_outergain()) );
+            TRY( emitter.set(dsp) );
+
+            dsp = emitter.get(AAX_DISTANCE_FILTER);
+            TRY( dsp.set(AAX_REF_DISTANCE, sample->get_reference_dist()) );
+            TRY( dsp.set(AAX_MAX_DISTANCE, sample->get_max_dist()) );
+            TRY( dsp.set(AAX_RELATIVE_HUMIDITY, sample->get_humidity()) );
+            TRY( dsp.set(AAX_TEMPERATURE, sample->get_temperature(),
+                                          AAX_DEGREES_CELSIUS) );
+            TRY( emitter.set(dsp) );
+       }
+    }
+    else {
+        SG_LOG( SG_SOUND, SG_ALERT, "Error: source: " << sample->get_source() << " not found");
     }
 }
 
 
-vector<const char*> SGSoundMgr::get_available_devices()
+vector<std::string> SGSoundMgr::get_available_devices()
 {
+    vector<std::string> devices;
 #ifdef ENABLE_SOUND
     std::string on = " on ";
     std::string colon = ": ";
@@ -690,18 +668,19 @@ vector<const char*> SGSoundMgr::get_available_devices()
                 else if (*r) name += on + r;
                 else if (*i) name += colon + i;
 
-                d->_devices.push_back( strdup(name.c_str()) );
+                devices.push_back( name );
             }
         }
     }
+    testForError("get_available_devices");
 #endif
-    return d->_devices;
+    return devices;
 }
 
 
 bool SGSoundMgr::testForError(void *p, std::string s)
 {
-   if (p == NULL) {
+   if (p == nullptr) {
       SG_LOG( SG_SOUND, SG_ALERT, "Error: " << s);
       return true;
    }
@@ -724,7 +703,7 @@ bool SGSoundMgr::testForError(std::string s, std::string name)
 
 bool SGSoundMgr::is_working() const 
 {
-    return ((const void*)d->_aax != NULL ? true : false);
+    return ((const void*)d->_aax != nullptr ? true : false);
 }
 
 const SGQuatd& SGSoundMgr::get_orientation() const
