@@ -38,7 +38,6 @@
 
 #include "soundmgr.hxx"
 #include "readwav.hxx"
-#include "soundmgr_openal_private.hxx"
 #include "sample_group.hxx"
 
 #include <simgear/sg_inlines.h>
@@ -46,15 +45,29 @@
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
 
+#if defined(__APPLE__)
+# include <OpenAL/al.h>
+# include <OpenAL/alc.h>
+#elif defined(OPENALSDK)
+# include <al.h>
+# include <alc.h>
+#else
+# include <AL/al.h>
+# include <AL/alc.h>
+#endif
+
 using std::vector;
 
 
 #define MAX_SOURCES	128
-#define SPEED_OF_SOUND	340.4f
 
 #ifndef ALC_ALL_DEVICES_SPECIFIER
 # define ALC_ALL_DEVICES_SPECIFIER	0x1013
 #endif
+#ifndef AL_EXT_SOURCE_RADIUS
+#define AL_EXT_SOURCE_RADIUS 1
+#define AL_SOURCE_RADIUS		0x1031
+#endif
 #ifndef AL_FORMAT_MONO_MULAW_EXT
 # define AL_FORMAT_MONO_MULAW_EXT	0x10014
 #endif
@@ -65,24 +78,35 @@ using std::vector;
 # define AL_UNPACK_BLOCK_ALIGNMENT_SOFT	0x200C
 #endif
 
+
+struct refUint {
+    unsigned int refctr;
+    ALuint id;
+
+    refUint() { refctr = 0; id = (ALuint)-1; };
+    refUint(ALuint i) { refctr = 1; id = i; };
+    ~refUint() {};
+};
+
+using buffer_map = std::map < std::string, refUint >;
+using sample_group_map = std::map < std::string, SGSharedPtr<SGSampleGroup> >;
+
+inline bool isNaN(float *v) {
+   return (SGMisc<float>::isNaN(v[0]) || SGMisc<float>::isNaN(v[1]) || SGMisc<float>::isNaN(v[2]));
+}
+
+
 class SGSoundMgr::SoundManagerPrivate
 {
 public:
-    SoundManagerPrivate() :
-        _device(NULL),
-        _context(NULL),
-        _absolute_pos(SGVec3d::zeros()),
-        _base_pos(SGVec3d::zeros()),
-        _orientation(SGQuatd::zeros())
-    {
-        
-        
-    }
+    SoundManagerPrivate() = default;
+    ~SoundManagerPrivate() = default;
     
     void init()
     {
         _at_up_vec[0] = 0.0; _at_up_vec[1] = 0.0; _at_up_vec[2] = -1.0;
         _at_up_vec[3] = 0.0; _at_up_vec[4] = 1.0; _at_up_vec[5] = 0.0;
+        _source_radius = alIsExtensionPresent((ALchar *)"AL_EXT_SOURCE_RADIUS");
     }
     
     void update_pos_and_orientation()
@@ -110,18 +134,21 @@ public:
         _absolute_pos = _base_pos;
     }
         
-    ALCdevice *_device;
-    ALCcontext *_context;
+    ALCdevice *_device = nullptr;
+    ALCcontext *_context = nullptr;
     
     std::vector<ALuint> _free_sources;
     std::vector<ALuint> _sources_in_use;
     
-    SGVec3d _absolute_pos;
-    SGVec3d _base_pos;
-    SGQuatd _orientation;
+    SGVec3d _absolute_pos = SGVec3d::zeros();
+    SGVec3d _base_pos = SGVec3d::zeros();
+    SGQuatd _orientation = SGQuatd::zeros();
     // Orientation of the listener. 
     // first 3 elements are "at" vector, second 3 are "up" vector
     ALfloat _at_up_vec[6];
+
+    bool _bad_doppler = false;
+    bool _source_radius = false;
     
     sample_group_map _sample_groups;
     buffer_map _buffers;
@@ -133,19 +160,9 @@ public:
 //
 
 // constructor
-SGSoundMgr::SGSoundMgr() :
-    _active(false),
-    _changed(true),
-    _volume(0.0),
-    _velocity(SGVec3d::zeros()),
-    _bad_doppler(false),
-    _renderer("unknown"),
-    _vendor("unknown")
-{
+SGSoundMgr::SGSoundMgr() {
     d.reset(new SoundManagerPrivate);
     d->_base_pos = SGVec3d::fromGeod(_geod_pos);
-
-    _block_support = alIsExtensionPresent((ALchar *)"AL_SOFT_block_alignment");
 }
 
 // destructor
@@ -174,10 +191,10 @@ void SGSoundMgr::init()
     d->_sources_in_use.clear();
     d->_sources_in_use.reserve( MAX_SOURCES );
 
-    ALCdevice *device = NULL;
+    ALCdevice *device = nullptr;
     const char* devname = _device_name.c_str();
     if (_device_name == "")
-        devname = NULL; // use default device
+        devname = nullptr; // use default device
     else
     {
         // try non-default device
@@ -185,13 +202,15 @@ void SGSoundMgr::init()
     }
 
     if ((!devname)||(testForError(device, "Audio device not available, trying default.")) ) {
-        device = alcOpenDevice(NULL);
+        // REVIEW: Memory Leak - 26 bytes in 1 blocks are still reachable
+        device = alcOpenDevice(nullptr);
         if (testForError(device, "Default audio device not available.") ) {
            return;
         }
     }
 
-    ALCcontext *context = alcCreateContext(device, NULL);
+    // REVIEW: Memory Leak - 64 bytes in 1 blocks are still reachable
+    ALCcontext *context = alcCreateContext(device, nullptr);
     testForALCError("context creation.");
     if ( testForError(context, "Unable to create a valid context.") ) {
         alcCloseDevice (device);
@@ -205,7 +224,7 @@ void SGSoundMgr::init()
         return;
     }
 
-    if (d->_context != NULL)
+    if (d->_context != nullptr)
     {
         SG_LOG(SG_SOUND, SG_ALERT, "context is already assigned");
     }
@@ -220,7 +239,9 @@ void SGSoundMgr::init()
     alListenerfv( AL_VELOCITY, SGVec3f::zeros().data() );
 
     alDopplerFactor(1.0f);
-    alDopplerVelocity(SPEED_OF_SOUND);
+    alDopplerVelocity(_sound_velocity);
+
+    _block_support = alIsExtensionPresent((ALchar *)"AL_SOFT_block_alignment");
 
     // gain = AL_REFERENCE_DISTANCE / (AL_REFERENCE_DISTANCE +
     //        AL_ROLLOFF_FACTOR * (distance - AL_REFERENCE_DISTANCE));
@@ -250,10 +271,10 @@ void SGSoundMgr::init()
 
     if (_vendor == "Creative Labs Inc.") {
        alDopplerFactor(100.0f);
-       _bad_doppler = true;
+       d->_bad_doppler = true;
     } else if (_vendor == "OpenAL Community" && _renderer == "OpenAL Soft") {
        alDopplerFactor(100.0f);
-       _bad_doppler = true;
+       d->_bad_doppler = true;
     }
 
     if (d->_free_sources.empty()) {
@@ -284,10 +305,8 @@ void SGSoundMgr::activate()
     if ( is_working() ) {
         _active = true;
                 
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->activate();
+        for ( auto current : d->_sample_groups ) {
+            current.second->activate();
         }
     }
 #endif
@@ -298,10 +317,8 @@ void SGSoundMgr::stop()
 {
 #ifdef ENABLE_SOUND
     // first stop all sample groups
-    for ( auto current = d->_sample_groups.begin();
-               current != d->_sample_groups.end(); ++current ) {
-        SGSampleGroup *sgrp = current->second;
-        sgrp->stop();
+    for ( auto current : d->_sample_groups ) {
+        current.second->stop();
     }
 
     // clear all OpenAL sources
@@ -312,9 +329,8 @@ void SGSoundMgr::stop()
     d->_free_sources.clear();
 
     // clear any OpenAL buffers before shutting down
-    for ( auto current = d->_buffers.begin();
-               current != d->_buffers.begin(); ++current ) {
-        refUint ref = current->second;
+    for ( auto current : d->_buffers ) {
+        refUint ref = current.second;
         ALuint buffer = ref.id;
         alDeleteBuffers(1, &buffer);
         testForError("SGSoundMgr::stop: delete buffers");
@@ -327,8 +343,8 @@ void SGSoundMgr::stop()
         _active = false;
         alcDestroyContext(d->_context);
         alcCloseDevice(d->_device);
-        d->_context = NULL;
-        d->_device = NULL;
+        d->_context = nullptr;
+        d->_device = nullptr;
 
         _renderer = "unknown";
         _vendor = "unknown";
@@ -340,10 +356,8 @@ void SGSoundMgr::suspend()
 {
 #ifdef ENABLE_SOUND
     if (is_working()) {
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->suspend();
+        for ( auto current : d->_sample_groups ) {
+            current.second->suspend();
         }
         _active = false;
     }
@@ -354,10 +368,8 @@ void SGSoundMgr::resume()
 {
 #ifdef ENABLE_SOUND
     if (is_working()) {
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->resume();
+        for ( auto current : d->_sample_groups ) {
+            current.second->resume();
         }
         _active = true;
     }
@@ -375,10 +387,8 @@ void SGSoundMgr::update( double dt )
             d->update_pos_and_orientation();
         }
 
-        for ( auto current = d->_sample_groups.begin();
-                   current != d->_sample_groups.end(); ++current ) {
-            SGSampleGroup *sgrp = current->second;
-            sgrp->update(dt);
+        for ( auto current : d->_sample_groups ) {
+            current.second->update(dt);
         }
 
         if (_changed) {
@@ -396,19 +406,19 @@ if (isNaN(toVec3f(_velocity).data())) printf("NaN in listener velocity\n");
             if ( _velocity[0] || _velocity[1] || _velocity[2] ) {
                 velocity = hlOr.backTransform(_velocity*SG_FEET_TO_METER);
 
-                if ( _bad_doppler ) {
-                    double fact = 100.0;
-                    double mag = length( velocity );
+                if ( d->_bad_doppler ) {
+                    float fact = 100.0f;
+                    float mag = length( velocity );
 
-                    if (mag > SPEED_OF_SOUND) {
-                        fact *= SPEED_OF_SOUND / mag;
+                    if (mag > _sound_velocity) {
+                        fact *= _sound_velocity / mag;
                     }
                     alDopplerFactor(fact);
                 }
             }
 
             alListenerfv( AL_VELOCITY, toVec3f(velocity).data() );
-            // alDopplerVelocity(SPEED_OF_SOUND);
+//          alDopplerVelocity(_sound_velocity);
             testForError("update");
             _changed = false;
         }
@@ -424,6 +434,7 @@ bool SGSoundMgr::add( SGSampleGroup *sgrp, const std::string& refname )
     auto sample_grp_it = d->_sample_groups.find( refname );
     if ( sample_grp_it != d->_sample_groups.end() ) {
         // sample group already exists
+        SG_LOG(SG_SOUND, SG_DEV_ALERT, "Duplicate add of sample-group:" << refname);
         return false;
     }
 
@@ -457,7 +468,7 @@ bool SGSoundMgr::exists( const std::string &refname ) {
 
 
 // return a pointer to the SGSampleGroup if the specified sound exists
-// in the sound manager system, otherwise return NULL
+// in the sound manager system, otherwise return nullptr
 SGSampleGroup *SGSoundMgr::find( const std::string &refname, bool create ) {
     auto sample_grp_it = d->_sample_groups.find( refname );
     if ( sample_grp_it == d->_sample_groups.end() ) {
@@ -468,7 +479,7 @@ SGSampleGroup *SGSoundMgr::find( const std::string &refname, bool create ) {
             return sgrp;
         }
         else 
-            return NULL;
+            return nullptr;
     }
 
     return sample_grp_it->second;
@@ -534,7 +545,7 @@ unsigned int SGSoundMgr::request_buffer(SGSoundSample *sample)
     if ( !sample->is_valid_buffer() ) {
         // sample was not yet loaded or removed again
         std::string sample_name = sample->get_sample_name();
-        void *sample_data = NULL;
+        void* sample_data = nullptr;
 
         // see if the sample name is already cached
         auto buffer_it = d->_buffers.find( sample_name );
@@ -592,6 +603,7 @@ unsigned int SGSoundMgr::request_buffer(SGSoundSample *sample)
         case SG_SAMPLE_STEREO8:
             SG_LOG(SG_SOUND, SG_POPUP, "Stereo sound detected:\n" << sample->get_sample_name() << "\nUse two separate mono files instead if required.");
             format = AL_FORMAT_STEREO8;
+            break;
         default:
             SG_LOG(SG_SOUND, SG_ALERT, "unsupported audio format");
             return buffer;
@@ -787,13 +799,17 @@ void SGSoundMgr::update_sample_config( SGSoundSample *sample, SGVec3d& position,
         alSourcef( source, AL_MAX_DISTANCE, sample->get_max_dist() );
         alSourcef( source, AL_REFERENCE_DISTANCE,
                            sample->get_reference_dist() );
+        if (d->_source_radius) {
+            alSourcef( source, AL_SOURCE_RADIUS,
+                               0.5f*sample->get_reference_dist() );
+        }
         testForError("distance rolloff");
     }
 }
 
 
 bool SGSoundMgr::load( const std::string &samplepath,
-                       void **dbuf,
+                       void** dbuf,
                        int *fmt,
                        size_t *sz,
                        int *frq,
@@ -806,19 +822,18 @@ bool SGSoundMgr::load( const std::string &samplepath,
     unsigned int blocksz;
     ALsizei size;
     ALsizei freq;
-    ALvoid *data;
-
     ALfloat freqf;
 
-    data = simgear::loadWAVFromFile(samplepath, format, size, freqf, blocksz);
+    auto data = simgear::loadWAVFromFile(samplepath, format, size, freqf, blocksz);
     freq = (ALsizei)freqf;
-    if (data == NULL) {
-        throw sg_io_exception("Failed to load wav file", sg_location(samplepath));
+    if (!data) {
+        return false;
     }
 
     if (format == AL_FORMAT_STEREO8 || format == AL_FORMAT_STEREO16) {
-        free(data);
-        throw sg_io_exception("Warning: STEREO files are not supported for 3D audio effects: " + samplepath);
+         free(data);
+        SG_LOG(SG_IO, SG_DEV_ALERT, "Warning: STEREO files are not supported for 3D audio effects: " << samplepath);
+        return false;
     }
 
     *dbuf = (void *)data;
@@ -836,10 +851,11 @@ vector<std::string> SGSoundMgr::get_available_devices()
 #ifdef ENABLE_SOUND
     const ALCchar *s;
 
-    if (alcIsExtensionPresent(NULL, "ALC_enumerate_all_EXT") == AL_TRUE) {
-        s = alcGetString(NULL, ALC_ALL_DEVICES_SPECIFIER);
+    if (alcIsExtensionPresent(nullptr, "ALC_enumerate_all_EXT") == AL_TRUE) {
+        // REVIEW: Memory Leak - 4,136 bytes in 1 blocks are still reachable
+        s = alcGetString(nullptr, ALC_ALL_DEVICES_SPECIFIER);
     } else {
-        s = alcGetString(NULL, ALC_DEVICE_SPECIFIER);
+        s = alcGetString(nullptr, ALC_DEVICE_SPECIFIER);
     }
 
     if (s) {
@@ -860,7 +876,7 @@ vector<std::string> SGSoundMgr::get_available_devices()
 
 bool SGSoundMgr::testForError(void *p, std::string s)
 {
-   if (p == NULL) {
+   if (p == nullptr) {
       SG_LOG( SG_SOUND, SG_ALERT, "Error: " << s);
       return true;
    }
@@ -898,7 +914,7 @@ bool SGSoundMgr::testForALCError(std::string s)
 
 bool SGSoundMgr::is_working() const 
 {
-    return (d->_device != NULL);
+    return (d->_device != nullptr);
 }
 
 const SGQuatd& SGSoundMgr::get_orientation() const
