@@ -21,9 +21,7 @@
 //
 // $Id$
 
-#ifdef HAVE_CONFIG_H
-#  include <simgear_config.h>
-#endif
+#include <simgear_config.h>
 
 #include "xmlsound.hxx"
 
@@ -33,11 +31,13 @@
 #include <string.h>
 #include <stdio.h>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/debug/logstream.hxx>
-#include <simgear/props/props.hxx>
-#include <simgear/props/condition.hxx>
-#include <simgear/structure/exception.hxx>
+#include <simgear/math/sg_random.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/props/condition.hxx>
+#include <simgear/props/props.hxx>
+#include <simgear/structure/exception.hxx>
 
 #include "sample_group.hxx"
 #include "sample.hxx"
@@ -53,20 +53,8 @@ static double _snd_log(double v)   { return log(fabs(v)+1e-9); }
 // static double _snd_sqr(double v)   { return v*v; }
 // static double _snd_pow3(double v)  { return v*v*v; }
 
-static const struct {
-	const char *name;
-	double (*fn)(double);
-} __sound_fn[] = {
-	{"inv", _snd_inv},
-	{"abs", _snd_abs},
-	{"sqrt", _snd_sqrt},
-	{"log", _snd_log10},
-	{"ln", _snd_log},
-	{"", NULL}
-};
-
 SGXmlSound::SGXmlSound()
-  : _sample(NULL),
+  : _sample(nullptr),
     _active(false),
     _name(""),
     _mode(SGXmlSound::ONCE),
@@ -76,6 +64,11 @@ SGXmlSound::SGXmlSound()
     _delay(0.0),
     _stopping(0.0)
 {
+    _sound_fn["inv"] = _snd_inv;
+    _sound_fn["abs"] = _snd_abs;
+    _sound_fn["sqrt"] = _snd_sqrt;
+    _sound_fn["log"] = _snd_log10;
+    _sound_fn["ln"] = _snd_log;
 }
 
 SGXmlSound::~SGXmlSound()
@@ -83,14 +76,16 @@ SGXmlSound::~SGXmlSound()
     if (_sample)
         _sample->stop();
 
-    if (_sgrp && (_name != ""))
-        _sgrp->remove(_name);
+    auto group = _sgrp.lock();
+    if (group && !_name.empty()) {
+        group->remove(_name);
+    }
 
     _volume.clear();
     _pitch.clear();
 }
 
-void
+bool
 SGXmlSound::init( SGPropertyNode *root,
                   SGPropertyNode *node,
                   SGSampleGroup *sgrp,
@@ -126,12 +121,13 @@ SGXmlSound::init( SGPropertyNode *root,
       _property = root->getNode(propval, true);
 
    SGPropertyNode *condition = node->getChild("condition");
-   if (condition != NULL)
+   if (condition != nullptr)
       _condition = sgReadCondition(root, condition);
 
-   if (!_property && !_condition)
-      SG_LOG(SG_SOUND, SG_DEV_WARN,
-             "  Neither a condition nor a property specified");
+   if (!_property && !_condition) {
+       simgear::reportFailure(simgear::LoadFailure::Misconfigured, simgear::ErrorCode::AudioFX,
+                              "SGXmlSound: node:" + _name, path);
+   }
 
    _delay = node->getDoubleValue("delay-sec", 0.0);
 
@@ -142,10 +138,10 @@ SGXmlSound::init( SGPropertyNode *root,
    float v = 0.0;
    std::vector<SGPropertyNode_ptr> kids = node->getChildren("volume");
    for (i = 0; (i < kids.size()) && (i < SGXmlSound::MAXPROP); i++) {
-      _snd_prop volume = {NULL, NULL, NULL, NULL, 1.0, 0.0, 0.0, 0.0, false};
+      _snd_prop volume = {nullptr, nullptr, nullptr, nullptr, 1.0, 0.0, 0.0, 0.0, false};
 
       SGPropertyNode *n = kids[i]->getChild("expression");
-      if (n != NULL) {
+      if (n != nullptr) {
          volume.expr = SGReadDoubleExpression(root, n->getChild(0));
       }
 
@@ -155,9 +151,9 @@ SGXmlSound::init( SGPropertyNode *root,
 
       string intern_str = kids[i]->getStringValue("internal", "");
       if (intern_str == "dt_play")
-         volume.intern = &_dt_play;
+         volume.intern = std::make_shared<double>(_dt_play);
       else if (intern_str == "dt_stop")
-         volume.intern = &_dt_stop;
+         volume.intern = std::make_shared<double>(_dt_stop);
 
       if ((volume.factor = kids[i]->getDoubleValue("factor", 1.0)) != 0.0)
          if (volume.factor < 0.0) {
@@ -166,28 +162,29 @@ SGXmlSound::init( SGPropertyNode *root,
          }
 
       string type_str = kids[i]->getStringValue("type", "");
-      if ( type_str != "" ) {
+      if ( type_str != "" && type_str != "lin" ) {
 
-         for (int j=0; __sound_fn[j].fn; j++)
-           if ( type_str == __sound_fn[j].name ) {
-               volume.fn = __sound_fn[j].fn;
-               break;
-            }
+         auto it = _sound_fn.find(type_str);
+         if (it != _sound_fn.end()) {
+            volume.fn = it->second;
+         }
 
          if (!volume.fn)
-            SG_LOG(SG_SOUND,SG_INFO,
-                   "  Unknown volume type, default to 'lin'");
+            SG_LOG(SG_SOUND, SG_DEV_WARN,
+                   "  Unknown volume type, default to 'lin'"
+                   " in section: " << _name);
       }
 
       volume.offset = kids[i]->getDoubleValue("offset", 0.0);
 
       if ((volume.min = kids[i]->getDoubleValue("min", 0.0)) < 0.0)
-         SG_LOG( SG_SOUND, SG_WARN,
-          "Volume minimum value below 0. Forced to 0.");
+         SG_LOG(SG_SOUND, SG_DEV_WARN,
+          "Volume minimum value below 0. Forced to 0"
+          " in section: " << _name);
 
       volume.max = kids[i]->getDoubleValue("max", 0.0);
       if (volume.max && (volume.max < volume.min) )
-         SG_LOG(SG_SOUND,SG_ALERT,
+         SG_LOG(SG_SOUND, SG_DEV_ALERT,
                 "  Volume maximum below minimum. Neglected.");
 
       _volume.push_back(volume);
@@ -205,10 +202,13 @@ SGXmlSound::init( SGPropertyNode *root,
    float p = 0.0;
    kids = node->getChildren("pitch");
    for (i = 0; (i < kids.size()) && (i < SGXmlSound::MAXPROP); i++) {
-      _snd_prop pitch = {NULL, NULL, NULL, NULL, 1.0, 1.0, 0.0, 0.0, false};
+      _snd_prop pitch = {nullptr, nullptr, nullptr, nullptr, 1.0, 1.0, 0.0, 0.0, false};
+
+      double randomness = kids[i]->getDoubleValue("random", 0.0);
+      randomness *= sg_random();
 
       SGPropertyNode *n = kids[i]->getChild("expression");
-      if (n != NULL) {
+      if (n != nullptr) {
          pitch.expr = SGReadDoubleExpression(root, n->getChild(0));
       }
 
@@ -218,9 +218,9 @@ SGXmlSound::init( SGPropertyNode *root,
 
       string intern_str = kids[i]->getStringValue("internal", "");
       if (intern_str == "dt_play")
-         pitch.intern = &_dt_play;
+         pitch.intern = std::make_shared<double>(_dt_play);
       else if (intern_str == "dt_stop")
-         pitch.intern = &_dt_stop;
+         pitch.intern = std::make_shared<double>(_dt_stop);
 
       if ((pitch.factor = kids[i]->getDoubleValue("factor", 1.0)) != 0.0)
          if (pitch.factor < 0.0) {
@@ -229,29 +229,32 @@ SGXmlSound::init( SGPropertyNode *root,
          }
 
       string type_str = kids[i]->getStringValue("type", "");
-      if ( type_str != "" ) {
+      if ( type_str != "" && type_str != "lin" ) {
 
-         for (int j=0; __sound_fn[j].fn; j++)
-            if ( type_str == __sound_fn[j].name ) {
-               pitch.fn = __sound_fn[j].fn;
-               break;
-            }
+         auto it = _sound_fn.find(type_str);
+         if (it != _sound_fn.end()) {
+            pitch.fn = it->second;
+         }
 
          if (!pitch.fn)
-            SG_LOG(SG_SOUND,SG_INFO,
-                   "  Unknown pitch type, default to 'lin'");
+            SG_LOG(SG_SOUND, SG_DEV_WARN,
+                   "  Unknown pitch type, default to 'lin'"
+                   " in section: " << _name);
       }
 
       pitch.offset = kids[i]->getDoubleValue("offset", 1.0);
+      pitch.offset += randomness;
 
       if ((pitch.min = kids[i]->getDoubleValue("min", 0.0)) < 0.0)
-         SG_LOG(SG_SOUND,SG_WARN,
-                "  Pitch minimum value below 0. Forced to 0.");
+         SG_LOG(SG_SOUND, SG_DEV_WARN,
+                "  Pitch minimum value below 0. Forced to 0"
+                " in section: " << _name);
 
       pitch.max = kids[i]->getDoubleValue("max", 0.0);
       if (pitch.max && (pitch.max < pitch.min) )
-         SG_LOG(SG_SOUND,SG_ALERT,
-                "  Pitch maximum below minimum. Neglected");
+         SG_LOG(SG_SOUND, SG_DEV_ALERT,
+                "  Pitch maximum below minimum. Neglected"
+                " in section: " << _name);
 
       _pitch.push_back(pitch);
       p += pitch.offset;
@@ -263,7 +266,7 @@ SGXmlSound::init( SGPropertyNode *root,
    SGVec3f offset_pos = SGVec3f::zeros();
    SGPropertyNode_ptr prop = node->getChild("position");
    SGPropertyNode_ptr pos_prop[3];
-   if ( prop != NULL ) {
+   if ( prop != nullptr ) {
        offset_pos[0] = -prop->getDoubleValue("x", 0.0);
        offset_pos[1] = -prop->getDoubleValue("y", 0.0);
        offset_pos[2] = -prop->getDoubleValue("z", 0.0);
@@ -293,7 +296,7 @@ SGXmlSound::init( SGPropertyNode *root,
    float outer = 360.0;
    float outer_gain = 0.0;
    prop = node->getChild("orientation");
-   if ( prop != NULL ) {
+   if ( prop != nullptr ) {
       dir = SGVec3f(-prop->getFloatValue("x", 0.0),
                     -prop->getFloatValue("y", 0.0),
                     -prop->getFloatValue("z", 0.0));
@@ -310,11 +313,13 @@ SGXmlSound::init( SGPropertyNode *root,
    } else {
       _sgrp = sgrp;
    }
+
    string soundFileStr = node->getStringValue("path", "");
    _sample = new SGSoundSample(soundFileStr.c_str(), path);
    if (!_sample->file_path().exists()) {
-      throw sg_io_exception("XML sound: couldn't find file: '" + soundFileStr + "'");
-      return;
+       simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::AudioFX,
+                              "SGXmlSound: node:" + _name + "; can't find:" + soundFileStr, path);
+       return false;
    }
 
    _sample->set_relative_position( offset_pos );
@@ -325,34 +330,37 @@ SGXmlSound::init( SGPropertyNode *root,
    _sample->set_max_dist( max_dist );
    _sample->set_volume( v );
    _sample->set_pitch( p );
-   _sgrp->add( _sample, _name );
+
+   auto group = _sgrp.lock();
+   group->add(_sample, _name);
+
+   return true;
 }
 
 void
 SGXmlSound::update (double dt)
 {
-   double curr_value = 0.0;
-
    //
-   // If the state changes to false, stop playing.
+   // Sound trigger condition:
+   // - if a condition is defined, test it.
+   // - for mode IN_TRANSIT, check that the property changed
+   // - otherwise just check the property
    //
-   if (_property)
-       curr_value = _property->getDoubleValue();
+   bool condition = false;
+   if (_condition)
+       condition = _condition->test();
+   else if (_property)
+   {
+       if (_mode == SGXmlSound::IN_TRANSIT) {
+           double curr_value = _property->getDoubleValue();
+           condition = (curr_value != _prev_value);
+           _prev_value = curr_value;
+       } else {
+           condition = _property->getBoolValue();
+       }
+   }
 
-   // If a condition is defined, test whether it is FALSE,
-   // else
-   //   if a property is defined then test if it's value is FALSE
-   //      or if the mode is IN_TRANSIT then
-   //            test whether the current value matches the previous value.
-   if ( 							// Lisp, anyone?
-       (_condition && !_condition->test()) ||
-       (!_condition && _property &&
-        (
-         !curr_value ||
-         ( (_mode == SGXmlSound::IN_TRANSIT) && (curr_value == _prev_value) )
-         )
-        )
-       )
+   if (!condition)
    {
        if ((_mode != SGXmlSound::IN_TRANSIT) || (_stopping > MAX_TRANSIT_TIME))
        {
@@ -373,6 +381,22 @@ SGXmlSound::update (double dt)
        return;
    }
 
+   if (_sample->test_out_of_range())
+   {
+       // When out of range, sample is not played, but logic
+       // (playing condition, _dt_*) is updated as if we were playing it.
+       //
+       // Exception: for mode ONCE, sample immediately finishes when going out of range
+       // (even if the sample is long, you can not go out of range, then back in range to hear the end of it).
+
+       if (_sample->is_playing()) {
+           SG_LOG(SG_SOUND, SG_DEBUG, "Stopping audio after " << _dt_play
+                  << " sec: " << _name << " (out of range)");
+
+           _sample->stop();
+       }
+   }
+
    //
    // mode is ONCE and the sound is still playing?
    //
@@ -392,7 +416,6 @@ SGXmlSound::update (double dt)
       // clear the delay timer.
       //
       _dt_play += dt;
-      _prev_value = curr_value;
       _stopping = 0.0;
    }
 
@@ -400,21 +423,67 @@ SGXmlSound::update (double dt)
       return;
 
    //
-   // Update the volume
+   // Do we need to start playing the sample?
    //
-   int i;
+   if (!_active) {
+
+      if (!_sample->test_out_of_range()) {
+          if (_mode == SGXmlSound::ONCE)
+             _sample->play_once();
+          else
+             _sample->play_looped();
+
+          SG_LOG(SG_SOUND, SG_DEBUG, "Playing audio after " << _dt_stop
+                                       << " sec: " << _name);
+          SG_LOG(SG_SOUND, SG_DEBUG, "Playing " << ((_mode == ONCE) ? "once" : "looped"));
+      }
+
+      _active = true;
+      _dt_stop = 0.0;
+   }
+
+   // Remark: at this point of the function, _active is always true
+   // The sample might not be playing if
+   // - it was played once and stopped, or
+   // - it went out of range
+
+   //
+   // If a looped sample stopped playing but is still active,
+   // it can only be because it went out of range. If needed, restart it.
+   //
+   if (!_sample->is_playing() && _mode != SGXmlSound::ONCE && !_sample->test_out_of_range())
+   {
+       _sample->play_looped();
+
+       SG_LOG(SG_SOUND, SG_DEBUG, "Restarting sample (was out of range): " << _name);
+   }
+
+   //
+   // Change sample state
+   //
+   if (_sample->is_playing())
+   {
+       _sample->set_volume(volume());
+       _sample->set_pitch(pitch());
+   }
+}
+
+double SGXmlSound::volume()
+{
    int max = _volume.size();
    double volume = 1.0;
    double volume_offset = 0.0;
    bool expr = false;
 
-   for(i = 0; i < max; i++) {
+   for(int i = 0; i < max; i++) {
       double v = 1.0;
 
       if (_volume[i].expr) {
-         v = _volume[i].expr->getValue(NULL);
-         expr = true;
-         continue;
+          double expression_value = _volume[i].expr->getValue(nullptr);
+          if (expression_value >= 0)
+              volume *= expression_value;
+		  expr = true;
+		  continue;
       }
 
       if (_volume[i].prop) {
@@ -435,34 +504,47 @@ SGXmlSound::update (double dt)
 
       if (_volume[i].max && (v > _volume[i].max))
          v = _volume[i].max;
-
       else if (v < _volume[i].min)
          v = _volume[i].min;
 
-      if (_volume[i].subtract)				// Hack!
-         volume = _volume[i].offset - v;
 
+      if (_volume[i].subtract){				// Hack!
+          v = v + _volume[i].offset;
+          if (v >= 0)
+            volume = volume * v;
+      }
       else {
-         volume_offset += _volume[i].offset;
-         volume *= v;
+          if (v >= 0) {
+              volume_offset += _volume[i].offset;
+              volume = volume * v;
+          }
       }
    }
 
-   //
-   // Update the pitch
-   //
-   max = _pitch.size();
+   double vol = volume_offset + volume;
+   if (vol > 1.0) {
+      SG_LOG(SG_SOUND, SG_DEBUG, "Sound volume too large for '"
+              << _name << "':  " << vol << "  ->  clipping to 1.0");
+      vol = 1.0;
+   }
+
+   return vol;
+}
+
+double SGXmlSound::pitch()
+{
+   int max = _pitch.size();
    double pitch = 1.0;
    double pitch_offset = 0.0;
+   bool expr = false;
 
-   expr = false;
-   for(i = 0; i < max; i++) {
+   for(int i = 0; i < max; i++) {
       double p = 1.0;
 
       if (_pitch[i].expr) {
-         p = _pitch[i].expr->getValue(NULL);
-         expr = true;
-         continue;
+          pitch *= _pitch[i].expr->getValue(nullptr);
+          expr = true;
+          continue;
       }
 
       if (_pitch[i].prop) {
@@ -496,37 +578,5 @@ SGXmlSound::update (double dt)
       }
    }
 
-   //
-   // Change sample state
-   //
-
-   double vol = volume_offset + volume;
-   if (vol > 1.0) {
-      SG_LOG(SG_SOUND, SG_DEBUG, "Sound volume too large for '"
-              << _name << "':  " << vol << "  ->  clipping to 1.0");
-      vol = 1.0;
-   }
-   _sample->set_volume(vol);
-   _sample->set_pitch( pitch_offset + pitch );
-
-
-   //
-   // Do we need to start playing the sample?
-   //
-   if (!_active) {
-
-      if (_mode == SGXmlSound::ONCE)
-         _sample->play(false);
-
-      else
-         _sample->play(true);
-
-      SG_LOG(SG_SOUND, SG_DEBUG, "Playing audio after " << _dt_stop
-                                   << " sec: " << _name);
-      SG_LOG(SG_SOUND, SG_DEBUG,
-                         "Playing " << ((_mode == ONCE) ? "once" : "looped"));
-
-      _active = true;
-      _dt_stop = 0.0;
-   }
+   return pitch_offset + pitch;
 }
