@@ -1,6 +1,6 @@
 /* -*-c++-*-
  *
- * Copyright (C) 2006-2007 Mathias Froehlich 
+ * Copyright (C) 2006-2007 Mathias Froehlich
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,8 +31,8 @@
 #include <utility>
 #include <string>
 #include <sstream>
-#include <boost/bind.hpp>
 #include <cstring> // for strcmp
+#include <cassert>
 
 #include <simgear/props/props.hxx>
 
@@ -70,14 +70,10 @@ const expression::Value eval(const Expression* exp,
 
 template<typename T>
 static bool
-SGReadValueFromString(const char* str, T& value)
+SGReadValueFromString(const std::string& str, T& value)
 {
-    if (!str) {
-        SG_LOG(SG_IO, SG_ALERT, "Cannot read string content.");
-        return false;
-    }
     std::stringstream s;
-    s.str(std::string(str));
+    s.str(str);
     s >> value;
     if (s.fail()) {
         SG_LOG(SG_IO, SG_ALERT, "Cannot read string content.");
@@ -86,35 +82,6 @@ SGReadValueFromString(const char* str, T& value)
     return true;
 }
 
-template<>
-bool
-SGReadValueFromString(const char* str, bool& value)
-{
-    if (!str) {
-        SG_LOG(SG_IO, SG_ALERT, "Cannot read string content.");
-        return false;
-    }
-    std::stringstream s;
-    s.str(std::string(str));
-    s >> value;
-    if (!s.fail())
-        return true;
-
-    std::string stdstr;
-    if (!SGReadValueFromString(str, stdstr))
-        return false;
-
-    if (stdstr == "true" || stdstr == "True" || stdstr == "TRUE") {
-        value = true;
-        return true;
-    }
-    if (stdstr == "false" || stdstr == "False" || stdstr == "FALSE") {
-        value = true;
-        return true;
-    }
-    return false;
-}
-
 template<typename T>
 static bool
 SGReadValueFromContent(const SGPropertyNode *node, T& value)
@@ -170,7 +137,7 @@ SGReadExpression(SGPropertyNode *inputRoot, const SGPropertyNode *expression)
     if (!expression)
         return 0;
 
-    std::string name = expression->getName();
+    std::string name = expression->getNameString();
 
     if (name == "value") {
         T value;
@@ -187,10 +154,7 @@ SGReadExpression(SGPropertyNode *inputRoot, const SGPropertyNode *expression)
                    "No inputRoot argument given!");
             return 0;
         }
-        if (!expression->getStringValue()) {
-            SG_LOG(SG_IO, SG_ALERT, "Cannot read \"" << name << "\" expression.");
-            return 0;
-        }
+
         SGPropertyNode* inputNode;
         inputNode = inputRoot->getNode(expression->getStringValue(), true);
         return new SGPropertyExpression<T>(inputNode);
@@ -358,7 +322,7 @@ SGReadExpression(SGPropertyNode *inputRoot, const SGPropertyNode *expression)
         // find input expression - i.e a child not named 'entry'
         const SGPropertyNode* inputNode = NULL;
         for (int i=0; (i<expression->nChildren()) && !inputNode; ++i) {
-            if (strcmp(expression->getChild(i)->getName(), "entry") == 0) {
+            if (expression->getChild(i)->getNameString() == "entry") {
                 continue;
             }
             
@@ -369,7 +333,7 @@ SGReadExpression(SGPropertyNode *inputRoot, const SGPropertyNode *expression)
             SG_LOG(SG_IO, SG_ALERT, "Cannot read \"" << name << "\" expression: no input found");
             return 0;
         }
-        
+
         SGSharedPtr<SGExpression<T> > inputExpression;
         inputExpression = SGReadExpression<T>(inputRoot, inputNode);
         if (!inputExpression) {
@@ -638,36 +602,6 @@ SGReadExpression(SGPropertyNode *inputRoot, const SGPropertyNode *expression)
         }
         return new SGAtan2Expression<T>(inputExpressions[0], inputExpressions[1]);
     }
-    if (name == "div") {
-        if (expression->nChildren() != 2) {
-            SG_LOG(SG_IO, SG_ALERT, "Cannot read \"" << name << "\" expression.");
-            return 0;
-        }
-        SGSharedPtr<SGExpression<T> > inputExpressions[2] = {
-            SGReadExpression<T>(inputRoot, expression->getChild(0)),
-            SGReadExpression<T>(inputRoot, expression->getChild(1))
-        };
-        if (!inputExpressions[0] || !inputExpressions[1]) {
-            SG_LOG(SG_IO, SG_ALERT, "Cannot read \"" << name << "\" expression.");
-            return 0;
-        }
-        return new SGDivExpression<T>(inputExpressions[0], inputExpressions[1]);
-    }
-    if (name == "mod") {
-        if (expression->nChildren() != 2) {
-            SG_LOG(SG_IO, SG_ALERT, "Cannot read \"" << name << "\" expression.");
-            return 0;
-        }
-        SGSharedPtr<SGExpression<T> > inputExpressions[2] = {
-            SGReadExpression<T>(inputRoot, expression->getChild(0)),
-            SGReadExpression<T>(inputRoot, expression->getChild(1))
-        };
-        if (!inputExpressions[0] || !inputExpressions[1]) {
-            SG_LOG(SG_IO, SG_ALERT, "Cannot read \"" << name << "\" expression.");
-            return 0;
-        }
-        return new SGModExpression<T>(inputExpressions[0], inputExpressions[1]);
-    }
     if (name == "pow") {
         if (expression->nChildren() != 2) {
             SG_LOG(SG_IO, SG_ALERT, "Cannot read \"" << name << "\" expression.");
@@ -849,12 +783,12 @@ ExpParserRegistrar leRegistrar("less-equal", predParser<LessEqualExpression>);
 template<typename Logicop>
 Expression* logicopParser(const SGPropertyNode* exp, Parser* parser)
 {
-    using namespace boost;
-    vector<Expression*> children;
+    std::vector<Expression*> children;
     parser->readChildren(exp, children);
-    vector<Expression*>::iterator notBool =
-        find_if(children.begin(), children.end(),
-                boost::bind(&Expression::getType, _1) != BOOL);
+    std::vector<Expression*>::const_iterator notBool =
+        std::find_if(children.begin(), children.end(),
+                     [] (const Expression* const e) {
+                         return e->getType () != BOOL; });
     if (notBool != children.end())
         throw("non boolean operand to logical expression");
     Logicop *expr = new Logicop;
@@ -868,9 +802,10 @@ ExpParserRegistrar orRegistrar("or", logicopParser<OrExpression>);
 size_t BindingLayout::addBinding(const string& name, Type type)
 {
     //XXX error checkint
-    vector<VariableBinding>::iterator itr
-        = find_if(bindings.begin(), bindings.end(),
-                  boost::bind(&VariableBinding::name, _1) == name);
+    std::vector<VariableBinding>::const_iterator itr =
+        std::find_if(bindings.begin(), bindings.end(),
+                     [&name] (const VariableBinding& v) {
+                         return v.name == name; });
     if (itr != bindings.end())
         return itr->location;
     size_t result = bindings.size();
@@ -881,11 +816,10 @@ size_t BindingLayout::addBinding(const string& name, Type type)
 bool BindingLayout::findBinding(const std::string& name,
                                 VariableBinding& result) const
 {
-    using namespace std;
-    using namespace boost;
-    vector<VariableBinding>::const_iterator itr
-        = find_if(bindings.begin(), bindings.end(),
-                  boost::bind(&VariableBinding::name, _1) == name);
+    std::vector<VariableBinding>::const_iterator itr =
+        std::find_if(bindings.begin(), bindings.end(),
+                     [&name] (const VariableBinding& v) {
+                         return v.name == name; });
     if (itr != bindings.end()) {
         result = *itr;
         return true;
