@@ -32,6 +32,7 @@
 #include <cstring>
 
 #include <string>
+#include <unordered_map>
 
 #ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>  // for get/setitimer, gettimeofday, struct timeval
@@ -62,6 +63,9 @@ static const double SIDRATE = 0.9972695677;
 // tzContainer stores all the current Timezone control points/
 std::unique_ptr<SGTimeZoneContainer> static_tzContainer;
 
+using StartTimeByYearHash = std::unordered_map<int, time_t>;
+static StartTimeByYearHash global_yearToGMTHash;
+
 void SGTime::init( const SGGeod& location, const SGPath& root, time_t init_time )
 {
     gst_diff = -9999.0;
@@ -72,28 +76,33 @@ void SGTime::init( const SGGeod& location, const SGPath& root, time_t init_time
 	cur_time = time(NULL); 
     }
 
+    char* gmt = asctime(gmtime(&cur_time));
+    char* local = asctime(localtime(&cur_time));
+    gmt[strlen(gmt) - 1] = '\0';
+    local[strlen(local) - 1] = '\0';
     SG_LOG( SG_EVENT, SG_DEBUG,
-                "Current greenwich mean time = " << asctime(gmtime(&cur_time)));
+                "Current greenwich mean time = " << gmt);
     SG_LOG( SG_EVENT, SG_DEBUG,
-             "Current local time          = " << asctime(localtime(&cur_time)));
+             "Current local time          = " << local);
 
     if ( !root.isNull()) {
         if (!static_tzContainer.get()) {
-            SGPath zone( root );
-            zone.append( "zone.tab" );
-            SG_LOG( SG_EVENT, SG_INFO, "Reading timezone info from: " << zone );
-            std::string zs = zone.local8BitStr();
-            static_tzContainer.reset(new SGTimeZoneContainer( zs.c_str() ));
+            const auto zonePath = root / "timezone16.bin";
+            SG_LOG( SG_EVENT, SG_INFO, "Reading timezone info from: " << zonePath );
+            static_tzContainer.reset(new SGTimeZoneContainer(zonePath));
         }
 
         SGTimeZone* nearestTz = static_tzContainer->getNearest(location);
-
-        SGPath name( root );
-        name.append( nearestTz->getDescription() );
-        zonename = name.utf8Str();
-        SG_LOG( SG_EVENT, SG_DEBUG, "Using zonename = " << zonename );
+        if (nearestTz) {
+            description = nearestTz->getDescription();
+            SGPath name = root / description;
+            zonename = name.utf8Str();
+            SG_LOG( SG_EVENT, SG_DEBUG, "Using zonename = " << zonename );
+        } else {
+            zonename.clear();
+        }
     } else {
-        zonename.erase();
+        zonename.clear();
     }
 }
 
@@ -113,11 +122,6 @@ SGTime::SGTime() {
 }
 
 
-SGTime::~SGTime()
-{
-}
-
-
 // given Julian Date and Longitude (decimal degrees West) compute
 // Local Sidereal Time, in decimal hours.
 //
@@ -143,12 +147,19 @@ static double sidereal_precise( double mjd, double lng )
 // return a courser but cheaper estimate of sidereal time
 static double sidereal_course( time_t cur_time, const struct tm *gmt, double lng )
 {
-    time_t start_gmt, now;
+    time_t now;
     double diff, part, days, hours, lstTmp;
   
     now = cur_time;
-    start_gmt = sgTimeGetGMT(gmt->tm_year, 2, 21, 12, 0, 0);
-  
+
+    // sgTimeGetGMT is expensive when called frequently, so cache the
+    // result.
+    time_t start_gmt = global_yearToGMTHash[gmt->tm_year];
+    if (start_gmt == 0) {
+        start_gmt = sgTimeGetGMT(gmt->tm_year, 2, 21, 12, 0, 0);
+        global_yearToGMTHash[gmt->tm_year] = start_gmt;
+    }
+
     diff = (now - start_gmt) / (3600.0 * 24.0);
   
     part = fmod(diff, 1.0);
@@ -218,8 +229,9 @@ void SGTime::update( const SGGeod& location, time_t ct, long int warp )
 
 
 // Given lon/lat, update timezone information and local_offset
-void SGTime::updateLocal( const SGGeod& aLocation, const string& root ) {
-  SGGeod location(aLocation);
+bool SGTime::updateLocal(const SGGeod& aLocation, const SGPath& root)
+{
+    SGGeod location(aLocation);
     if (!aLocation.isValid()) {
         location = SGGeod();
     }
@@ -227,9 +239,18 @@ void SGTime::updateLocal( const SGGeod& aLocation, const string& root ) {
     time_t currGMT;
     time_t aircraftLocalTime;
     SGTimeZone* nearestTz = static_tzContainer->getNearest(location);
-    SGPath zone( root );
-    zone.append ( nearestTz->getDescription() );
+    if (!nearestTz) {
+        // this situation is ocurring very occasionaly for with the
+        // Shuttle + Hubble, see this bug report:
+        // https://sourceforge.net/p/flightgear/codetickets/2780/
+        SG_LOG(SG_ENVIRONMENT, SG_ALERT, "SGTime::updateLocal: Timezone not found for location: " << location);
+        return false;
+    }
+
+    description = nearestTz->getDescription();
+    SGPath zone = root / description;
     zonename = zone.utf8Str();
+    SG_LOG( SG_EVENT, SG_DEBUG, "Using zonename = " << zonename );
 
     //Avoid troubles when zone.tab hasn't got the right line endings
     if (zonename[zonename.size()-1] == '\r')
@@ -239,11 +260,19 @@ void SGTime::updateLocal( const SGGeod& aLocation, const string& root ) {
     }
 
     currGMT = sgTimeGetGMT( gmtime(&cur_time) );
-    std::string zs = zone.local8BitStr();
-    aircraftLocalTime = sgTimeGetGMT( (fgLocaltime(&cur_time, zs.c_str())) );
-    local_offset = aircraftLocalTime - currGMT;
+    std::string zs = zone.utf8Str();
+    struct tm *local_time = fgLocaltime(&cur_time, zs.c_str());
+    if (local_time) {
+      aircraftLocalTime = sgTimeGetGMT( local_time );
+      local_offset = aircraftLocalTime - currGMT;
     // cout << "Using " << local_offset << " as local time offset Timezone is " 
     //      << zonename << endl;
+    } else {
+      SG_LOG(SG_EVENT, SG_ALERT, "Timezone file not found: " << zs);
+      return false;
+    }
+
+    return true;
 }
 
 
@@ -375,10 +404,11 @@ time_t sgTimeGetGMT(int year, int month, int day, int hour, int min, int sec)
 }
 
 // format time
-char* sgTimeFormatTime( const struct tm* p, char* buf )
+std::string sgTimeFormatTime( const struct tm* p )
 {
-    sprintf( buf, "%d/%d/%2d %d:%02d:%02d", 
-	     p->tm_mon, p->tm_mday, p->tm_year,
-	     p->tm_hour, p->tm_min, p->tm_sec);
-    return buf;
+    std::ostringstream buf;
+
+    buf << 1900+p->tm_year << '-' << p->tm_mon+1 << '-' << p->tm_mday << 'T';
+    buf << p->tm_hour << ':' << p->tm_min << ':' << p->tm_sec;
+    return buf.str();
 }
